@@ -11,14 +11,16 @@ interface AbuseFilterProtectedVarsAccessLoggerHook {
 	 * the ability to override the logger by aborting additional logging.
 	 *
 	 * This is useful if an extension wants to divert a log to their own logger
-	 * (eg. CheckUser wants to centralize its IP access logs).
+	 * (eg. CheckUser wants to centralize its IP access logs). It's recommended
+	 * that in such cases the extension removes its variables from $params['variables']
+	 * and leaves the rest of them to be stored in their own log.
 	 *
 	 * @param UserIdentity $performer
 	 * @param string $target
 	 * @param string $action
 	 * @param bool $shouldDebounce
 	 * @param int $timestamp
-	 * @param array $params
+	 * @param array &$params
 	 * @return bool|void True or no return value to continue or false to abort
 	 */
 	public function onAbuseFilterLogProtectedVariableValueAccess(
@@ -27,6 +29,6 @@ interface AbuseFilterProtectedVarsAccessLoggerHook {
 		string $action,
 		bool $shouldDebounce,
 		int $timestamp,
-		array $params
+		array &$params
 	);
 }
