@@ -10,7 +10,7 @@
 	'use strict';
 
 	// @var {jQuery} Filter editor for JS and jQuery handling
-	var $filterBox,
+	let $filterBox,
 		// Filter editor for Ace specific functions
 		filterEditor,
 		// @var {jQuery} Hidden textarea for submitting form
@@ -28,9 +28,9 @@
 	 * @return {string} current warning message
 	 */
 	function getCurrentMessage( action ) {
-		var existing = action === 'warn' ? warnMessageExisting : disallowMessageExisting,
-			other = action === 'warn' ? warnMessageOther : disallowMessageOther,
-			message = existing.getValue();
+		const existing = action === 'warn' ? warnMessageExisting : disallowMessageExisting,
+			other = action === 'warn' ? warnMessageOther : disallowMessageOther;
+		let message = existing.getValue();
 
 		if ( message === 'other' ) {
 			message = other.getValue();
@@ -61,15 +61,11 @@
 	 * Switch between Ace Editor and classic textarea
 	 */
 	function switchEditor() {
+		useAce = !useAce;
+		$filterBox.toggle( useAce );
+		$plainTextBox.toggle( !useAce );
 		if ( useAce ) {
-			useAce = false;
-			$filterBox.hide();
-			$plainTextBox.show();
-		} else {
-			useAce = true;
 			filterEditor.session.setValue( $plainTextBox.val() );
-			$filterBox.show();
-			$plainTextBox.hide();
 		}
 	}
 
@@ -79,8 +75,7 @@
 	 * @param {Object} response Data returned from the AJAX request
 	 */
 	function processSyntaxResult( response ) {
-		var position,
-			data = response.abusefilterchecksyntax;
+		const data = response.abusefilterchecksyntax;
 
 		if ( data.status === 'ok' ) {
 			// Successful
@@ -100,7 +95,8 @@
 			if ( useAce ) {
 				filterEditor.focus();
 				// Convert index (used in textareas) in position {row, column} for ace
-				position = filterEditor.session.getDocument().indexToPosition( data.character );
+				const position = filterEditor.session.getDocument()
+					.indexToPosition( data.character );
 				filterEditor.navigateTo( position.row, position.column );
 				filterEditor.scrollToRow( position.row );
 			} else {
@@ -135,7 +131,7 @@
 	 * @param {jQuery.Event} e The event fired when the function is called
 	 */
 	function doSyntaxCheck() {
-		var filter = $plainTextBox.val();
+		const filter = $plainTextBox.val();
 
 		$( this )
 			.prop( 'disabled', true )
@@ -154,7 +150,7 @@
 	 * Fired by a change event from the #wpFilterBuilder dropdown
 	 */
 	function addText() {
-		var $filterBuilder = $( '#wpFilterBuilder' );
+		const $filterBuilder = $( '#wpFilterBuilder' );
 
 		if ( $filterBuilder.prop( 'selectedIndex' ) === 0 ) {
 			return;
@@ -179,7 +175,7 @@
 	 * @param {jQuery.Event} e The event fired when the function is called
 	 */
 	function fetchFilter() {
-		var filterId = $( '#mw-abusefilter-load-filter input' ).val().trim();
+		const filterId = $( '#mw-abusefilter-load-filter input' ).val().trim();
 
 		if ( filterId === '' ) {
 			return;
@@ -196,10 +192,10 @@
 			abfendid: filterId,
 			abflimit: 1
 		} )
-			.always( function removeSpinner() {
+			.always( () => {
 				$.removeSpinner( 'fetch-spinner' );
 			} )
-			.done( function insertFilter( data ) {
+			.done( ( data ) => {
 				if ( data.query.abusefilters[ 0 ] !== undefined ) {
 					if ( useAce ) {
 						filterEditor.setValue( data.query.abusefilters[ 0 ].pattern );
@@ -214,18 +210,10 @@
 	 * that don't have checked boxes
 	 */
 	function hideDeselectedActions() {
-		$( '.mw-abusefilter-action-checkbox input' ).each( function showHideParams() {
+		$( '.mw-abusefilter-action-checkbox input' ).each( ( i, element ) => {
 			// mw-abusefilter-action-checkbox-{$action}
-			var action = this.parentNode.id.slice( 31 ),
-				$params = $( '#mw-abusefilter-' + action + '-parameters' );
-
-			if ( $params.length ) {
-				if ( this.checked ) {
-					$params.show();
-				} else {
-					$params.hide();
-				}
-			}
+			const action = element.parentNode.id.slice( 31 );
+			$( '#mw-abusefilter-' + action + '-parameters' ).toggle( element.checked );
 		} );
 	}
 
@@ -235,35 +223,37 @@
 	 * @param {string} action The action the message refers to
 	 */
 	function previewMessage( action ) {
-		var api,
-			args = [
-				'<nowiki>' + $( 'input[name=wpFilterDescription]' ).val() + '</nowiki>',
-				$( '#mw-abusefilter-edit-id' ).children().last().text()
-			],
-			message = getCurrentMessage( action ),
-			// mw-abusefilter-warn-preview, mw-abusefilter-disallow-preview
-			$element = $( '#mw-abusefilter-' + action + '-preview' ),
+		// The following messages are generated here:
+		// * mw-abusefilter-warn-preview
+		// * mw-abusefilter-disallow-preview
+		const $element = $( '#mw-abusefilter-' + action + '-preview' ),
 			previewButton = action === 'warn' ? toggleWarnPreviewButton : toggleDisallowPreviewButton;
 
 		if ( $element.css( 'display' ) !== 'none' ) {
 			$element.hide();
 			previewButton.setFlags( { destructive: false, progressive: true } );
 		} else {
-			api = new mw.Api();
+			const api = new mw.Api();
+			const args = [
+				'<nowiki>' + $( 'input[name=wpFilterDescription]' ).val() + '</nowiki>',
+				$( '#mw-abusefilter-edit-id' ).children().last().text()
+			];
+			const message = getCurrentMessage( action );
+
 			api.get( {
 				action: 'query',
 				meta: 'allmessages',
 				ammessages: message,
 				amargs: args.join( '|' )
 			} )
-				.done( function parseMessage( data ) {
+				.done( ( data ) => {
 					api.parse( data.query.allmessages[ 0 ][ '*' ], {
 						disablelimitreport: '',
 						preview: '',
 						prop: 'text',
 						title: 'MediaWiki:' + message
 					} )
-						.done( function showMessage( html ) {
+						.done( ( html ) => {
 							$element.show().html( html );
 							previewButton.setFlags(
 								{ destructive: true, progressive: false }
@@ -279,7 +269,7 @@
 	 * @param {string} action The action for which the message is used
 	 */
 	function editMessage( action ) {
-		var message = getCurrentMessage( action ),
+		const message = getCurrentMessage( action ),
 			defaultMsg = action === 'warn' ? 'warning' : 'disallowed',
 			url = mw.util.getUrl( 'MediaWiki:' + message, {
 				action: 'edit',
@@ -293,17 +283,17 @@
 	 * Called if the filter group (#mw-abusefilter-edit-group-input select) is changed. Allows
 	 * using different default messages for 'warn' and 'disallow', based on the group.
 	 *
-	 * @this HTMLELement
+	 * @this HTMLElement
 	 * @param {jQuery.Event} e The event fired when the function is called
 	 */
 	function onFilterGroupChange() {
-		var $afWarnMessageExisting, $afDisallowMessageExisting, newVal;
+		let newVal;
 
 		if (
 			!$( '#mw-abusefilter-action-checkbox-warn input' ).is( ':checked' ) &&
 			$( this ).val() in mw.config.get( 'wgAbuseFilterDefaultWarningMessage' )
 		) {
-			$afWarnMessageExisting = $( '#mw-abusefilter-warn-message-existing select' );
+			const $afWarnMessageExisting = $( '#mw-abusefilter-warn-message-existing select' );
 			newVal = mw.config.get( 'wgAbuseFilterDefaultWarningMessage' )[ $( this ).val() ];
 
 			if ( $afWarnMessageExisting.find( 'option[value=\'' + newVal + '\']' ).length ) {
@@ -319,7 +309,7 @@
 			!$( '#mw-abusefilter-action-checkbox-disallow input' ).is( ':checked' ) &&
 			$( this ).val() in mw.config.get( 'wgAbuseFilterDefaultDisallowMessage' )
 		) {
-			$afDisallowMessageExisting = $( '#mw-abusefilter-disallow-message-existing select' );
+			const $afDisallowMessageExisting = $( '#mw-abusefilter-disallow-message-existing select' );
 			newVal = mw.config.get( 'wgAbuseFilterDefaultDisallowMessage' )[ $( this ).val() ];
 
 			if ( $afDisallowMessageExisting.find( 'option[value=\'' + newVal + '\']' ).length ) {
@@ -337,30 +327,27 @@
 	 */
 	function toggleCustomMessages() {
 		// Use the table over here as hideDeselectedActions might alter the visibility of the div
-		var $warnOptions = $( '#mw-abusefilter-warn-parameters > table' ),
+		const $warnOptions = $( '#mw-abusefilter-warn-parameters > table' ),
 			$disallowOptions = $( '#mw-abusefilter-disallow-parameters > table' );
 
-		if ( $( '#wpFilterGlobal' ).is( ':checked' ) ) {
+		const isGlobalFilter = $( '#wpFilterGlobal' ).is( ':checked' );
+		if ( isGlobalFilter ) {
 			// It's a global filter, so use the default message and hide the option from the user
 			warnMessageExisting.setValue( 'abusefilter-warning' );
 			disallowMessageExisting.setValue( 'abusefilter-disallowed' );
-
-			$warnOptions.hide();
-			$disallowOptions.hide();
-		} else {
-			$warnOptions.show();
-			$disallowOptions.show();
 		}
+		$warnOptions.toggle( !isGlobalFilter );
+		$disallowOptions.toggle( !isGlobalFilter );
 	}
 
 	/**
 	 * Called if the user presses a key in the load filter field.
 	 *
-	 * @this HTMLELement
+	 * @this HTMLElement
 	 * @param {jQuery.Event} e The event fired when the function is called
 	 */
 	function onFilterKeypress( e ) {
-		if ( e.type === 'keypress' && e.which === 13 ) {
+		if ( e.type === 'keypress' && e.which === OO.ui.Keys.ENTER ) {
 			e.preventDefault();
 			$( '#mw-abusefilter-load' ).trigger( 'click' );
 		}
@@ -370,18 +357,15 @@
 	 * Warn if the user changed anything and tries to leave the window
 	 */
 	function setWarnOnLeave() {
-		var warnOnLeave,
-			$form = $( '#mw-abusefilter-editing-form' ),
+		const $form = $( '#mw-abusefilter-editing-form' ),
 			origValues = $form.serialize();
 
-		warnOnLeave = mw.confirmCloseWindow( {
-			test: function () {
-				return $form.serialize() !== origValues;
-			},
+		const warnOnLeave = mw.confirmCloseWindow( {
+			test: () => $form.serialize() !== origValues,
 			message: mw.msg( 'abusefilter-edit-warn-leave' )
 		} );
 
-		$form.on( 'submit', function () {
+		$form.on( 'submit', () => {
 			warnOnLeave.release();
 		} );
 	}
@@ -394,12 +378,11 @@
 	 */
 	function buildSelector( action, config ) {
 		// mw-abusefilter-throttle-parameters, mw-abusefilter-tag-parameters
-		var $container = $( '#mw-abusefilter-' + action + '-parameters' ),
+		const $container = $( '#mw-abusefilter-' + action + '-parameters' ),
 			// Character used to separate elements in the textarea.
-			separator = action === 'throttle' ? '\n' : ',',
-			selector, field, fieldOpts, hiddenField;
+			separator = action === 'throttle' ? '\n' : ',';
 
-		selector = new OO.ui.TagMultiselectWidget( {
+		const selector = new OO.ui.TagMultiselectWidget( {
 			inputPosition: 'outline',
 			allowArbitrary: true,
 			allowEditTags: true,
@@ -412,7 +395,7 @@
 			disabled: config.disabled
 		} );
 
-		fieldOpts = {
+		const fieldOpts = {
 			label: $( $.parseHTML( config.label ) ),
 			align: 'top'
 		};
@@ -420,11 +403,11 @@
 			fieldOpts.help = new OO.ui.HtmlSnippet( config.help );
 		}
 
-		field = new OO.ui.FieldLayout( selector, fieldOpts );
+		const field = new OO.ui.FieldLayout( selector, fieldOpts );
 
 		// mw-abusefilter-hidden-throttle-field, mw-abusefilter-hidden-tag-field
-		hiddenField = OO.ui.infuse( $( '#mw-abusefilter-hidden-' + action + '-field' ) );
-		selector.on( 'change', function () {
+		const hiddenField = OO.ui.infuse( $( '#mw-abusefilter-hidden-' + action + '-field' ) );
+		selector.on( 'change', () => {
 			hiddenField.setValue( selector.getValue().join( separator ) );
 		} );
 
@@ -434,14 +417,12 @@
 	}
 
 	// On ready initialization
-	$( function () {
-		var basePath, readOnly,
-			$exportBox = $( '#mw-abusefilter-export' ),
+	$( () => {
+		const $exportBox = $( '#mw-abusefilter-export' ),
 			isFilterEditor = mw.config.get( 'isFilterEditor' ),
 			tagConfig = mw.config.get( 'tagConfig' ),
 			throttleConfig = mw.config.get( 'throttleConfig' ),
-			$switchEditorBtn = $( '#mw-abusefilter-switcheditor' ),
-			cbEnabled, cbDeleted;
+			$switchEditorBtn = $( '#mw-abusefilter-switcheditor' );
 
 		if ( isFilterEditor ) {
 			// Configure the actual editing interface
@@ -467,26 +448,34 @@
 
 		if ( $( '#wpAceFilterEditor' ).length ) {
 			// CodeEditor is installed.
-			mw.loader.using( [ 'ext.abuseFilter.ace' ] ).then( function () {
+			mw.loader.using( [ 'ext.abuseFilter.ace' ] ).then( () => {
 				$filterBox = $( '#wpAceFilterEditor' );
 
 				filterEditor = ace.edit( 'wpAceFilterEditor' );
 				filterEditor.session.setMode( 'ace/mode/abusefilter' );
 
 				// Ace setup from codeEditor extension
-				basePath = mw.config.get( 'wgExtensionAssetsPath', '' );
-				if ( basePath.slice( 0, 2 ) === '//' ) {
+				let basePath = mw.config.get( 'wgExtensionAssetsPath', '' );
+				if ( basePath.startsWith( '//' ) ) {
 					// ACE uses web workers, which have importScripts, which don't like
 					// relative links. This is a problem only when the assets are on another
 					// server, so this rewrite should suffice.
 					basePath = window.location.protocol + basePath;
 				}
-				ace.config.set( 'basePath', basePath + '/CodeEditor/modules/ace' );
+				ace.config.set( 'basePath', basePath + '/CodeEditor/modules/lib/ace' );
 
 				// Settings for Ace editor box
-				readOnly = mw.config.get( 'aceConfig' ).aceReadOnly;
+				const readOnly = mw.config.get( 'aceConfig' ).aceReadOnly;
 
-				filterEditor.setTheme( 'ace/theme/textmate' );
+				// Get theme of user and set it for the editor
+				// Copied from CodeEditor extension commit 8b7b17f9ee357d2c789909f452a57fa7b4237384
+				const htmlClasses = document.documentElement.classList;
+				const inDarkMode = htmlClasses.contains( 'skin-theme-clientpref-night' ) || (
+					htmlClasses.contains( 'skin-theme-clientpref-os' ) &&
+					window.matchMedia && window.matchMedia( '( prefers-color-scheme: dark )' ).matches
+				);
+
+				filterEditor.setTheme( inDarkMode ? 'ace/theme/monokai' : 'ace/theme/textmate' );
 				filterEditor.setReadOnly( readOnly );
 				filterEditor.$blockScrolling = Infinity;
 
@@ -494,8 +483,8 @@
 				switchEditor();
 
 				// Hide the syntax ok message when the text changes and sync dummy box
-				filterEditor.on( 'change', function () {
-					var $el = $( '#mw-abusefilter-syntaxresult' );
+				filterEditor.on( 'change', () => {
+					const $el = $( '#mw-abusefilter-syntaxresult' );
 
 					if ( $el.data( 'syntaxOk' ) ) {
 						$el.hide();
@@ -511,19 +500,18 @@
 					$filterBox.css( 'resize', 'both' );
 					// Refresh Ace editor size (notably its scrollbars) when the container
 					// is resized, otherwise it would be refreshed only on window resize
-					new ResizeObserver( function () {
+					new ResizeObserver( () => {
 						filterEditor.resize();
 					} ).observe( $filterBox[ 0 ] );
 				}
 
-				$switchEditorBtn.on( 'click', switchEditor );
-				$switchEditorBtn.show();
+				$switchEditorBtn.on( 'click', switchEditor ).show();
 			} );
 		}
 
 		// Hide the syntax ok message when the text changes
-		$plainTextBox.on( 'change', function () {
-			var $el = $( '#mw-abusefilter-syntaxresult' );
+		$plainTextBox.on( 'change', () => {
+			const $el = $( '#mw-abusefilter-syntaxresult' );
 
 			if ( $el.data( 'syntaxOk' ) ) {
 				$el.hide();
@@ -536,16 +524,24 @@
 		if ( isFilterEditor ) {
 			// Add logic for flags and consequences
 			$( '#mw-abusefilter-warn-preview-button' ).on( 'click',
-				function () { previewMessage( 'warn' ); }
+				() => {
+					previewMessage( 'warn' );
+				}
 			);
 			$( '#mw-abusefilter-disallow-preview-button' ).on( 'click',
-				function () { previewMessage( 'disallow' ); }
+				() => {
+					previewMessage( 'disallow' );
+				}
 			);
 			$( '#mw-abusefilter-warn-edit-button' ).on( 'click',
-				function () { editMessage( 'warn' ); }
+				() => {
+					editMessage( 'warn' );
+				}
 			);
 			$( '#mw-abusefilter-disallow-edit-button' ).on( 'click',
-				function () { editMessage( 'disallow' ); }
+				() => {
+					editMessage( 'disallow' );
+				}
 			);
 			$( '.mw-abusefilter-action-checkbox input' ).on( 'click', hideDeselectedActions );
 			hideDeselectedActions();
@@ -553,11 +549,11 @@
 			$( '#wpFilterGlobal' ).on( 'change', toggleCustomMessages );
 			toggleCustomMessages();
 
-			cbEnabled = OO.ui.infuse( $( '#wpFilterEnabled' ) );
-			cbDeleted = OO.ui.infuse( $( '#wpFilterDeleted' ) );
-			OO.ui.infuse( $( '#wpFilterDeletedLabel' ) );
+			const cbEnabled = OO.ui.infuse( $( '#wpFilterEnabled' ) );
+			const cbDeleted = OO.ui.infuse( $( '#wpFilterDeleted' ) );
+
 			cbEnabled.on( 'change',
-				function () {
+				() => {
 					cbDeleted.setDisabled( cbEnabled.isSelected() );
 					if ( cbEnabled.isSelected() ) {
 						cbDeleted.setSelected( false );
@@ -566,7 +562,7 @@
 			);
 
 			cbDeleted.on( 'change',
-				function () {
+				() => {
 					if ( cbDeleted.isSelected() ) {
 						cbEnabled.setSelected( false );
 					}
@@ -576,7 +572,7 @@
 			$( '#mw-abusefilter-edit-group-input select' ).on( 'change', onFilterGroupChange );
 
 			$( '#mw-abusefilter-export-link' ).on( 'click',
-				function ( e ) {
+				( e ) => {
 					e.preventDefault();
 					$exportBox.toggle();
 				}
