@@ -13,11 +13,11 @@ use MediaWiki\Tests\ExtensionJsonTestBase;
 class AbuseFilterExtensionJsonTest extends ExtensionJsonTestBase {
 
 	/** @inheritDoc */
-	protected string $extensionJsonPath = __DIR__ . '/../../../extension.json';
+	protected static string $extensionJsonPath = __DIR__ . '/../../../extension.json';
 
-	public function provideHookHandlerNames(): iterable {
+	public static function provideHookHandlerNames(): iterable {
 		$extHookHandlers = [ 'CheckUser', 'ConfirmEdit', 'Echo', 'UserMerge' ];
-		foreach ( $this->getExtensionJson()['HookHandlers'] ?? [] as $name => $specification ) {
+		foreach ( self::getExtensionJson()['HookHandlers'] ?? [] as $name => $specification ) {
 			if ( in_array( $name, $extHookHandlers ) && !ExtensionRegistry::getInstance()->isLoaded( $name ) ) {
 				continue;
 			}
