@@ -22,11 +22,13 @@ use MediaWiki\User\UserRigorOptions;
 use Wikimedia\IPUtils;
 use Wikimedia\Timestamp\ConvertibleTimestamp;
 
+// @codeCoverageIgnoreStart
 $IP = getenv( 'MW_INSTALL_PATH' );
 if ( $IP === false ) {
 	$IP = __DIR__ . '/../../..';
 }
 require_once "$IP/maintenance/Maintenance.php";
+// @codeCoverageIgnoreEnd
 
 /**
  * Populates the CheckUser tables with simulated data that can be useful for
@@ -41,7 +43,7 @@ class PopulateCheckUserTablesWithSimulatedData extends Maintenance {
 		'move' => [ 'move', 'move_redir' ],
 		'delete' => [ 'delete', 'restore' ],
 		'suppress' => [ 'delete' ],
-		'merge' => [ 'merge' ]
+		'merge' => [ 'merge' ],
 	];
 
 	/** @var array<string,?ClientHintsData> */
@@ -751,12 +753,12 @@ class PopulateCheckUserTablesWithSimulatedData extends Maintenance {
 		if ( $type === 'move' ) {
 			$logEntry->setParameters( [
 				'4::target' => $this->getPrefix() . wfRandomString(),
-				'5::noredir' => '0'
+				'5::noredir' => '0',
 			] );
 		} elseif ( $type === 'merge' ) {
 			$logEntry->setParameters( [
 				'4::dest' => $this->getPrefix() . wfRandomString(),
-				'5::mergepoint' => $logEntry->getTimestamp()
+				'5::mergepoint' => $logEntry->getTimestamp(),
 			] );
 		} elseif ( $type === 'delete' && $action === 'undelete' ) {
 			$logEntry->setParameters( [
@@ -966,5 +968,7 @@ class PopulateCheckUserTablesWithSimulatedData extends Maintenance {
 	}
 }
 
+// @codeCoverageIgnoreStart
 $maintClass = PopulateCheckUserTablesWithSimulatedData::class;
 require_once RUN_MAINTENANCE_IF_MAIN;
+// @codeCoverageIgnoreEnd
