@@ -33,4 +33,7 @@ interface CheckUserQueryInterface {
 
 	/** @var string The virtual database domain for the central index tables */
 	public const VIRTUAL_GLOBAL_DB_DOMAIN = 'virtual-checkuser-global';
+
+	/** @var string The virtual database domain for the suggested investigations feature */
+	public const VIRTUAL_DB_DOMAIN = 'virtual-checkuser';
 }
