@@ -5,7 +5,7 @@ namespace MediaWiki\CheckUser\Tests\Structure;
 class BundleSizeTest extends \MediaWiki\Tests\Structure\BundleSizeTestBase {
 
 	/** @inheritDoc */
-	public function getBundleSizeConfig(): string {
+	public static function getBundleSizeConfigData(): string {
 		return dirname( __DIR__, 3 ) . '/bundlesize.config.json';
 	}
 }
