@@ -3,7 +3,7 @@
 QUnit.module( 'ext.checkUser.clientHints', QUnit.newMwEnvironment( {} ) );
 
 QUnit.test( 'Client hints code is setup if navigator.userAgentData.getHighEntropyValues() is available', function ( assert ) {
-	const clientHints = require( '../../../modules/ext.checkUser.clientHints/index.js' );
+	const clientHints = require( 'ext.checkUser.clientHints/index.js' );
 	const responseMock = {
 		platform: 'macOS'
 	};
@@ -18,12 +18,12 @@ QUnit.test( 'Client hints code is setup if navigator.userAgentData.getHighEntrop
 } );
 
 QUnit.test( 'Client hints code is not setup if navigator.userAgentData is available but navigator.userAgentData.getHighEntropyValues() is not available', ( assert ) => {
-	const clientHints = require( '../../../modules/ext.checkUser.clientHints/index.js' );
+	const clientHints = require( 'ext.checkUser.clientHints/index.js' );
 	const navigatorData = { userAgentData: {} };
 	assert.false( clientHints.init( navigatorData ) );
 } );
 QUnit.test( 'Client hints code is not setup if navigator.userAgentData is not defined', ( assert ) => {
-	const clientHints = require( '../../../modules/ext.checkUser.clientHints/index.js' );
+	const clientHints = require( 'ext.checkUser.clientHints/index.js' );
 	const navigatorData = {};
 	assert.false( clientHints.init( navigatorData ) );
 } );
