@@ -34,15 +34,16 @@
 	};
 
 	$.wikiEditor.extensions.codeEditor = function ( context ) {
-		var saveAndExtend,
-			textSelectionFn,
-			hasErrorsOnSave = false,
+		let hasErrorsOnSave = false,
 			selectedLine = 0,
-			returnFalse = function () { return false; },
+			textSelectionFn = null;
+		const returnFalse = function () {
+				return false;
+			},
 			api = new mw.Api();
 
 		// Initialize state
-		var cookieEnabled = parseInt( mw.cookie.get( 'codeEditor-' + context.instance + '-showInvisibleChars' ), 10 );
+		let cookieEnabled = parseInt( mw.cookie.get( 'codeEditor-' + context.instance + '-showInvisibleChars' ), 10 );
 		context.showInvisibleChars = ( cookieEnabled === 1 );
 		cookieEnabled = parseInt( mw.cookie.get( 'codeEditor-' + context.instance + '-lineWrappingActive' ), 10 );
 		context.lineWrappingActive = ( cookieEnabled === 1 );
@@ -56,7 +57,7 @@
 		 *
 		 * This is also where we can attach some extra information to the events.
 		 */
-		context.evt = $.extend( context.evt, {
+		context.evt = Object.assign( context.evt, {
 			keydown: returnFalse,
 			change: returnFalse,
 			delayedChange: returnFalse,
@@ -64,11 +65,11 @@
 			paste: returnFalse,
 			ready: returnFalse,
 			codeEditorSubmit: function () {
-				var form = this;
+				const form = this;
 				context.evt.codeEditorSync();
 				if ( hasErrorsOnSave ) {
 					hasErrorsOnSave = false;
-					OO.ui.confirm( mw.msg( 'codeeditor-save-with-errors' ) ).done( function ( confirmed ) {
+					OO.ui.confirm( mw.msg( 'codeeditor-save-with-errors' ) ).then( ( confirmed ) => {
 						if ( confirmed ) {
 							// Programmatic submit doesn't retrigger this event listener
 							form.submit();
@@ -79,9 +80,7 @@
 				return true;
 			},
 			codeEditorSave: function () {
-				if ( context.codeEditor.getSession().getAnnotations().some( function ( ann ) {
-					return ann.type === 'error';
-				} ) ) {
+				if ( context.codeEditor.getSession().getAnnotations().some( ( ann ) => ann.type === 'error' ) ) {
 					hasErrorsOnSave = true;
 				}
 			},
@@ -97,7 +96,7 @@
 		/**
 		 * Internally used functions
 		 */
-		context.fn = $.extend( context.fn, {
+		context.fn = Object.assign( context.fn, {
 			isCodeEditorActive: function () {
 				return context.codeEditorActive;
 			},
@@ -116,14 +115,14 @@
 			aceGotoLineColumn: function () {
 				OO.ui.prompt( mw.msg( 'codeeditor-gotoline-prompt' ), {
 					textInput: { placeholder: mw.msg( 'codeeditor-gotoline-placeholder' ) }
-				} ).done( function ( result ) {
+				} ).then( ( result ) => {
 					if ( !result ) {
 						return;
 					}
 
-					var matches = result.split( ':' );
-					var line = 0;
-					var column = 0;
+					const matches = result.split( ':' );
+					let line = 0;
+					let column = 0;
 
 					if ( matches.length > 0 ) {
 						line = +matches[ 0 ];
@@ -146,7 +145,7 @@
 				} );
 			},
 			setupCodeEditorToolbar: function () {
-				var toggleEditor = function ( ctx ) {
+				const toggleEditor = function ( ctx ) {
 					ctx.codeEditorActive = !ctx.codeEditorActive;
 
 					ctx.fn.setCodeEditorPreference( ctx.codeEditorActive );
@@ -159,7 +158,7 @@
 						ctx.fn.disableCodeEditor();
 					}
 				};
-				var toggleInvisibleChars = function ( ctx ) {
+				const toggleInvisibleChars = function ( ctx ) {
 					ctx.showInvisibleChars = !ctx.showInvisibleChars;
 
 					ctx.fn.changeCookieValue( 'showInvisibleChars', ctx.showInvisibleChars ? 1 : 0 );
@@ -167,8 +166,8 @@
 
 					ctx.codeEditor.setShowInvisibles( ctx.showInvisibleChars );
 				};
-				var toggleSearchReplace = function ( ctx ) {
-					var searchBox = ctx.codeEditor.searchBox;
+				const toggleSearchReplace = function ( ctx ) {
+					const searchBox = ctx.codeEditor.searchBox;
 					if ( searchBox && $( searchBox.element ).css( 'display' ) !== 'none' ) {
 						searchBox.hide();
 					} else {
@@ -177,7 +176,7 @@
 						);
 					}
 				};
-				var toggleLineWrapping = function ( ctx ) {
+				const toggleLineWrapping = function ( ctx ) {
 					ctx.lineWrappingActive = !ctx.lineWrappingActive;
 
 					ctx.fn.changeCookieValue( 'lineWrappingActive', ctx.lineWrappingActive ? 1 : 0 );
@@ -185,13 +184,13 @@
 
 					ctx.codeEditor.getSession().setUseWrapMode( ctx.lineWrappingActive );
 				};
-				var indent = function ( ctx ) {
+				const indent = function ( ctx ) {
 					ctx.codeEditor.execCommand( 'indent' );
 				};
-				var outdent = function ( ctx ) {
+				const outdent = function ( ctx ) {
 					ctx.codeEditor.execCommand( 'outdent' );
 				};
-				var gotoLine = function ( ctx ) {
+				const gotoLine = function ( ctx ) {
 					ctx.codeEditor.execCommand( 'gotolinecolumn' );
 				};
 
@@ -201,7 +200,7 @@
 						'codeeditor-main': {
 							tools: {
 								codeEditor: {
-									labelMsg: 'codeeditor-toolbar-toggle',
+									label: mw.msg( 'codeeditor-toolbar-toggle' ),
 									type: 'toggle',
 									oouiIcon: 'markup',
 									action: {
@@ -214,7 +213,7 @@
 						'codeeditor-format': {
 							tools: {
 								indent: {
-									labelMsg: 'codeeditor-indent',
+									label: mw.msg( 'codeeditor-indent' ),
 									type: 'button',
 									oouiIcon: 'indent',
 									action: {
@@ -223,7 +222,7 @@
 									}
 								},
 								outdent: {
-									labelMsg: 'codeeditor-outdent',
+									label: mw.msg( 'codeeditor-outdent' ),
 									type: 'button',
 									oouiIcon: 'outdent',
 									action: {
@@ -237,7 +236,7 @@
 						'codeeditor-style': {
 							tools: {
 								invisibleChars: {
-									labelMsg: 'codeeditor-invisibleChars-toggle',
+									label: mw.msg( 'codeeditor-invisibleChars-toggle' ),
 									type: 'toggle',
 									oouiIcon: 'pilcrow',
 									action: {
@@ -246,7 +245,7 @@
 									}
 								},
 								lineWrapping: {
-									labelMsg: 'codeeditor-lineWrapping-toggle',
+									label: mw.msg( 'codeeditor-lineWrapping-toggle' ),
 									type: 'toggle',
 									oouiIcon: 'wrapping',
 									action: {
@@ -255,7 +254,7 @@
 									}
 								},
 								gotoLine: {
-									labelMsg: 'codeeditor-gotoline',
+									label: mw.msg( 'codeeditor-gotoline' ),
 									type: 'button',
 									oouiIcon: 'gotoLine',
 									action: {
@@ -264,7 +263,7 @@
 									}
 								},
 								toggleSearchReplace: {
-									labelMsg: 'codeeditor-searchReplace-toggle',
+									label: mw.msg( 'codeeditor-searchReplace-toggle' ),
 									type: 'button',
 									oouiIcon: 'articleSearch',
 									action: {
@@ -284,7 +283,7 @@
 				$( '.group-codeeditor-main' ).prependTo( '.section-main' );
 			},
 			updateButtonIcon: function ( targetName, iconFn ) {
-				var target = '.tool[rel=' + targetName + ']',
+				const target = '.tool[rel=' + targetName + ']',
 					$button = context.modules.toolbar.$toolbar.find( target );
 
 				$button.data( 'setActive' )( iconFn() );
@@ -299,22 +298,21 @@
 				context.fn.updateButtonIcon( 'lineWrapping', context.fn.isLineWrappingActive );
 			},
 			setCodeEditorPreference: function ( prefValue ) {
-				// Do not try to save options for anonymous user
-				if ( mw.user.isAnon() ) {
-					return;
-				}
-
 				// Abort any previous request
 				api.abort();
 
 				api.saveOption( 'usecodeeditor', prefValue ? 1 : 0 )
-					.fail( function ( code, result ) {
+					.catch( ( code, result ) => {
 						if ( code === 'http' && result.textStatus === 'abort' ) {
 							// Request was aborted. Ignore error
 							return;
 						}
+						if ( code === 'notloggedin' ) {
+							// Expected for non-registered users
+							return;
+						}
 
-						var message = 'Failed to set code editor preference: ' + code;
+						let message = 'Failed to set code editor preference: ' + code;
 						if ( result.error && result.error.info ) {
 							message += '\n' + result.error.info;
 						}
@@ -325,24 +323,24 @@
 			 * Sets up the iframe in place of the textarea to allow more advanced operations
 			 */
 			setupCodeEditor: function () {
-				var $box = context.$textarea;
-				var lang = mw.config.get( 'wgCodeEditorCurrentLanguage' );
-				var basePath = mw.config.get( 'wgExtensionAssetsPath', '' );
+				const $box = context.$textarea;
+				let lang = mw.config.get( 'wgCodeEditorCurrentLanguage' );
+				let basePath = mw.config.get( 'wgExtensionAssetsPath', '' );
 				if ( basePath.slice( 0, 2 ) === '//' ) {
 					// ACE uses web workers, which have importScripts, which don't like relative links.
 					// This is a problem only when the assets are on another server, so this rewrite should suffice
 					// Protocol relative
 					basePath = window.location.protocol + basePath;
 				}
-				ace.config.set( 'basePath', basePath + '/CodeEditor/modules/ace' );
+				ace.config.set( 'basePath', basePath + '/CodeEditor/modules/lib/ace' );
 
 				if ( lang ) {
 					// Ace doesn't like replacing a textarea directly.
 					// We'll stub this out to sit on top of it...
 					// line-height is needed to compensate for oddity in WikiEditor extension, which zeroes the line-height on a parent container
 					// eslint-disable-next-line no-jquery/no-parse-html-literal
-					var container = context.$codeEditorContainer = $( '<div style="position: relative"><div class="editor" style="line-height: 1.5em; top: 0; left: 0; right: 0; bottom: 0; position: absolute;"></div></div>' ).insertAfter( $box );
-					var editdiv = container.find( '.editor' );
+					const container = context.$codeEditorContainer = $( '<div style="position: relative"><div class="editor" style="line-height: 1.5em; top: 0; left: 0; right: 0; bottom: 0; position: absolute;"></div></div>' ).insertAfter( $box );
+					const editdiv = container.find( '.editor' );
 
 					$box.css( 'display', 'none' );
 					container.height( $box.height() );
@@ -365,10 +363,18 @@
 					context.codeEditor.setReadOnly( $box.prop( 'readonly' ) );
 					context.codeEditor.setShowInvisibles( context.showInvisibleChars );
 
+					const htmlClasses = document.documentElement.classList;
+					const inDarkMode = htmlClasses.contains( 'skin-theme-clientpref-night' ) || (
+						htmlClasses.contains( 'skin-theme-clientpref-os' ) &&
+						window.matchMedia && window.matchMedia( '(prefers-color-scheme: dark)' ).matches
+					);
+
 					// The options to enable
 					context.codeEditor.setOptions( {
 						enableBasicAutocompletion: true,
-						enableSnippets: true
+						enableLiveAutocompletion: true,
+						enableSnippets: true,
+						theme: inDarkMode ? 'ace/theme/monokai' : 'ace/theme/textmate'
 					} );
 
 					context.codeEditor.commands.addCommand( {
@@ -382,26 +388,32 @@
 						.on( 'submit', context.evt.codeEditorSubmit )
 						.find( '#wpSave' ).on( 'click', context.evt.codeEditorSave );
 
-					var session = context.codeEditor.getSession();
+					const session = context.codeEditor.getSession();
 
 					// Use proper tabs
 					session.setUseSoftTabs( false );
 					session.setUseWrapMode( context.lineWrappingActive );
 
 					// Configure any workers
-					session.on( 'changeMode', function ( e, session2 ) {
+					session.on( 'changeMode', ( e, session2 ) => {
 						// eslint-disable-next-line no-jquery/variable-pattern
-						var mode = session2.getMode().$id;
+						const mode = session2.getMode().$id;
 						if ( mode === 'ace/mode/javascript' ) {
 							session2.$worker.send( 'changeOptions', [ {
-								maxerr: 1000
+								maxerr: 1000,
+								globals: { mw: true, mediaWiki: true, $: true, jQuery: true, OO: true }
 							} ] );
 						}
 					} );
 
 					mw.hook( 'codeEditor.configure' ).fire( session );
 
-					ace.config.loadModule( 'ace/ext/modelist', function ( modelist ) {
+					// Add an Ace change handler to pass changes to Edit Recovery.
+					mw.hook( 'editRecovery.loadEnd' ).add( ( data ) => {
+						session.on( 'change', data.fieldChangeHandler );
+					} );
+
+					ace.config.loadModule( 'ace/ext/modelist', ( modelist ) => {
 						if ( !modelist || !modelist.modesByName[ lang ] ) {
 							lang = 'text';
 						}
@@ -409,10 +421,12 @@
 					} );
 
 					// Use jQuery UI resizable() so that users can make the box taller
+
 					container.resizable( {
 						handles: 's',
 						minHeight: $box.height(),
 						resize: function () {
+
 							context.codeEditor.resize();
 						}
 					} );
@@ -427,6 +441,8 @@
 
 					context.fn.setupStatusBar();
 
+					document.body.classList.remove( 'codeeditor-loading' );
+
 					// Let modules know we're ready to start working with the content
 					context.fn.trigger( 'ready' );
 				}
@@ -468,8 +484,8 @@
 			 */
 			codeEditorMonitorFragment: function () {
 				function onHashChange() {
-					var regexp = /#mw-ce-l(\d+)/;
-					var result = regexp.exec( window.location.hash );
+					const regexp = /#mw-ce-l(\d+)/;
+					const result = regexp.exec( window.location.hash );
 
 					if ( result === null ) {
 						return;
@@ -493,12 +509,11 @@
 			 * the position of the cursor.
 			 */
 			setupStatusBar: function () {
-				var shouldUpdateAnnotations,
+				let shouldUpdateAnnotations,
 					shouldUpdateSelection,
 					shouldUpdateLineInfo,
-					nextAnnotation,
-					delayedUpdate,
-					editor = context.codeEditor,
+					nextAnnotation;
+				const editor = context.codeEditor,
 					lang = ace.require( 'ace/lib/lang' ),
 					$errors = $( '<span>' ).addClass( 'codeEditor-status-worker-cell ace_gutter-cell ace_error' ).text( '0' ),
 					$warnings = $( '<span>' ).addClass( 'codeEditor-status-worker-cell ace_gutter-cell ace_warning' ).text( '0' ),
@@ -529,21 +544,21 @@
 				 * Update all the information in the status bar
 				 */
 				function updateStatusBar() {
-					var errors = 0,
+					let errors = 0,
 						warnings = 0,
 						infos = 0,
 						distance,
 						shortestDistance = Infinity,
 						closestAnnotation,
-						currentLine = editor.selection.lead.row,
-						annotations = editor.getSession().getAnnotations(),
 						closestType;
+					const currentLine = editor.selection.lead.row,
+						annotations = editor.getSession().getAnnotations();
 
 					// Reset the next annotation
 					nextAnnotation = null;
 
-					for ( var i = 0; i < annotations.length; i++ ) {
-						var annotation = annotations[ i ];
+					for ( let i = 0; i < annotations.length; i++ ) {
+						const annotation = annotations[ i ];
 						distance = Math.abs( currentLine - annotation.row );
 
 						if ( distance < shortestDistance ) {
@@ -597,7 +612,7 @@
 					// The cursor position has changed
 					if ( shouldUpdateSelection || shouldUpdateLineInfo ) {
 						// Adapted from Ajax.org's ace/ext/statusbar module
-						var status = [];
+						const status = [];
 
 						if ( editor.$vimModeHandler ) {
 							addToStatus( status, editor.$vimModeHandler.getStatusText() );
@@ -605,10 +620,10 @@
 							addToStatus( status, 'REC' );
 						}
 
-						var c = editor.selection.lead;
+						const c = editor.selection.lead;
 						addToStatus( status, ( c.row + 1 ) + ':' + c.column, '' );
 						if ( !editor.selection.isEmpty() ) {
-							var r = editor.getSelectionRange();
+							const r = editor.getSelectionRange();
 							addToStatus( status, '(' + ( r.end.row - r.start.row ) + ':' + ( r.end.column - r.start.column ) + ')' );
 						}
 						status.pop();
@@ -619,14 +634,14 @@
 				}
 
 				// Function to delay/debounce updates for the StatusBar
-				delayedUpdate = lang.delayedCall( function () {
+				const delayedUpdate = lang.delayedCall( () => {
 					updateStatusBar( editor );
 				} );
 
 				/**
 				 * Click handler that allows you to skip to the next annotation
 				 */
-				$workerStatus.on( 'click', function ( e ) {
+				$workerStatus.on( 'click', ( e ) => {
 					if ( nextAnnotation ) {
 						context.codeEditor.navigateTo( nextAnnotation.row, nextAnnotation.column );
 						// Scroll up a bit to give some context
@@ -635,15 +650,15 @@
 					}
 				} );
 
-				editor.getSession().on( 'changeAnnotation', function () {
+				editor.getSession().on( 'changeAnnotation', () => {
 					shouldUpdateAnnotations = true;
 					delayedUpdate.schedule( 100 );
 				} );
-				editor.on( 'changeStatus', function () {
+				editor.on( 'changeStatus', () => {
 					shouldUpdateLineInfo = true;
 					delayedUpdate.schedule( 100 );
 				} );
-				editor.on( 'changeSelection', function () {
+				editor.on( 'changeSelection', () => {
 					shouldUpdateSelection = true;
 					delayedUpdate.schedule( 100 );
 				} );
@@ -652,7 +667,7 @@
 				shouldUpdateLineInfo = shouldUpdateSelection = shouldUpdateAnnotations = true;
 				updateStatusBar( editor );
 
-				context.$statusBar.insertAfter( $( '.wikiEditor-ui-view-wikitext .wikiEditor-ui-bottom' ) );
+				context.$statusBar.insertAfter( context.$ui.find( '.wikiEditor-ui-bottom' ) );
 			},
 			removeStatusBar: function () {
 				context.codeEditor.getSession().removeListener( 'changeAnnotation' );
@@ -673,11 +688,11 @@
 		 * @param {Object} base
 		 * @param {Object} extended
 		 */
-		saveAndExtend = function ( base, extended ) {
+		const saveAndExtend = function ( base, extended ) {
 			// eslint-disable-next-line no-jquery/no-map-util
-			$.map( extended, function ( func, name ) {
+			$.map( extended, ( func, name ) => {
 				if ( name in base ) {
-					var orig = base[ name ];
+					const orig = base[ name ];
 					base[ name ] = function () {
 						if ( context.codeEditorActive ) {
 							return func.apply( this, arguments );
@@ -722,6 +737,7 @@
 
 			setContents: function ( newContents ) {
 				context.codeEditor.getSession().setValue( newContents );
+				return context.$textarea;
 			},
 
 			/**
@@ -734,6 +750,18 @@
 				return context.codeEditor.getCopyText();
 			},
 
+			/**
+			 * Replace the current selection with the given text.
+			 * DO NOT CALL THIS DIRECTLY, use $.textSelection( 'functionname', options ) instead
+			 *
+			 * @param {string} text
+			 * @return {jQuery}
+			 */
+			replaceSelection: function ( text ) {
+				context.codeEditor.insert( text );
+				return context.$textarea;
+			},
+
 			/**
 			 * Inserts text at the begining and end of a text selection, optionally inserting text at the caret when
 			 * selection is empty.
@@ -744,10 +772,10 @@
 			 */
 			encapsulateSelection: function ( options ) {
 				// Does not yet handle 'ownline', 'splitlines' option
-				var sel = context.codeEditor.getSelection();
-				var range = sel.getRange();
-				var selText = textSelectionFn.getSelection();
-				var isSample = false;
+				const sel = context.codeEditor.getSelection();
+				const range = sel.getRange();
+				let selText = textSelectionFn.getSelection();
+				let isSample = false;
 
 				if ( !selText ) {
 					selText = options.peri;
@@ -756,7 +784,7 @@
 					selText = options.peri;
 				}
 
-				var text = options.pre;
+				let text = options.pre;
 				text += selText;
 				text += options.post;
 				context.codeEditor.insert( text );
@@ -772,9 +800,24 @@
 			/**
 			 * Gets the position (in resolution of bytes not nessecarily characters) in a textarea
 			 * DO NOT CALL THIS DIRECTLY, use $.textSelection( 'functionname', options ) instead
+			 *
+			 * @param {Object} options
+			 * @param {Object} [options.startAndEnd=false] Return range of the selection rather than just start
+			 * @return {number|number[]} If options.startAndEnd is true, returns an array holding the start and
+			 * end of the selection, else returns only the start of the selection as a single number.
 			 */
-			getCaretPosition: function () {
-				mw.log( 'codeEditor stub function getCaretPosition called' );
+			getCaretPosition: function ( options ) {
+				const selection = context.codeEditor.getSelection(),
+					range = selection.getRange(),
+					doc = context.codeEditor.getSession().getDocument(),
+					startOffset = doc.positionToIndex( range.start );
+
+				if ( options.startAndEnd ) {
+					const endOffset = doc.positionToIndex( range.end );
+					return [ startOffset, endOffset ];
+				}
+
+				return startOffset;
 			},
 
 			/**
@@ -787,11 +830,11 @@
 			setSelection: function ( options ) {
 				// Ace stores positions for ranges as row/column pairs.
 				// To convert from character offsets, we'll need to iterate through the document
-				var doc = context.codeEditor.getSession().getDocument();
-				var lines = doc.getAllLines();
+				const doc = context.codeEditor.getSession().getDocument();
+				const lines = doc.getAllLines();
 
-				var offsetToPos = function ( offset ) {
-					var row, col, pos;
+				const offsetToPos = function ( offset ) {
+					let row, col, pos;
 
 					row = 0;
 					col = 0;
@@ -805,11 +848,11 @@
 					col = offset - pos;
 					return { row: row, column: col };
 				};
-				var start = offsetToPos( options.start );
-				var end = offsetToPos( options.end );
+				const start = offsetToPos( options.start );
+				const end = offsetToPos( options.end );
 
-				var sel = context.codeEditor.getSelection();
-				var range = sel.getRange();
+				const sel = context.codeEditor.getSelection();
+				const range = sel.getRange();
 				range.setStart( start.row, start.column );
 				range.setEnd( end.row, end.column );
 				sel.setSelectionRange( range );
