@@ -4,20 +4,22 @@ namespace MediaWiki\Extension\Gadgets;
 
 use MediaWiki\DomainEvent\DomainEventIngress;
 use MediaWiki\Page\Event\PageDeletedEvent;
+use MediaWiki\Page\Event\PageDeletedListener;
 use MediaWiki\Page\Event\PageRevisionUpdatedEvent;
+use MediaWiki\Page\Event\PageRevisionUpdatedListener;
 use MediaWiki\Title\TitleValue;
 
 /**
  * Event subscriber acting as an ingress for relevant events emitted
  * by MediaWiki core.
  */
-class EventIngress extends DomainEventIngress {
-	private GadgetRepo $gadgetRepo;
-
+class EventIngress
+	extends DomainEventIngress
+	implements PageRevisionUpdatedListener, PageDeletedListener
+{
 	public function __construct(
-		GadgetRepo $gadgetRepo
+		private readonly GadgetRepo $gadgetRepo,
 	) {
-		$this->gadgetRepo = $gadgetRepo;
 	}
 
 	/**
