@@ -6,11 +6,12 @@ use MediaWiki\JobQueue\Job;
 use MediaWiki\Title\Title;
 
 class PurgeSeenJob extends Job {
-	private LoginNotify $loginNotify;
-
-	public function __construct( Title $title, array $params, LoginNotify $loginNotify ) {
+	public function __construct(
+		Title $title,
+		array $params,
+		private readonly LoginNotify $loginNotify,
+	) {
 		parent::__construct( 'LoginNotifyPurgeSeen', $title, $params );
-		$this->loginNotify = $loginNotify;
 	}
 
 	/** @inheritDoc */
