@@ -1,7 +1,8 @@
 <?php
 
-namespace phpunit\unit\WikiTexVC\MMLNodes;
+namespace MediaWiki\Extension\Math\Tests\WikiTexVC\MMLNodes;
 
+use MediaWiki\Extension\Math\WikiTexVC\MMLmappings\TexConstants\Variants;
 use MediaWiki\Extension\Math\WikiTexVC\MMLnodes\MMLmn;
 use MediaWiki\Extension\Math\WikiTexVC\MMLnodes\MMLmsqrt;
 use MediaWikiUnitTestCase;
@@ -16,10 +17,10 @@ use MediaWikiUnitTestCase;
 class MMLmsqrtTest extends MediaWikiUnitTestCase {
 	public function testConstructor() {
 		$mn = new MMLmn( '', [], '5' );
-		$msqrt = new MMLmsqrt( '', [ 'mathvariant' => 'bold' ], $mn );
+		$msqrt = new MMLmsqrt( '', [ 'mathvariant' => Variants::BOLD ], $mn );
 
 		$this->assertEquals( 'msqrt', $msqrt->getName() );
-		$this->assertEquals( [ 'mathvariant' => 'bold' ], $msqrt->getAttributes() );
+		$this->assertEquals( [ 'mathvariant' => Variants::BOLD ], $msqrt->getAttributes() );
 		$this->assertEquals( $msqrt->getChildren(), [ $mn ] );
 	}
 }
