@@ -12,10 +12,9 @@ class HookRunner implements
 	FilterPageTitlesForEditHook,
 	FilterPageTitlesForRenameHook
 {
-	private HookContainer $hookContainer;
-
-	public function __construct( HookContainer $hookContainer ) {
-		$this->hookContainer = $hookContainer;
+	public function __construct(
+		private readonly HookContainer $hookContainer,
+	) {
 	}
 
 	/**
