@@ -32,6 +32,16 @@ ve.ce.TableRowNode.static.tagName = 'tr';
 
 /* Methods */
 
+/**
+ * @inheritdoc
+ */
+ve.ce.TableRowNode.prototype.initialize = function () {
+	// Parent method
+	ve.ce.TableRowNode.super.prototype.initialize.call( this );
+
+	ve.ce.TableCellNode.static.updateStyles( this.$element, this.model );
+};
+
 /**
  * @inheritdoc
  */
