@@ -93,6 +93,7 @@ ve.init.mw.DesktopArticleTarget.static.toolbarGroups.push(
 	{
 		name: 'help',
 		align: 'after',
+		excludeFromTargetWidget: true,
 		type: 'mwHelpList',
 		icon: 'help',
 		indicator: null,
@@ -103,11 +104,13 @@ ve.init.mw.DesktopArticleTarget.static.toolbarGroups.push(
 	{
 		name: 'notices',
 		align: 'after',
+		excludeFromTargetWidget: true,
 		include: [ { group: 'notices' } ]
 	},
 	{
 		name: 'pageMenu',
 		align: 'after',
+		excludeFromTargetWidget: true,
 		type: 'list',
 		icon: 'menu',
 		indicator: null,
@@ -120,6 +123,7 @@ ve.init.mw.DesktopArticleTarget.static.toolbarGroups.push(
 	{
 		name: 'editMode',
 		align: 'after',
+		excludeFromTargetWidget: true,
 		type: 'list',
 		icon: 'edit',
 		title: ve.msg( 'visualeditor-mweditmode-tooltip' ),
@@ -130,6 +134,7 @@ ve.init.mw.DesktopArticleTarget.static.toolbarGroups.push(
 	{
 		name: 'save',
 		align: 'after',
+		excludeFromTargetWidget: true,
 		type: 'bar',
 		include: [ { group: 'save' } ]
 	}
@@ -241,7 +246,7 @@ ve.init.mw.DesktopArticleTarget.prototype.setupToolbar = function ( surface ) {
 		}
 		this.toolbarSetupDeferred.resolve();
 
-		this.toolbarSetupDeferred.done( () => {
+		this.toolbarSetupDeferred.then( () => {
 			const newSurface = this.getSurface();
 			// Check the surface wasn't torn down while the toolbar was animating
 			if ( newSurface ) {
@@ -394,13 +399,13 @@ ve.init.mw.DesktopArticleTarget.prototype.activate = function ( dataPromise ) {
 		this.toolbarSetupDeferred = ve.createDeferred();
 
 		$( 'html' ).addClass( 've-activating' );
-		ve.promiseAll( [ this.activatingDeferred, this.toolbarSetupDeferred ] ).done( () => {
+		ve.promiseAll( [ this.activatingDeferred, this.toolbarSetupDeferred ] ).then( () => {
 			if ( !this.suppressNormalStartupDialogs ) {
 				this.maybeShowWelcomeDialog();
 				this.maybeShowMetaDialog();
 			}
 			this.afterActivate();
-		} ).fail( () => {
+		}, () => {
 			$( 'html' ).removeClass( 've-activating' );
 		} );
 
@@ -677,10 +682,10 @@ ve.init.mw.DesktopArticleTarget.prototype.loadFail = function ( code, errorDetai
 
 	OO.ui.confirm( $confirmPromptMessage, {
 		actions: [
-			{ action: 'accept', label: OO.ui.msg( 'ooui-dialog-process-retry' ), flags: 'primary' },
-			{ action: 'reject', label: OO.ui.msg( 'ooui-dialog-message-reject' ), flags: 'safe' }
+			{ action: 'accept', label: ve.msg( 'ooui-dialog-process-retry' ), flags: 'primary' },
+			{ action: 'reject', label: ve.msg( 'ooui-dialog-message-reject' ), flags: 'safe' }
 		]
-	} ).done( ( confirmed ) => {
+	} ).then( ( confirmed ) => {
 		if ( confirmed ) {
 			// Retry load
 			this.load();
@@ -795,7 +800,7 @@ ve.init.mw.DesktopArticleTarget.prototype.onMetaItemRemoved = function ( metaIte
  * @param {ve.dm.MetaItem[]} categoryItems Array of category metaitems to display
  */
 ve.init.mw.DesktopArticleTarget.prototype.rebuildCategories = function ( categoryItems ) {
-	this.renderCategories( categoryItems ).done( ( $categories ) => {
+	this.renderCategories( categoryItems ).then( ( $categories ) => {
 		// Clone the existing catlinks for any specific properties which might
 		// be needed by the rest of the page. Also gives us a not-attached
 		// version, which we can pass to wikipage.categories as it requests.
@@ -843,6 +848,8 @@ ve.init.mw.DesktopArticleTarget.prototype.saveComplete = function ( data ) {
 			$( '#t-permalink' ).add( '#coll-download-as-rl' ).find( 'a' ).each( ( i, el ) => {
 				const permalinkUrl = new URL( el.href );
 				permalinkUrl.searchParams.set( 'oldid', data.newrevid );
+				// permalinkUrl is safe
+				// eslint-disable-next-line local/no-unsanitized-href
 				$( el ).attr( 'href', permalinkUrl.toString() );
 			} );
 		}
@@ -868,7 +875,7 @@ ve.init.mw.DesktopArticleTarget.prototype.serialize = function () {
 	// Parent method
 	const promise = ve.init.mw.DesktopArticleTarget.super.prototype.serialize.apply( this, arguments );
 
-	return promise.fail( ( error, response ) => {
+	return promise.then( null, ( error, response ) => {
 		const $errorMessages = this.extractErrorMessages( response );
 		OO.ui.alert( $errorMessages );
 
@@ -1460,14 +1467,18 @@ ve.init.mw.DesktopArticleTarget.prototype.switchToFallbackWikitextEditor = funct
 /**
  * @inheritdoc
  */
-ve.init.mw.DesktopArticleTarget.prototype.reloadSurface = function () {
+ve.init.mw.DesktopArticleTarget.prototype.reloadSurface = function ( newMode ) {
 	this.activating = true;
 	this.activatingDeferred = ve.createDeferred();
 
 	// Parent method
 	ve.init.mw.DesktopArticleTarget.super.prototype.reloadSurface.apply( this, arguments );
 
-	this.activatingDeferred.done( () => {
+	this.activatingDeferred.then( () => {
+		if ( newMode === 'source' ) {
+			mw.hook( 've.wikitextInteractive' ).fire();
+		}
+
 		this.updateHistoryState();
 		this.afterActivate();
 		this.setupTriggerListeners();
