@@ -7,8 +7,7 @@ use MediaWiki\Notification\NotificationEnvelope;
 /**
  * An example Middleware that allows to remove notifications of specific types
  *
- * @since 1.44
- * @unstable
+ * @since 1.45
  */
 class SuppressNotificationByTypeMiddleware extends FilterMiddleware {
 
@@ -24,8 +23,9 @@ class SuppressNotificationByTypeMiddleware extends FilterMiddleware {
 	/**
 	 * Decide whether we want to remove notification from the list
 	 */
-	protected function filter( NotificationEnvelope $envelope ): bool {
-		return $envelope->getNotification()->getType() !== $this->notificationToSuppress;
+	protected function filter( NotificationEnvelope $envelope ): FilterMiddlewareAction {
+		return $envelope->getNotification()->getType() === $this->notificationToSuppress ?
+			FilterMiddlewareAction::REMOVE : FilterMiddlewareAction::KEEP;
 	}
 
 }
