@@ -1,33 +1,20 @@
 <?php
+declare( strict_types = 1 );
 /**
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * http://www.gnu.org/copyleft/gpl.html
- *
+ * @license GPL-2.0-or-later
  * @file
  */
 
 namespace MediaWiki\Storage;
 
-use BagOStuff;
-use Content;
-use Liuggio\StatsdClient\Factory\StatsdDataFactoryInterface;
+use JsonException;
+use MediaWiki\Content\Content;
 use MediaWiki\HookContainer\HookContainer;
 use MediaWiki\HookContainer\HookRunner;
+use MediaWiki\Json\JsonCodec;
 use MediaWiki\Page\PageIdentity;
+use MediaWiki\Page\WikiPage;
 use MediaWiki\Page\WikiPageFactory;
-use MediaWiki\Parser\ParserOutput;
 use MediaWiki\Parser\ParserOutputFlags;
 use MediaWiki\Revision\SlotRecord;
 use MediaWiki\Storage\Hook\ParserOutputStashForEditHook;
@@ -35,10 +22,10 @@ use MediaWiki\User\UserEditTracker;
 use MediaWiki\User\UserFactory;
 use MediaWiki\User\UserIdentity;
 use Psr\Log\LoggerInterface;
-use stdClass;
+use Wikimedia\ObjectCache\BagOStuff;
 use Wikimedia\Rdbms\IConnectionProvider;
 use Wikimedia\ScopedCallback;
-use WikiPage;
+use Wikimedia\Stats\StatsFactory;
 
 /**
  * Manage the pre-emptive page parsing for edits to wiki pages.
@@ -52,22 +39,8 @@ use WikiPage;
  * @ingroup Page
  */
 class PageEditStash {
-	/** @var BagOStuff */
-	private $cache;
-	/** @var IConnectionProvider */
-	private $dbProvider;
-	/** @var LoggerInterface */
-	private $logger;
-	/** @var StatsdDataFactoryInterface */
-	private $stats;
 	/** @var ParserOutputStashForEditHook */
 	private $hookRunner;
-	/** @var UserEditTracker */
-	private $userEditTracker;
-	/** @var UserFactory */
-	private $userFactory;
-	/** @var WikiPageFactory */
-	private $wikiPageFactory;
 	/** @var int */
 	private $initiator;
 
@@ -86,35 +59,40 @@ class PageEditStash {
 	public const INITIATOR_USER = 1;
 	public const INITIATOR_JOB_OR_CLI = 2;
 
+	// Format version 2 was added in MW 1.40 but relies on PHP serialization
+	//   of ParserOutput which was last supported in MW 1.44.
+	// Format version 3 became the default in MW 1.45
+	public const CURRENT_FORMAT_VERSION = 3;
+	// Used for forward/backward compatibility; set to empty array to disable.
+	// As this is a short term stash (5 minutes) preservation
+	// across upgrades is not expected/guaranteed so long as
+	// CURRENT_FORMAT_VERSION is bumped.
+	public const OTHER_FORMAT_VERSIONS = [];
+
 	/**
 	 * @param BagOStuff $cache
 	 * @param IConnectionProvider $dbProvider
 	 * @param LoggerInterface $logger
-	 * @param StatsdDataFactoryInterface $stats
+	 * @param StatsFactory $stats
 	 * @param UserEditTracker $userEditTracker
 	 * @param UserFactory $userFactory
 	 * @param WikiPageFactory $wikiPageFactory
+	 * @param JsonCodec $jsonCodec
 	 * @param HookContainer $hookContainer
 	 * @param int $initiator Class INITIATOR__* constant
 	 */
 	public function __construct(
-		BagOStuff $cache,
-		IConnectionProvider $dbProvider,
-		LoggerInterface $logger,
-		StatsdDataFactoryInterface $stats,
-		UserEditTracker $userEditTracker,
-		UserFactory $userFactory,
-		WikiPageFactory $wikiPageFactory,
+		private BagOStuff $cache,
+		private IConnectionProvider $dbProvider,
+		private LoggerInterface $logger,
+		private StatsFactory $stats,
+		private UserEditTracker $userEditTracker,
+		private UserFactory $userFactory,
+		private WikiPageFactory $wikiPageFactory,
+		private JsonCodec $jsonCodec,
 		HookContainer $hookContainer,
 		$initiator
 	) {
-		$this->cache = $cache;
-		$this->dbProvider = $dbProvider;
-		$this->logger = $logger;
-		$this->stats = $stats;
-		$this->userEditTracker = $userEditTracker;
-		$this->userFactory = $userFactory;
-		$this->wikiPageFactory = $wikiPageFactory;
 		$this->hookRunner = new HookRunner( $hookContainer );
 		$this->initiator = $initiator;
 	}
@@ -135,7 +113,8 @@ class PageEditStash {
 		}
 
 		$page = $pageUpdater->getPage();
-		$key = $this->getStashKey( $page, $this->getContentHash( $content ), $user );
+		$contentHash = $this->getContentHash( $content );
+		$key = $this->getStashKey( $page, $contentHash, $user );
 		$fname = __METHOD__;
 
 		// Use the primary DB to allow for fast blocking locks on the "save path" where this
@@ -156,23 +135,33 @@ class PageEditStash {
 
 		$cutoffTime = time() - self::PRESUME_FRESH_TTL_SEC;
 
-		// Reuse any freshly build matching edit stash cache
+		// Reuse any freshly built matching edit stash cache
 		$editInfo = $this->getStashValue( $key );
+		// Forward and backward compatibility
+		// @phan-suppress-next-line PhanEmptyForeach
+		foreach ( self::OTHER_FORMAT_VERSIONS as $other_version ) {
+			if ( $editInfo !== false ) {
+				break;
+			}
+			$newKey = $this->getStashKey( $page, $contentHash, $user, $other_version );
+			$editInfo = $this->getStashValue( $newKey );
+		}
 		if ( $editInfo && (int)wfTimestamp( TS_UNIX, $editInfo->timestamp ) >= $cutoffTime ) {
 			$alreadyCached = true;
 		} else {
 			$pageUpdater->setContent( SlotRecord::MAIN, $content );
 
-			$update = $pageUpdater->prepareUpdate( EDIT_INTERNAL ); // applies pre-safe transform
+			$update = $pageUpdater->prepareUpdate( EDIT_INTERNAL ); // applies pre-save transform
 			$output = $update->getCanonicalParserOutput(); // causes content to be parsed
 			$output->setCacheTime( $update->getRevision()->getTimestamp() );
 
 			// emulate a cache value that kind of looks like a PreparedEdit, for use below
-			$editInfo = (object)[
-				'pstContent' => $update->getRawContent( SlotRecord::MAIN ),
-				'output'     => $output,
-				'timestamp'  => $output->getCacheTime()
-			];
+			$editInfo = new PageEditStashContents(
+				pstContent: $update->getRawContent( SlotRecord::MAIN ),
+				output:     $output,
+				timestamp:  $output->getCacheTime(),
+				edits:      $this->userEditTracker->getUserEditCount( $user ),
+			);
 
 			$alreadyCached = false;
 		}
@@ -194,9 +183,7 @@ class PageEditStash {
 
 			$code = $this->storeStashValue(
 				$key,
-				$editInfo->pstContent,
-				$editInfo->output,
-				$editInfo->timestamp,
+				$editInfo,
 				$user
 			);
 
@@ -233,7 +220,7 @@ class PageEditStash {
 	 * The cache is rejected if template or file changes are detected.
 	 * Note that foreign template or file transclusions are not checked.
 	 *
-	 * This returns an object with the following fields:
+	 * This returns a PageEditStashContents object with the following fields:
 	 *   - pstContent: the Content after pre-save-transform
 	 *   - output: the ParserOutput instance
 	 *   - timestamp: the timestamp of the parse
@@ -242,9 +229,12 @@ class PageEditStash {
 	 * @param PageIdentity $page
 	 * @param Content $content
 	 * @param UserIdentity $user to get parser options from
-	 * @return stdClass|false Returns edit stash object or false on cache miss
+	 * @return PageEditStashContents|false Returns edit stash object or
+	 *   false on cache miss
 	 */
-	public function checkCache( PageIdentity $page, Content $content, UserIdentity $user ) {
+	public function checkCache(
+		PageIdentity $page, Content $content, UserIdentity $user
+	): PageEditStashContents|false {
 		$legacyUser = $this->userFactory->newFromUserIdentity( $user );
 		if (
 			// The context is not an HTTP POST request
@@ -260,7 +250,9 @@ class PageEditStash {
 
 		$logger = $this->logger;
 
-		$key = $this->getStashKey( $page, $this->getContentHash( $content ), $user );
+		$contentHash = $this->getContentHash( $content );
+		$key = $this->getStashKey( $page, $contentHash, $user );
+
 		$logContext = [
 			'key' => $key,
 			'title' => (string)$page,
@@ -268,8 +260,20 @@ class PageEditStash {
 		];
 
 		$editInfo = $this->getAndWaitForStashValue( $key );
+		// Forward and backward compatibility
+		// @phan-suppress-next-line PhanEmptyForeach
+		foreach ( self::OTHER_FORMAT_VERSIONS as $other_version ) {
+			if ( $editInfo !== false ) {
+				break;
+			}
+			$newKey = $this->getStashKey( $page, $contentHash, $user, $other_version );
+			// Not "getAndWait" because there shouldn't be anyone actively
+			// generating cache entries from other format versions, they are
+			// just left over from rollforward/rollback.
+			$editInfo = $this->getStashValue( $newKey );
+		}
 		if ( !is_object( $editInfo ) || !$editInfo->output ) {
-			$this->incrStatsByContent( 'cache_misses.no_stash', $content );
+			$this->incrCacheReadStats( 'miss', 'no_stash', $content );
 			if ( $this->recentStashEntryCount( $user ) > 0 ) {
 				$logger->info( "Empty cache for key '{key}' but not for user.", $logContext );
 			} else {
@@ -285,28 +289,28 @@ class PageEditStash {
 		$isCacheUsable = true;
 		if ( $age <= self::PRESUME_FRESH_TTL_SEC ) {
 			// Assume nothing changed in this time
-			$this->incrStatsByContent( 'cache_hits.presumed_fresh', $content );
+			$this->incrCacheReadStats( 'hit', 'presumed_fresh', $content );
 			$logger->debug( "Timestamp-based cache hit for key '{key}'.", $logContext );
 		} elseif ( !$user->isRegistered() ) {
 			$lastEdit = $this->lastEditTime( $user );
 			$cacheTime = $editInfo->output->getCacheTime();
 			if ( $lastEdit < $cacheTime ) {
 				// Logged-out user made no local upload/template edits in the meantime
-				$this->incrStatsByContent( 'cache_hits.presumed_fresh', $content );
+				$this->incrCacheReadStats( 'hit', 'presumed_fresh', $content );
 				$logger->debug( "Edit check based cache hit for key '{key}'.", $logContext );
 			} else {
 				$isCacheUsable = false;
-				$this->incrStatsByContent( 'cache_misses.proven_stale', $content );
+				$this->incrCacheReadStats( 'miss', 'proven_stale', $content );
 				$logger->info( "Stale cache for key '{key}' due to outside edits.", $logContext );
 			}
 		} else {
 			if ( $editInfo->edits === $this->userEditTracker->getUserEditCount( $user ) ) {
 				// Logged-in user made no local upload/template edits in the meantime
-				$this->incrStatsByContent( 'cache_hits.presumed_fresh', $content );
+				$this->incrCacheReadStats( 'hit', 'presumed_fresh', $content );
 				$logger->debug( "Edit count based cache hit for key '{key}'.", $logContext );
 			} else {
 				$isCacheUsable = false;
-				$this->incrStatsByContent( 'cache_misses.proven_stale', $content );
+				$this->incrCacheReadStats( 'miss', 'proven_stale', $content );
 				$logger->info( "Stale cache for key '{key}'due to outside edits.", $logContext );
 			}
 		}
@@ -336,7 +340,7 @@ class PageEditStash {
 			foreach ( $flagsMaybeReparse as $flag ) {
 				if ( $editInfo->output->getOutputFlag( $flag ) ) {
 					$logger->debug(
-						"Cache for key '{key}' has $flag; post-insertion parse possible.",
+						"Cache for key '{key}' has {$flag->value}; post-insertion parse possible.",
 						$logContext
 					);
 				}
@@ -346,24 +350,25 @@ class PageEditStash {
 		return $editInfo;
 	}
 
-	/**
-	 * @param string $subkey
-	 * @param Content $content
-	 */
-	private function incrStatsByContent( $subkey, Content $content ) {
-		$this->stats->increment( 'editstash.' . $subkey ); // overall for b/c
-		$this->stats->increment( 'editstash_by_model.' . $content->getModel() . '.' . $subkey );
+	private function incrCacheReadStats( string $result, string $reason, Content $content ): void {
+		static $subtypeByResult = [ 'miss' => 'cache_misses', 'hit' => 'cache_hits' ];
+		$this->stats->getCounter( "editstash_cache_checks_total" )
+			->setLabel( 'reason', $reason )
+			->setLabel( 'result', $result )
+			->setLabel( 'model', $content->getModel() )
+			->copyToStatsdAt( [
+				'editstash.' . $subtypeByResult[ $result ] . '.' . $reason,
+				'editstash_by_model.' . $content->getModel() . '.' . $subtypeByResult[ $result ] . '.' . $reason ] )
+			->increment();
 	}
 
-	/**
-	 * @param string $key
-	 * @return bool|stdClass
-	 */
-	private function getAndWaitForStashValue( $key ) {
+	private function getAndWaitForStashValue( string $key ): PageEditStashContents|false {
 		$editInfo = $this->getStashValue( $key );
 
 		if ( !$editInfo ) {
-			$start = microtime( true );
+			$timer = $this->stats->getTiming( 'editstash_lock_wait_seconds' )
+				->start();
+
 			// We ignore user aborts and keep parsing. Block on any prior parsing
 			// so as to use its results and make use of the time spent parsing.
 			$dbw = $this->dbProvider->getPrimaryDatabase();
@@ -372,8 +377,7 @@ class PageEditStash {
 				$dbw->unlock( $key, __METHOD__ );
 			}
 
-			$timeMs = 1000 * max( 0, microtime( true ) - $start );
-			$this->stats->timing( 'editstash.lock_wait_time', $timeMs );
+			$timer->stop();
 		}
 
 		return $editInfo;
@@ -381,9 +385,9 @@ class PageEditStash {
 
 	/**
 	 * @param string $textHash
-	 * @return string|bool Text or false if missing
+	 * @return string|false Text or false if missing
 	 */
-	public function fetchInputText( $textHash ) {
+	public function fetchInputText( string $textHash ): string|false {
 		$textKey = $this->cache->makeKey( 'stashedit', 'text', $textHash );
 
 		return $this->cache->get( $textKey );
@@ -394,7 +398,7 @@ class PageEditStash {
 	 * @param string $textHash
 	 * @return bool Success
 	 */
-	public function stashInputText( $text, $textHash ) {
+	public function stashInputText( string $text, string $textHash ): bool {
 		$textKey = $this->cache->makeKey( 'stashedit', 'text', $textHash );
 
 		return $this->cache->set(
@@ -409,7 +413,7 @@ class PageEditStash {
 	 * @param UserIdentity $user
 	 * @return string|null TS_MW timestamp or null
 	 */
-	private function lastEditTime( UserIdentity $user ) {
+	private function lastEditTime( UserIdentity $user ): ?string {
 		$time = $this->dbProvider->getReplicaDatabase()->newSelectQueryBuilder()
 			->select( 'MAX(rc_timestamp)' )
 			->from( 'recentchanges' )
@@ -427,7 +431,7 @@ class PageEditStash {
 	 * @param Content $content
 	 * @return string
 	 */
-	private function getContentHash( Content $content ) {
+	private function getContentHash( Content $content ): string {
 		return sha1( implode( "\n", [
 			$content->getModel(),
 			$content->getDefaultFormat(),
@@ -447,9 +451,14 @@ class PageEditStash {
 	 * @param UserIdentity $user User to get parser options from
 	 * @return string
 	 */
-	private function getStashKey( PageIdentity $page, $contentHash, UserIdentity $user ) {
+	private function getStashKey(
+		PageIdentity $page,
+		string $contentHash,
+		UserIdentity $user,
+		int $version = self::CURRENT_FORMAT_VERSION
+	): string {
 		return $this->cache->makeKey(
-			'stashedit-info-v2',
+			"stashedit-info-v{$version}",
 			md5( "{$page->getNamespace()}\n{$page->getDBkey()}" ),
 			// Account for the edit model/text
 			$contentHash,
@@ -458,14 +467,11 @@ class PageEditStash {
 		);
 	}
 
-	/**
-	 * @param string $key
-	 * @return stdClass|bool Object map (pstContent,output,outputID,timestamp,edits) or false
-	 */
-	private function getStashValue( $key ) {
+	private function getStashValue( string $key ): PageEditStashContents|false {
 		$serial = $this->cache->get( $key );
 
-		return $this->unserializeStashInfo( $serial );
+		return $serial === false ? false :
+			$this->unserializeStashInfo( $serial );
 	}
 
 	/**
@@ -474,19 +480,16 @@ class PageEditStash {
 	 * This makes a simple version of WikiPage::prepareContentForEdit() as stash info
 	 *
 	 * @param string $key
-	 * @param Content $pstContent Pre-Save transformed content
-	 * @param ParserOutput $parserOutput
-	 * @param string $timestamp TS_MW
+	 * @param PageEditStashContents $stashInfo
 	 * @param UserIdentity $user
-	 * @return string|bool True or an error code
+	 * @return string|true True or an error code
 	 */
 	private function storeStashValue(
-		$key,
-		Content $pstContent,
-		ParserOutput $parserOutput,
-		$timestamp,
+		string $key,
+		PageEditStashContents $stashInfo,
 		UserIdentity $user
-	) {
+	): string|bool {
+		$parserOutput = $stashInfo->output;
 		// If an item is renewed, mind the cache TTL determined by config and parser functions.
 		// Put an upper limit on the TTL to avoid extreme template/file staleness.
 		$age = time() - (int)wfTimestamp( TS_UNIX, $parserOutput->getCacheTime() );
@@ -501,12 +504,6 @@ class PageEditStash {
 		}
 
 		// Store what is actually needed and split the output into another key (T204742)
-		$stashInfo = (object)[
-			'pstContent' => $pstContent,
-			'output'     => $parserOutput,
-			'timestamp'  => $timestamp,
-			'edits'      => $this->userEditTracker->getUserEditCount( $user ),
-		];
 		$serial = $this->serializeStashInfo( $stashInfo );
 		if ( $serial === false ) {
 			return 'store_error';
@@ -525,43 +522,38 @@ class PageEditStash {
 	 * @param UserIdentity $user
 	 * @param string $newKey
 	 */
-	private function pruneExcessStashedEntries( UserIdentity $user, $newKey ) {
+	private function pruneExcessStashedEntries( UserIdentity $user, string $newKey ): void {
 		$key = $this->cache->makeKey( 'stash-edit-recent', sha1( $user->getName() ) );
 
 		$keyList = $this->cache->get( $key ) ?: [];
 		if ( count( $keyList ) >= self::MAX_CACHE_RECENT ) {
 			$oldestKey = array_shift( $keyList );
-			$this->cache->delete( $oldestKey, BagOStuff::WRITE_PRUNE_SEGMENTS );
+			$this->cache->delete( $oldestKey, BagOStuff::WRITE_ALLOW_SEGMENTS );
 		}
 
 		$keyList[] = $newKey;
 		$this->cache->set( $key, $keyList, 2 * self::MAX_CACHE_TTL );
 	}
 
-	/**
-	 * @param UserIdentity $user
-	 * @return int
-	 */
-	private function recentStashEntryCount( UserIdentity $user ) {
+	private function recentStashEntryCount( UserIdentity $user ): int {
 		$key = $this->cache->makeKey( 'stash-edit-recent', sha1( $user->getName() ) );
 
 		return count( $this->cache->get( $key ) ?: [] );
 	}
 
-	private function serializeStashInfo( stdClass $stashInfo ) {
-		// @todo: use JSON with ParserOutput and Content
-		return serialize( $stashInfo );
+	private function serializeStashInfo( PageEditStashContents $stashInfo ): string|false {
+		try {
+			return $this->jsonCodec->serialize( $stashInfo );
+		} catch ( JsonException ) {
+			return false;
+		}
 	}
 
-	private function unserializeStashInfo( $serial ) {
-		if ( is_string( $serial ) ) {
-			// @todo: use JSON with ParserOutput and Content
-			$stashInfo = unserialize( $serial );
-			if ( is_object( $stashInfo ) && $stashInfo->output instanceof ParserOutput ) {
-				return $stashInfo;
-			}
+	private function unserializeStashInfo( string $serial ): PageEditStashContents|false {
+		try {
+			return $this->jsonCodec->deserialize( $serial, PageEditStashContents::class );
+		} catch ( JsonException ) {
+			return false;
 		}
-
-		return false;
 	}
 }
