@@ -1,19 +1,6 @@
 <?php
 /**
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
- *
+ * @license GPL-2.0-or-later
  * @file
  * @ingroup Actions
  */
@@ -43,7 +30,9 @@ use MediaWiki\Title\NamespaceInfo;
 use MediaWiki\Title\TitleFactory;
 use MediaWiki\Title\TitleFormatter;
 use MediaWiki\User\Options\UserOptionsLookup;
+use MediaWiki\Watchlist\WatchedItemStore;
 use MediaWiki\Watchlist\WatchlistManager;
+use Wikimedia\ParamValidator\TypeDef\ExpiryDef;
 use Wikimedia\Rdbms\IConnectionProvider;
 use Wikimedia\Rdbms\IDBAccessObject;
 use Wikimedia\Rdbms\ReadOnlyMode;
@@ -70,6 +59,7 @@ class DeleteAction extends FormAction {
 	protected const MSG_EDIT_REASONS_SUPPRESS = 'edit-reasons-suppress';
 
 	protected WatchlistManager $watchlistManager;
+	private WatchedItemStore $watchedItemStore;
 	protected LinkRenderer $linkRenderer;
 	private BacklinkCacheFactory $backlinkCacheFactory;
 	protected ReadOnlyMode $readOnlyMode;
@@ -89,6 +79,7 @@ class DeleteAction extends FormAction {
 		parent::__construct( $article, $context );
 		$services = MediaWikiServices::getInstance();
 		$this->watchlistManager = $services->getWatchlistManager();
+		$this->watchedItemStore = $services->getWatchedItemStore();
 		$this->linkRenderer = $services->getLinkRenderer();
 		$this->backlinkCacheFactory = $services->getBacklinkCacheFactory();
 		$this->readOnlyMode = $services->getReadOnlyMode();
@@ -101,27 +92,33 @@ class DeleteAction extends FormAction {
 		$this->dbProvider = $services->getConnectionProvider();
 	}
 
+	/** @inheritDoc */
 	public function getName() {
 		return 'delete';
 	}
 
+	/** @inheritDoc */
 	public function onSubmit( $data ) {
 		return false;
 	}
 
+	/** @inheritDoc */
 	public function onSuccess() {
 		return false;
 	}
 
+	/** @inheritDoc */
 	protected function usesOOUI() {
 		return true;
 	}
 
+	/** @inheritDoc */
 	protected function getPageTitle() {
 		$title = $this->getTitle();
 		return $this->msg( 'delete-confirm' )->plaintextParams( $title->getPrefixedText() );
 	}
 
+	/** @inheritDoc */
 	public function getRestriction() {
 		return 'delete';
 	}
@@ -198,7 +195,6 @@ class DeleteAction extends FormAction {
 		$suppress = $request->getCheck( 'wpSuppress' ) &&
 			$context->getAuthority()->isAllowed( 'suppressrevision' );
 
-		$context = $this->getContext();
 		$deletePage = $this->deletePageFactory->newDeletePage(
 			$this->getWikiPage(),
 			$context->getAuthority()
@@ -249,7 +245,14 @@ class DeleteAction extends FormAction {
 			$this->showLogEntries();
 		}
 
-		$this->watchlistManager->setWatch( $request->getCheck( 'wpWatch' ), $context->getAuthority(), $title );
+		$expiry = $this->getRequest()->getText( 'wpWatchlistExpiry' );
+
+		if ( $context->getConfig()->get( MainConfigNames::WatchlistExpiry ) && $expiry !== '' ) {
+			$expiry = ExpiryDef::normalizeExpiry( $expiry, TS_ISO_8601 );
+		} else {
+			$expiry = null;
+		}
+		$this->watchlistManager->setWatch( $request->getCheck( 'wpWatch' ), $context->getAuthority(), $title, $expiry );
 	}
 
 	/**
@@ -503,16 +506,37 @@ class DeleteAction extends FormAction {
 			$fields['Watch'] = [
 				'type' => 'check',
 				'id' => 'wpWatch',
+				'infusable' => true,
 				'tabindex' => 4,
 				'default' => $checkWatch,
 				'label-message' => 'watchthis',
 			];
 		}
+
+		$context = $this->getContext();
+		# Add a dropdown for watchlist expiry times in the form, T261229
+		if ( $context->getConfig()->get( MainConfigNames::WatchlistExpiry ) ) {
+			$expiryOptions = WatchAction::getExpiryOptions(
+				$context,
+				$this->watchedItemStore->getWatchedItem( $user, $title )
+			);
+
+			$fields['WatchlistExpiry'] = [
+					'type' => 'select',
+					'name' => 'wpWatchlistExpiry',
+					'id' => 'wpWatchlistExpiry',
+					'tabindex' => 5,
+					'infusable' => true,
+					'label-message' => 'confirm-watch-label',
+					'options' => $expiryOptions['options'],
+			];
+		}
+
 		if ( $this->isSuppressionAllowed() ) {
 			$fields['Suppress'] = [
 				'type' => 'check',
 				'id' => 'wpSuppress',
-				'tabindex' => 5,
+				'tabindex' => 6,
 				'default' => false,
 				'label-message' => 'revdelete-suppress',
 			];
@@ -521,7 +545,7 @@ class DeleteAction extends FormAction {
 		$fields['ConfirmB'] = [
 			'type' => 'submit',
 			'id' => 'wpConfirmB',
-			'tabindex' => 6,
+			'tabindex' => 7,
 			'buttonlabel' => $this->getFormMsg( self::MSG_SUBMIT )->text(),
 			'flags' => [ 'primary', 'destructive' ],
 		];
