@@ -2,38 +2,32 @@
 /**
  * Copyright © 2006 Yuri Astrakhan "<Firstname><Lastname>@gmail.com"
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * http://www.gnu.org/copyleft/gpl.html
- *
+ * @license GPL-2.0-or-later
  * @file
  */
 
+namespace MediaWiki\Api;
+
+use MediaWiki\ChangeTags\ChangeTagsStore;
 use MediaWiki\CommentFormatter\CommentFormatter;
 use MediaWiki\Content\IContentHandlerFactory;
 use MediaWiki\Content\Renderer\ContentRenderer;
 use MediaWiki\Content\Transform\ContentTransformer;
+use MediaWiki\Page\PageIdentity;
 use MediaWiki\ParamValidator\TypeDef\UserDef;
+use MediaWiki\Parser\ParserFactory;
 use MediaWiki\Revision\RevisionRecord;
 use MediaWiki\Revision\RevisionStore;
 use MediaWiki\Revision\SlotRoleRegistry;
+use MediaWiki\Status\Status;
 use MediaWiki\Storage\NameTableAccessException;
 use MediaWiki\Storage\NameTableStore;
 use MediaWiki\Title\Title;
+use MediaWiki\Title\TitleFormatter;
 use MediaWiki\User\ActorMigration;
+use MediaWiki\User\TempUser\TempUserCreator;
+use MediaWiki\User\UserFactory;
 use Wikimedia\ParamValidator\ParamValidator;
-use Wikimedia\Rdbms\Platform\ISQLPlatform;
 
 /**
  * A query action to enumerate revisions of a given page, or show top revisions
@@ -45,40 +39,28 @@ use Wikimedia\Rdbms\Platform\ISQLPlatform;
  */
 class ApiQueryRevisions extends ApiQueryRevisionsBase {
 
-	/** @var RevisionStore */
-	private $revisionStore;
+	private RevisionStore $revisionStore;
+	private NameTableStore $changeTagDefStore;
+	private ChangeTagsStore $changeTagsStore;
+	private ActorMigration $actorMigration;
+	private TitleFormatter $titleFormatter;
 
-	/** @var NameTableStore */
-	private $changeTagDefStore;
-
-	/** @var ActorMigration */
-	private $actorMigration;
-
-	/**
-	 * @param ApiQuery $query
-	 * @param string $moduleName
-	 * @param RevisionStore $revisionStore
-	 * @param IContentHandlerFactory $contentHandlerFactory
-	 * @param ParserFactory $parserFactory
-	 * @param SlotRoleRegistry $slotRoleRegistry
-	 * @param NameTableStore $changeTagDefStore
-	 * @param ActorMigration $actorMigration
-	 * @param ContentRenderer $contentRenderer
-	 * @param ContentTransformer $contentTransformer
-	 * @param CommentFormatter $commentFormatter
-	 */
 	public function __construct(
 		ApiQuery $query,
-		$moduleName,
+		string $moduleName,
 		RevisionStore $revisionStore,
 		IContentHandlerFactory $contentHandlerFactory,
 		ParserFactory $parserFactory,
 		SlotRoleRegistry $slotRoleRegistry,
 		NameTableStore $changeTagDefStore,
+		ChangeTagsStore $changeTagsStore,
 		ActorMigration $actorMigration,
 		ContentRenderer $contentRenderer,
 		ContentTransformer $contentTransformer,
-		CommentFormatter $commentFormatter
+		CommentFormatter $commentFormatter,
+		TempUserCreator $tempUserCreator,
+		UserFactory $userFactory,
+		TitleFormatter $titleFormatter
 	) {
 		parent::__construct(
 			$query,
@@ -90,14 +72,18 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 			$slotRoleRegistry,
 			$contentRenderer,
 			$contentTransformer,
-			$commentFormatter
+			$commentFormatter,
+			$tempUserCreator,
+			$userFactory
 		);
 		$this->revisionStore = $revisionStore;
 		$this->changeTagDefStore = $changeTagDefStore;
+		$this->changeTagsStore = $changeTagsStore;
 		$this->actorMigration = $actorMigration;
+		$this->titleFormatter = $titleFormatter;
 	}
 
-	protected function run( ApiPageSet $resultPageSet = null ) {
+	protected function run( ?ApiPageSet $resultPageSet = null ) {
 		$params = $this->extractRequestParams( false );
 
 		// If any of those parameters are used, work in 'enumeration' mode.
@@ -157,14 +143,13 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 		$useIndex = [];
 		if ( $resultPageSet === null ) {
 			$this->parseParameters( $params );
-			$opts = [ 'page' ];
+			$queryBuilder = $this->revisionStore->newSelectQueryBuilder( $db )
+				->joinComment()
+				->joinPage();
 			if ( $this->fld_user ) {
-				$opts[] = 'user';
+				$queryBuilder->joinUser();
 			}
-			$revQuery = $this->revisionStore->getQueryInfo( $opts );
-			$this->addTables( $revQuery['tables'] );
-			$this->addFields( $revQuery['fields'] );
-			$this->addJoinConds( $revQuery['joins'] );
+			$this->getQueryBuilder()->merge( $queryBuilder );
 		} else {
 			$this->limit = $this->getParameter( 'limit' ) ?: 10;
 			// Always join 'page' so orphaned revisions are filtered out
@@ -178,7 +163,9 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 		}
 
 		if ( $this->fld_tags ) {
-			$this->addFields( [ 'ts_tags' => ChangeTags::makeTagSummarySubquery( 'revision' ) ] );
+			$this->addFields( [
+				'ts_tags' => $this->changeTagsStore->makeTagSummarySubquery( 'revision' )
+			] );
 		}
 
 		if ( $params['tag'] !== null ) {
@@ -188,7 +175,7 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 			);
 			try {
 				$this->addWhereFld( 'ct_tag_id', $this->changeTagDefStore->getId( $params['tag'] ) );
-			} catch ( NameTableAccessException $exception ) {
+			} catch ( NameTableAccessException ) {
 				// Return nothing.
 				$this->addWhere( '1=0' );
 			}
@@ -198,11 +185,14 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 			// For each page we will request, the user must have read rights for that page
 			$status = Status::newGood();
 
-			/** @var Title $title */
-			foreach ( $pageSet->getGoodTitles() as $title ) {
-				if ( !$this->getAuthority()->authorizeRead( 'read', $title ) ) {
+			/** @var PageIdentity $pageIdentity */
+			foreach ( $pageSet->getGoodPages() as $pageIdentity ) {
+				if ( !$this->getAuthority()->authorizeRead( 'read', $pageIdentity ) ) {
 					$status->fatal( ApiMessage::create(
-						[ 'apierror-cannotviewtitle', wfEscapeWikiText( $title->getPrefixedText() ) ],
+						[
+							'apierror-cannotviewtitle',
+							wfEscapeWikiText( $this->titleFormatter->getPrefixedText( $pageIdentity ) ),
+						],
 						'accessdenied'
 					) );
 				}
@@ -245,21 +235,20 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 			}
 			if ( $revids ) {
 				$db = $this->getDB();
-				$sql = $db->unionQueries( [
-					$db->selectSQLText(
-						'revision',
-						[ 'id' => 'rev_id', 'ts' => 'rev_timestamp' ],
-						[ 'rev_id' => $revids ],
-						__METHOD__
-					),
-					$db->selectSQLText(
-						'archive',
-						[ 'id' => 'ar_rev_id', 'ts' => 'ar_timestamp' ],
-						[ 'ar_rev_id' => $revids ],
-						__METHOD__
-					),
-				], $db::UNION_DISTINCT );
-				$res = $db->query( $sql, __METHOD__, ISQLPlatform::QUERY_CHANGE_NONE );
+				$uqb = $db->newUnionQueryBuilder();
+				$uqb->add(
+					$db->newSelectQueryBuilder()
+						->select( [ 'id' => 'rev_id', 'ts' => 'rev_timestamp' ] )
+						->from( 'revision' )
+						->where( [ 'rev_id' => $revids ] )
+				);
+				$uqb->add(
+					$db->newSelectQueryBuilder()
+						->select( [ 'id' => 'ar_rev_id', 'ts' => 'ar_timestamp' ] )
+						->from( 'archive' )
+						->where( [ 'ar_rev_id' => $revids ] )
+				);
+				$res = $uqb->caller( __METHOD__ )->fetchResultSet();
 				foreach ( $res as $row ) {
 					if ( (int)$row->id === (int)$params['startid'] ) {
 						$params['start'] = $row->ts;
@@ -341,8 +330,7 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 				// Paranoia: avoid brute force searches (T19342)
 				if ( !$this->getAuthority()->isAllowed( 'deletedhistory' ) ) {
 					$bitmask = RevisionRecord::DELETED_USER;
-				} elseif ( !$this->getAuthority()->isAllowedAny( 'suppressrevision', 'viewsuppressed' )
-				) {
+				} elseif ( !$this->getAuthority()->isAllowedAny( 'suppressrevision', 'viewsuppressed' ) ) {
 					$bitmask = RevisionRecord::DELETED_USER | RevisionRecord::DELETED_RESTRICTED;
 				} else {
 					$bitmask = 0;
@@ -360,7 +348,9 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 			$this->addWhereFld( 'rev_id', array_keys( $revs ) );
 
 			if ( $params['continue'] !== null ) {
-				$this->addWhere( 'rev_id >= ' . (int)$params['continue'] );
+				$this->addWhere( $db->buildComparison( '>=', [
+					'rev_id' => (int)$params['continue']
+				] ) );
 			}
 			$this->addOption( 'ORDER BY', 'rev_id' );
 		} elseif ( $pageCount > 0 ) {
@@ -448,6 +438,7 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 		}
 	}
 
+	/** @inheritDoc */
 	public function getAllowedParams() {
 		$ret = parent::getAllowedParams() + [
 			'startid' => [
@@ -473,17 +464,21 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 					'older'
 				],
 				ApiBase::PARAM_HELP_MSG => 'api-help-param-direction',
+				ApiBase::PARAM_HELP_MSG_PER_VALUE => [
+					'newer' => 'api-help-paramvalue-direction-newer',
+					'older' => 'api-help-paramvalue-direction-older',
+				],
 				ApiBase::PARAM_HELP_MSG_INFO => [ [ 'singlepageonly' ] ],
 			],
 			'user' => [
 				ParamValidator::PARAM_TYPE => 'user',
-				UserDef::PARAM_ALLOWED_USER_TYPES => [ 'name', 'ip', 'id', 'interwiki' ],
+				UserDef::PARAM_ALLOWED_USER_TYPES => [ 'name', 'ip', 'temp', 'id', 'interwiki' ],
 				UserDef::PARAM_RETURN_OBJECT => true,
 				ApiBase::PARAM_HELP_MSG_INFO => [ [ 'singlepageonly' ] ],
 			],
 			'excludeuser' => [
 				ParamValidator::PARAM_TYPE => 'user',
-				UserDef::PARAM_ALLOWED_USER_TYPES => [ 'name', 'ip', 'id', 'interwiki' ],
+				UserDef::PARAM_ALLOWED_USER_TYPES => [ 'name', 'ip', 'temp', 'id', 'interwiki' ],
 				UserDef::PARAM_RETURN_OBJECT => true,
 				ApiBase::PARAM_HELP_MSG_INFO => [ [ 'singlepageonly' ] ],
 			],
@@ -498,30 +493,38 @@ class ApiQueryRevisions extends ApiQueryRevisionsBase {
 		return $ret;
 	}
 
+	/** @inheritDoc */
 	protected function getExamplesMessages() {
+		$title = Title::newMainPage()->getPrefixedText();
+		$mp = rawurlencode( $title );
+
 		return [
-			'action=query&prop=revisions&titles=API|Main%20Page&' .
+			"action=query&prop=revisions&titles=API|{$mp}&" .
 				'rvslots=*&rvprop=timestamp|user|comment|content'
 				=> 'apihelp-query+revisions-example-content',
-			'action=query&prop=revisions&titles=Main%20Page&rvlimit=5&' .
+			"action=query&prop=revisions&titles={$mp}&rvlimit=5&" .
 				'rvprop=timestamp|user|comment'
 				=> 'apihelp-query+revisions-example-last5',
-			'action=query&prop=revisions&titles=Main%20Page&rvlimit=5&' .
+			"action=query&prop=revisions&titles={$mp}&rvlimit=5&" .
 				'rvprop=timestamp|user|comment&rvdir=newer'
 				=> 'apihelp-query+revisions-example-first5',
-			'action=query&prop=revisions&titles=Main%20Page&rvlimit=5&' .
+			"action=query&prop=revisions&titles={$mp}&rvlimit=5&" .
 				'rvprop=timestamp|user|comment&rvdir=newer&rvstart=2006-05-01T00:00:00Z'
 				=> 'apihelp-query+revisions-example-first5-after',
-			'action=query&prop=revisions&titles=Main%20Page&rvlimit=5&' .
+			"action=query&prop=revisions&titles={$mp}&rvlimit=5&" .
 				'rvprop=timestamp|user|comment&rvexcludeuser=127.0.0.1'
 				=> 'apihelp-query+revisions-example-first5-not-localhost',
-			'action=query&prop=revisions&titles=Main%20Page&rvlimit=5&' .
+			"action=query&prop=revisions&titles={$mp}&rvlimit=5&" .
 				'rvprop=timestamp|user|comment&rvuser=MediaWiki%20default'
 				=> 'apihelp-query+revisions-example-first5-user',
 		];
 	}
 
+	/** @inheritDoc */
 	public function getHelpUrls() {
 		return 'https://www.mediawiki.org/wiki/Special:MyLanguage/API:Revisions';
 	}
 }
+
+/** @deprecated class alias since 1.43 */
+class_alias( ApiQueryRevisions::class, 'ApiQueryRevisions' );
