@@ -2,21 +2,7 @@
 /**
  * Class for blocks stored in the database.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * http://www.gnu.org/copyleft/gpl.html
- *
+ * @license GPL-2.0-or-later
  * @file
  */
 
@@ -28,7 +14,6 @@ use MediaWiki\Block\Restriction\NamespaceRestriction;
 use MediaWiki\Block\Restriction\PageRestriction;
 use MediaWiki\Block\Restriction\Restriction;
 use MediaWiki\Html\Html;
-use MediaWiki\MainConfigNames;
 use MediaWiki\MediaWikiServices;
 use MediaWiki\Title\Title;
 use MediaWiki\User\UserIdentity;
@@ -614,12 +599,9 @@ class DatabaseBlock extends AbstractBlock {
 	 * @inheritDoc
 	 */
 	public function appliesToRight( $right ) {
-		// Temporarily access service container until the feature flag is removed: T280532
-		$config = MediaWikiServices::getInstance()->getMainConfig();
-
 		$res = parent::appliesToRight( $right );
 
-		if ( !$res && $config->get( MainConfigNames::EnablePartialActionBlocks ) ) {
+		if ( !$res ) {
 			$blockActions = MediaWikiServices::getInstance()->getBlockActionInfo()
 				->getAllBlockActions();
 
