@@ -1,20 +1,6 @@
 <?php
 /**
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * http://www.gnu.org/copyleft/gpl.html
- *
+ * @license GPL-2.0-or-later
  * @since 1.18
  *
  * @author Alexandre Emsenhuber
@@ -41,7 +27,6 @@ use MediaWiki\Request\FauxRequest;
 use MediaWiki\Request\WebRequest;
 use MediaWiki\Session\CsrfTokenSet;
 use MediaWiki\Session\PHPSessionHandler;
-use MediaWiki\Session\SessionManager;
 use MediaWiki\Skin\Skin;
 use MediaWiki\StubObject\StubGlobalUser;
 use MediaWiki\Title\Title;
@@ -377,7 +362,7 @@ class RequestContext implements IContextSource, MutableContext {
 	}
 
 	public function hasUser(): bool {
-		if ( !defined( 'MW_PHPUNIT_TEST' ) && !defined( 'MW_PARSER_TEST' ) ) {
+		if ( !defined( 'MW_PHPUNIT_TEST' ) ) {
 			throw new LogicException( __METHOD__ . '() should be called only from tests!' );
 		}
 		return $this->user !== null;
@@ -632,7 +617,7 @@ class RequestContext implements IContextSource, MutableContext {
 	 * Resets singleton returned by getMain(). Should be called only from unit tests.
 	 */
 	public static function resetMain() {
-		if ( !defined( 'MW_PHPUNIT_TEST' ) && !defined( 'MW_PARSER_TEST' ) ) {
+		if ( !defined( 'MW_PHPUNIT_TEST' ) ) {
 			throw new LogicException( __METHOD__ . '() should be called only from unit tests!' );
 		}
 		self::$instance = null;
@@ -646,7 +631,7 @@ class RequestContext implements IContextSource, MutableContext {
 	 * @since 1.21
 	 */
 	public function exportSession() {
-		$session = SessionManager::getGlobalSession();
+		$session = $this->getRequest()->getSession();
 		return [
 			'ip' => $this->getRequest()->getIP(),
 			'headers' => $this->getRequest()->getAllHeaders(),
@@ -670,7 +655,7 @@ class RequestContext implements IContextSource, MutableContext {
 	 * error conditions arise.
 	 *
 	 * This is useful when background scripts inherit context when acting on
-	 * behalf of a user. In general the 'sessionId' parameter should be set
+	 * behalf of a user. In general, the 'sessionId' parameter should be set
 	 * to an empty string unless session importing is *truly* needed. This
 	 * feature is somewhat deprecated.
 	 *
@@ -680,16 +665,17 @@ class RequestContext implements IContextSource, MutableContext {
 	 */
 	public static function importScopedSession( array $params ) {
 		if ( $params['sessionId'] !== '' &&
-			SessionManager::getGlobalSession()->isPersistent()
+			self::getMain()->getRequest()->getSession()->isPersistent()
 		) {
 			// Check to avoid sending random cookies for the wrong users.
-			// This method should only called by CLI scripts or by HTTP job runners.
+			// This method should only be called by CLI scripts or by HTTP job runners.
 			throw new BadMethodCallException( "Sessions can only be imported when none is active." );
 		} elseif ( !IPUtils::isValid( $params['ip'] ) ) {
 			throw new InvalidArgumentException( "Invalid client IP address '{$params['ip']}'." );
 		}
 
-		$userFactory = MediaWikiServices::getInstance()->getUserFactory();
+		$services = MediaWikiServices::getInstance();
+		$userFactory = $services->getUserFactory();
 
 		if ( $params['userId'] ) { // logged-in user
 			$user = $userFactory->newFromId( (int)$params['userId'] );
@@ -701,7 +687,7 @@ class RequestContext implements IContextSource, MutableContext {
 			$user = $userFactory->newFromName( $params['ip'], UserRigorOptions::RIGOR_NONE );
 		}
 
-		$importSessionFunc = static function ( User $user, array $params ) {
+		$importSessionFunc = static function ( User $user, array $params ) use ( $services ) {
 			global $wgRequest;
 
 			$context = RequestContext::getMain();
@@ -716,7 +702,7 @@ class RequestContext implements IContextSource, MutableContext {
 			// Get new session, if applicable
 			$session = null;
 			if ( $params['sessionId'] !== '' ) { // don't make a new random ID
-				$manager = SessionManager::singleton();
+				$manager = $services->getSessionManager();
 				$session = $manager->getSessionById( $params['sessionId'], true )
 					?: $manager->getEmptySession();
 			}
