@@ -1,20 +1,6 @@
 <?php
 /**
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * http://www.gnu.org/copyleft/gpl.html
- *
+ * @license GPL-2.0-or-later
  * @file
  */
 
@@ -167,7 +153,7 @@ class SiteStatsUpdate implements DeferrableUpdate, MergeableUpdate {
 					} else {
 						$dbw->newInsertQueryBuilder()
 							->insertInto( 'site_stats' )
-							->row( array_merge( [ 'ss_row_id' => $shard ], $initValues ) )
+							->row( $initValues + [ 'ss_row_id' => $shard ] )
 							->onDuplicateKeyUpdate()
 							->uniqueIndexFields( [ 'ss_row_id' ] )
 							->set( $set )
@@ -189,15 +175,16 @@ class SiteStatsUpdate implements DeferrableUpdate, MergeableUpdate {
 		$services = MediaWikiServices::getInstance();
 		$config = $services->getMainConfig();
 
+		$rcl = $services->getRecentChangeLookup();
 		$dbr = $services->getConnectionProvider()->getReplicaDatabase( false, 'vslow' );
-		# Get non-bot users than did some recent action other than making accounts.
+		# Get non-bot users that did some recent action other than making accounts.
 		# If account creation is included, the number gets inflated ~20+ fold on enwiki.
 		$activeUsers = $dbr->newSelectQueryBuilder()
 			->select( 'COUNT(DISTINCT rc_actor)' )
 			->from( 'recentchanges' )
 			->join( 'actor', 'actor', 'actor_id=rc_actor' )
 			->where( [
-				$dbr->expr( 'rc_type', '!=', RC_EXTERNAL ), // Exclude external (Wikidata)
+				$dbr->expr( 'rc_source', '=', $rcl->getPrimarySources() ),
 				$dbr->expr( 'actor_user', '!=', null ),
 				$dbr->expr( 'rc_bot', '=', 0 ),
 				$dbr->expr( 'rc_log_type', '!=', 'newusers' )->or( 'rc_log_type', '=', null ),
