@@ -1,30 +1,16 @@
 <?php
 /**
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * http://www.gnu.org/copyleft/gpl.html
- *
+ * @license GPL-2.0-or-later
  * @file
  */
 
-namespace MediaWiki\Languages;
+namespace MediaWiki\Language;
 
 use InvalidArgumentException;
 use MediaWiki\Config\ServiceOptions;
 use MediaWiki\HookContainer\HookContainer;
 use MediaWiki\HookContainer\HookRunner;
-use MediaWiki\Language\LanguageCode;
+use MediaWiki\Languages\Data\Names;
 use MediaWiki\MainConfigNames;
 use MediaWiki\Title\TitleParser;
 use Wikimedia\ObjectCache\BagOStuff;
@@ -169,7 +155,7 @@ class LanguageNameUtils {
 			return false;
 		}
 
-		if ( isset( Data\Names::NAMES[$tag] ) || $this->getLanguageName( $tag, $tag ) !== '' ) {
+		if ( isset( Names::NAMES[$tag] ) || $this->getLanguageName( $tag, $tag ) !== '' ) {
 			return true;
 		}
 
@@ -228,7 +214,7 @@ class LanguageNameUtils {
 			$this->hookRunner->onLanguageGetTranslatedLanguageNames( $names, $inLanguage );
 		}
 
-		$mwNames = $this->options->get( MainConfigNames::ExtraLanguageNames ) + Data\Names::NAMES;
+		$mwNames = $this->options->get( MainConfigNames::ExtraLanguageNames ) + Names::NAMES;
 		if ( !$this->options->get( MainConfigNames::UsePigLatinVariant ) ) {
 			// Suppress Pig Latin unless explicitly enabled.
 			unset( $mwNames['en-x-piglatin'] );
@@ -333,3 +319,6 @@ class LanguageNameUtils {
 		return "$IP/languages/i18n/$code.json";
 	}
 }
+
+/** @deprecated class alias since 1.45 */
+class_alias( LanguageNameUtils::class, 'MediaWiki\\Languages\\LanguageNameUtils' );
