@@ -2,10 +2,12 @@
 
 namespace Wikimedia\WRStats;
 
+use RuntimeException;
+
 /**
  * Exception class for errors thrown by the WRStats library
  *
  * @since 1.39
  */
-class WRStatsError extends \Exception {
+class WRStatsError extends RuntimeException {
 }
