@@ -1,20 +1,6 @@
 <?php
 /**
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * http://www.gnu.org/copyleft/gpl.html
- *
+ * @license GPL-2.0-or-later
  * @file
  */
 namespace Wikimedia\Rdbms;
@@ -74,8 +60,6 @@ class LoadBalancer implements ILoadBalancerForOwner {
 
 	/** @var array[] $aliases Map of (table => (dbname, schema, prefix) map) */
 	private $tableAliases = [];
-	/** @var string[] Map of (index alias => index) */
-	private $indexAliases = [];
 	/** @var DatabaseDomain[]|string[] Map of (domain alias => DB domain) */
 	private $domainAliases = [];
 	/** @var callable[] Map of (name => callable) */
@@ -108,6 +92,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 
 	/** @var DatabaseDomain[] Map of (domain ID => domain instance) */
 	private $nonLocalDomainCache = [];
+	private ?string $uniqueIdentifier = null;
 
 	/**
 	 * @var int Modification counter for invalidating connections held by
@@ -234,6 +219,11 @@ class LoadBalancer implements ILoadBalancerForOwner {
 
 		$group = $params['defaultGroup'] ?? self::GROUP_GENERIC;
 		$this->defaultGroup = isset( $this->groupLoads[ $group ] ) ? $group : self::GROUP_GENERIC;
+		if ( empty( $params['shuffleSharding'] ) ) {
+			$this->uniqueIdentifier = null;
+		} else {
+			$this->uniqueIdentifier = $params['uniqueIdentifier'] ?? null;
+		}
 	}
 
 	private static function newTrackedConnectionsArray(): array {
@@ -252,6 +242,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return $this->localDomain->getId();
 	}
 
+	/** @inheritDoc */
 	public function resolveDomainID( $domain ): string {
 		return $this->resolveDomainInstance( $domain )->getId();
 	}
@@ -298,10 +289,16 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		if ( $groups === [] || $groups === false || $groups === $this->defaultGroup ) {
 			$resolvedGroup = $this->defaultGroup;
 		} elseif ( is_string( $groups ) ) {
+			if ( $groups !== 'dump' && $groups !== 'vslow' ) {
+				wfDeprecatedMsg( 'Asking for a replica from groups except dump/vslow is deprecated: ' . $groups );
+			}
 			$resolvedGroup = isset( $this->groupLoads[$groups] ) ? $groups : $this->defaultGroup;
 		} elseif ( is_array( $groups ) ) {
 			$resolvedGroup = $this->defaultGroup;
 			foreach ( $groups as $group ) {
+				if ( $group !== 'dump' && $group !== 'vslow' ) {
+					wfDeprecatedMsg( 'Asking for a replica from groups except dump/vslow is deprecated: ' . $group );
+				}
 				if ( isset( $this->groupLoads[$group] ) ) {
 					$resolvedGroup = $group;
 					break;
@@ -384,6 +381,11 @@ class LoadBalancer implements ILoadBalancerForOwner {
 	private function getRandomNonLagged( array $loads, $sessionLagLimit = INF ) {
 		$lags = $this->getLagTimes();
 
+		$dbs = [];
+		foreach ( $loads as $i => $load ) {
+			$srvName = $this->serverInfo->getServerName( $i );
+			$dbs[$srvName] = $i;
+		}
 		// Unset excessively lagged servers from the load group
 		foreach ( $lags as $i => $lag ) {
 			if ( $i !== ServerInfo::WRITER_INDEX ) {
@@ -399,6 +401,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 						[ 'db_server' => $srvName ]
 					);
 					unset( $loads[$i] );
+					unset( $dbs[$srvName] );
 				} elseif ( $lag > $maxServerLag ) {
 					$this->logger->debug(
 						__METHOD__ .
@@ -406,6 +409,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 						[ 'db_server' => $srvName, 'lag' => $lag, 'maxlag' => $maxServerLag ]
 					);
 					unset( $loads[$i] );
+					unset( $dbs[$srvName] );
 				}
 			}
 		}
@@ -417,9 +421,37 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		}
 
 		// Return a server index based on weighted random selection
-		return ArrayUtils::pickRandom( $loads );
+		return $this->shuffleSharding( $dbs, $loads );
 	}
 
+	/**
+	 * Apply shuffle sharding. Idea originally from Amazon's Route 53.
+	 *
+	 * It basically used a unique identifier from the request (IP is good enough) and uses hashing
+	 * to pick a consistent set of 2 or 3 from the pool. Then using randomness, picks one at random.
+	 *
+	 * It provides many benefits and makes the system more robust against issues or attacks.
+	 *
+	 * @param array $dbs Map of server name => server index
+	 * @param array $loads Map of (server index => weight) for a load group
+	 * @return int Index of the server
+	 */
+	private function shuffleSharding( $dbs, $loads ) {
+		// shuffle sharding doesn't make sense in small groups
+		if ( count( $loads ) <= 3 || $this->uniqueIdentifier === null ) {
+			return ArrayUtils::pickRandom( $loads );
+		}
+
+		ArrayUtils::consistentHashSort( $dbs, $this->uniqueIdentifier );
+		$keys = array_values( $dbs );
+		return ArrayUtils::pickRandom( [
+			$keys[0] => $loads[$keys[0]],
+			$keys[1] => $loads[$keys[1]],
+			$keys[2] => $loads[$keys[2]],
+		] );
+	}
+
+	/** @inheritDoc */
 	public function getReaderIndex( $group = false ) {
 		$group = is_string( $group ) ? $group : self::GROUP_GENERIC;
 
@@ -559,6 +591,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return $i;
 	}
 
+	/** @inheritDoc */
 	public function waitForAll( DBPrimaryPos $pos, $timeout = null ) {
 		$timeout = $timeout ?: self::MAX_WAIT_DEFAULT;
 
@@ -614,6 +647,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return false;
 	}
 
+	/** @inheritDoc */
 	public function getAnyOpenConnection( $i, $flags = 0 ) {
 		$i = ( $i === self::DB_PRIMARY ) ? ServerInfo::WRITER_INDEX : $i;
 		$conn = false;
@@ -749,6 +783,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return $ok;
 	}
 
+	/** @inheritDoc */
 	public function getConnection( $i, $groups = [], $domain = false, $flags = 0 ) {
 		if ( self::fieldHasBit( $flags, self::CONN_SILENCE_ERRORS ) ) {
 			throw new UnexpectedValueException(
@@ -764,6 +799,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return new DBConnRef( $this, [ $i, $groups, $domain, $flags ], $role, $this->modcount );
 	}
 
+	/** @inheritDoc */
 	public function getConnectionInternal( $i, $groups = [], $domain = false, $flags = 0 ): IDatabase {
 		$domain = $this->resolveDomainID( $domain );
 		$group = $this->resolveGroups( $groups, $i );
@@ -791,6 +827,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return $this->getServerConnection( $serverIndex, $domain, $flags );
 	}
 
+	/** @inheritDoc */
 	public function getServerConnection( $i, $domain, $flags = 0 ) {
 		$domainInstance = DatabaseDomain::newFromId( $domain );
 		// Number of connections made before getting the server index and handle
@@ -829,15 +866,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return $conn;
 	}
 
-	/**
-	 * @deprecated since 1.39.
-	 */
-	public function getConnectionRef( $i, $groups = [], $domain = false, $flags = 0 ): DBConnRef {
-		wfDeprecated( __METHOD__, '1.39' );
-		// @phan-suppress-next-line PhanTypeMismatchReturnSuperType to be removed soon
-		return $this->getConnection( $i, $groups, $domain, $flags );
-	}
-
+	/** @inheritDoc */
 	public function getMaintenanceConnectionRef(
 		$i,
 		$groups = [],
@@ -971,6 +1000,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		}
 	}
 
+	/** @inheritDoc */
 	public function getServerAttributes( $i ) {
 		return $this->databaseFactory->attributesFromType(
 			$this->getServerType( $i ),
@@ -1044,9 +1074,8 @@ class LoadBalancer implements ILoadBalancerForOwner {
 			] ),
 			Database::NEW_UNCONNECTED
 		);
-		// Set alternative table/index names before any queries can be issued
+		// Set alternative table names before any queries can be issued
 		$conn->setTableAliases( $this->tableAliases );
-		$conn->setIndexAliases( $this->indexAliases );
 		// Account for any active transaction round and listeners
 		$this->syncConnectionRoundState( $conn );
 		if ( $i === ServerInfo::WRITER_INDEX ) {
@@ -1059,7 +1088,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		try {
 			$conn->initConnection();
 			++$this->connectionCounter;
-		} catch ( DBConnectionError $e ) {
+		} catch ( DBConnectionError ) {
 			$this->lastErrorConn = $conn;
 			// ignore; let the DB handle the logging
 		}
@@ -1126,7 +1155,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 	 * @throws DBConnectionError
 	 * @return never
 	 */
-	private function reportConnectionError( $extraLbError = '' ) {
+	private function reportConnectionError( $extraLbError = '' ): never {
 		if ( $this->lastErrorConn instanceof IDatabaseForOwner ) {
 			$srvName = $this->lastErrorConn->getServerName();
 			$lastDbError = $this->lastErrorConn->lastError() ?: 'unknown error';
@@ -1170,30 +1199,37 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		throw $exception;
 	}
 
+	/** @inheritDoc */
 	public function getServerCount() {
 		return $this->serverInfo->getServerCount();
 	}
 
+	/** @inheritDoc */
 	public function hasReplicaServers() {
 		return $this->serverInfo->hasReplicaServers();
 	}
 
+	/** @inheritDoc */
 	public function hasStreamingReplicaServers() {
 		return $this->serverInfo->hasStreamingReplicaServers();
 	}
 
+	/** @inheritDoc */
 	public function getServerName( $i ): string {
 		return $this->serverInfo->getServerName( $i );
 	}
 
+	/** @inheritDoc */
 	public function getServerInfo( $i ) {
 		return $this->serverInfo->getServerInfo( $i );
 	}
 
+	/** @inheritDoc */
 	public function getServerType( $i ) {
 		return $this->serverInfo->getServerType( $i );
 	}
 
+	/** @inheritDoc */
 	public function getPrimaryPos() {
 		$conn = $this->getAnyOpenConnection( ServerInfo::WRITER_INDEX );
 		if ( $conn ) {
@@ -1201,7 +1237,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		}
 
 		/** @var IDatabaseForOwner|null $conn */
-		$conn = $this->getConnectionInternal( ServerInfo::WRITER_INDEX, self::CONN_SILENCE_ERRORS );
+		$conn = $this->getConnectionInternal( ServerInfo::WRITER_INDEX, [], false, self::CONN_SILENCE_ERRORS );
 		// @phan-suppress-next-line PhanRedundantCondition
 		if ( !$conn ) {
 			$this->reportConnectionError();
@@ -1285,11 +1321,13 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		}
 	}
 
+	/** @inheritDoc */
 	public function disable( $fname = __METHOD__ ) {
 		$this->closeAll( $fname );
 		$this->disabled = true;
 	}
 
+	/** @inheritDoc */
 	public function closeAll( $fname = __METHOD__ ) {
 		/** @noinspection PhpUnusedLocalVariableInspection */
 		$scope = ScopedCallback::newScopedIgnoreUserAbort();
@@ -1344,6 +1382,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		$conn->close( __METHOD__ );
 	}
 
+	/** @inheritDoc */
 	public function finalizePrimaryChanges( $fname = __METHOD__ ) {
 		$this->assertTransactionRoundStage( [ self::ROUND_CURSORY, self::ROUND_FINALIZED ] );
 		/** @noinspection PhpUnusedLocalVariableInspection */
@@ -1370,6 +1409,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return $total;
 	}
 
+	/** @inheritDoc */
 	public function approvePrimaryChanges( int $maxWriteDuration, $fname = __METHOD__ ) {
 		$this->assertTransactionRoundStage( self::ROUND_FINALIZED );
 		/** @noinspection PhpUnusedLocalVariableInspection */
@@ -1424,6 +1464,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		$this->trxRoundStage = self::ROUND_APPROVED;
 	}
 
+	/** @inheritDoc */
 	public function beginPrimaryChanges( $fname = __METHOD__ ) {
 		if ( $this->trxRoundFname !== null ) {
 			throw new DBTransactionError(
@@ -1450,6 +1491,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		$this->trxRoundStage = self::ROUND_CURSORY;
 	}
 
+	/** @inheritDoc */
 	public function commitPrimaryChanges( $fname = __METHOD__ ) {
 		$this->assertTransactionRoundStage( self::ROUND_APPROVED );
 		/** @noinspection PhpUnusedLocalVariableInspection */
@@ -1482,6 +1524,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		$this->trxRoundStage = self::ROUND_COMMIT_CALLBACKS;
 	}
 
+	/** @inheritDoc */
 	public function runPrimaryTransactionIdleCallbacks( $fname = __METHOD__ ) {
 		if ( $this->trxRoundStage === self::ROUND_COMMIT_CALLBACKS ) {
 			$type = IDatabase::TRIGGER_COMMIT;
@@ -1529,6 +1572,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 						)
 					);
 					$this->statsFactory->getCounter( 'rdbms_callback_writes_found_total' )
+						->setLabel( 'db_cluster', $this->getClusterName() )
 						->setLabel( 'db_server', $conn->getServerName() )
 						->increment();
 				} elseif ( $conn->trxLevel() ) {
@@ -1549,6 +1593,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return $errors[0] ?? null;
 	}
 
+	/** @inheritDoc */
 	public function runPrimaryTransactionListenerCallbacks( $fname = __METHOD__ ) {
 		if ( $this->trxRoundStage === self::ROUND_COMMIT_CALLBACKS ) {
 			$type = IDatabase::TRIGGER_COMMIT;
@@ -1573,6 +1618,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return $errors[0] ?? null;
 	}
 
+	/** @inheritDoc */
 	public function rollbackPrimaryChanges( $fname = __METHOD__ ) {
 		/** @noinspection PhpUnusedLocalVariableInspection */
 		$scope = ScopedCallback::newScopedIgnoreUserAbort();
@@ -1589,6 +1635,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		$this->trxRoundStage = self::ROUND_ROLLBACK_CALLBACKS;
 	}
 
+	/** @inheritDoc */
 	public function flushPrimarySessions( $fname = __METHOD__ ) {
 		$this->assertTransactionRoundStage( [ self::ROUND_CURSORY ] );
 		if ( $this->hasPrimaryChanges() ) {
@@ -1662,16 +1709,17 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		// server configuration arrays. Such handles will still be flushed during calls
 		// to {@link LoadBalancer::commitPrimaryChanges()}.
 		if ( $conn->getFlag( $conn::DBO_TRX ) ) {
-			// DB handle is participating in the active transction round
+			// DB handle is participating in the active transaction round
 			$conn->setLBInfo( $conn::LB_TRX_ROUND_LEVEL, $trxRoundLevel );
 			$conn->setLBInfo( $conn::LB_TRX_ROUND_FNAME, $this->trxRoundFname );
 		} else {
-			// DB handle is not participating in any transction rounds
+			// DB handle is not participating in any transaction rounds
 			$conn->setLBInfo( $conn::LB_TRX_ROUND_LEVEL, 0 );
 			$conn->setLBInfo( $conn::LB_TRX_ROUND_FNAME, null );
 		}
 	}
 
+	/** @inheritDoc */
 	public function flushReplicaSnapshots( $fname = __METHOD__ ) {
 		foreach ( $this->conns as $poolConnsByServer ) {
 			foreach ( $poolConnsByServer as $serverIndex => $serverConns ) {
@@ -1685,16 +1733,19 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		}
 	}
 
+	/** @inheritDoc */
 	public function flushPrimarySnapshots( $fname = __METHOD__ ) {
 		foreach ( $this->getOpenPrimaryConnections() as $conn ) {
 			$conn->flushSnapshot( $fname );
 		}
 	}
 
+	/** @inheritDoc */
 	public function hasPrimaryConnection() {
 		return (bool)$this->getAnyOpenConnection( ServerInfo::WRITER_INDEX );
 	}
 
+	/** @inheritDoc */
 	public function hasPrimaryChanges() {
 		foreach ( $this->getOpenPrimaryConnections() as $conn ) {
 			if ( $conn->writesOrCallbacksPending() ) {
@@ -1705,6 +1756,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return false;
 	}
 
+	/** @inheritDoc */
 	public function lastPrimaryChangeTimestamp() {
 		$lastTime = null;
 		foreach ( $this->getOpenPrimaryConnections() as $conn ) {
@@ -1714,6 +1766,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return $lastTime;
 	}
 
+	/** @inheritDoc */
 	public function hasOrMadeRecentPrimaryChanges( $age = null ) {
 		$age ??= self::MAX_WAIT_DEFAULT;
 
@@ -1721,6 +1774,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 			|| $this->lastPrimaryChangeTimestamp() > microtime( true ) - $age );
 	}
 
+	/** @inheritDoc */
 	public function pendingPrimaryChangeCallers() {
 		$fnames = [];
 		foreach ( $this->getOpenPrimaryConnections() as $conn ) {
@@ -1730,6 +1784,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return $fnames;
 	}
 
+	/** @inheritDoc */
 	public function explicitTrxActive() {
 		foreach ( $this->getOpenPrimaryConnections() as $conn ) {
 			if ( $conn->explicitTrxActive() ) {
@@ -1744,10 +1799,12 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		$this->logger->warning( __METHOD__ . ": setting lagged replica mode" );
 	}
 
+	/** @inheritDoc */
 	public function laggedReplicaUsed() {
 		return $this->laggedReplicaMode;
 	}
 
+	/** @inheritDoc */
 	public function getReadOnlyReason() {
 		if ( $this->readOnlyReason !== false ) {
 			return $this->readOnlyReason;
@@ -1782,7 +1839,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 				if ( $conn ) {
 					try {
 						$value = (int)$conn->serverIsReadOnly();
-					} catch ( DBError $e ) {
+					} catch ( DBError ) {
 						$value = is_int( $oldValue ) ? $oldValue : 0;
 					}
 				} else {
@@ -1799,6 +1856,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		);
 	}
 
+	/** @inheritDoc */
 	public function pingAll() {
 		$success = true;
 		foreach ( $this->getOpenConnections() as $conn ) {
@@ -1837,6 +1895,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		}
 	}
 
+	/** @inheritDoc */
 	public function getMaxLag() {
 		$host = '';
 		$maxLag = -1;
@@ -1858,6 +1917,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return [ $host, $maxLag, $maxIndex ];
 	}
 
+	/** @inheritDoc */
 	public function getLagTimes() {
 		if ( !$this->hasReplicaServers() ) {
 			return [ ServerInfo::WRITER_INDEX => 0 ]; // no replication = no lag
@@ -1888,6 +1948,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		);
 	}
 
+	/** @inheritDoc */
 	public function waitForPrimaryPos( IDatabase $conn ) {
 		if ( $conn->getLBInfo( self::INFO_SERVER_INDEX ) === ServerInfo::WRITER_INDEX ) {
 			return true; // not a replica DB server
@@ -1930,6 +1991,7 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		return $ok;
 	}
 
+	/** @inheritDoc */
 	public function setTransactionListener( $name, ?callable $callback = null ) {
 		if ( $callback ) {
 			$this->trxRecurringCallbacks[$name] = $callback;
@@ -1945,14 +2007,11 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		$this->tableAliases = $aliases;
 	}
 
-	public function setIndexAliases( array $aliases ) {
-		$this->indexAliases = $aliases;
-	}
-
 	public function setDomainAliases( array $aliases ) {
 		$this->domainAliases = $aliases;
 	}
 
+	/** @inheritDoc */
 	public function setLocalDomainPrefix( $prefix ) {
 		$oldLocalDomain = $this->localDomain;
 		$this->localDomain = new DatabaseDomain(
@@ -1970,11 +2029,13 @@ class LoadBalancer implements ILoadBalancerForOwner {
 		}
 	}
 
+	/** @inheritDoc */
 	public function redefineLocalDomain( $domain ) {
 		$this->closeAll( __METHOD__ );
 		$this->localDomain = DatabaseDomain::newFromId( $domain );
 	}
 
+	/** @inheritDoc */
 	public function setTempTablesOnlyMode( $value, $domain ) {
 		$old = $this->tempTablesOnlyMode[$domain] ?? false;
 		if ( $value ) {
@@ -2012,13 +2073,10 @@ class LoadBalancer implements ILoadBalancerForOwner {
 	 * @return array
 	 */
 	protected function getConnLogContext( IDatabase $conn, array $extras = [] ) {
-		return array_merge(
-			[
-				'db_server' => $conn->getServerName(),
-				'db_domain' => $conn->getDomainID()
-			],
-			$extras
-		);
+		return $extras + [
+			'db_server' => $conn->getServerName(),
+			'db_domain' => $conn->getDomainID(),
+		];
 	}
 
 	/**
@@ -2029,4 +2087,8 @@ class LoadBalancer implements ILoadBalancerForOwner {
 	public function setMockTime( &$time ) {
 		$this->loadMonitor->setMockTime( $time );
 	}
+
+	public function setDefaultGroupName( string $defaultGroup ): void {
+		$this->defaultGroup = $defaultGroup;
+	}
 }
