@@ -2,21 +2,7 @@
 /**
  * Contains a class for dealing with manual log entries
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * http://www.gnu.org/copyleft/gpl.html
- *
+ * @license GPL-2.0-or-later
  * @file
  * @author Niklas Laxström
  * @license GPL-2.0-or-later
@@ -297,7 +283,7 @@ class ManualLogEntry extends LogEntryBase implements Taggable {
 	/**
 	 * Set the bot flag in the recent changes to this value.
 	 *
-	 * @since 1.40
+	 * @since 1.40 (also backported to 1.39.14)
 	 * @param bool $forceBotFlag
 	 */
 	public function setForceBotFlag( bool $forceBotFlag ): void {
@@ -390,13 +376,15 @@ class ManualLogEntry extends LogEntryBase implements Taggable {
 	 * @since 1.23
 	 */
 	public function getRecentChange( $newId = 0 ) {
-		$formatter = MediaWikiServices::getInstance()->getLogFormatterFactory()->newFromEntry( $this );
+		$services = MediaWikiServices::getInstance();
+		$recentChangeStore = $services->getRecentChangeStore();
+		$formatter = $services->getLogFormatterFactory()->newFromEntry( $this );
 		$context = RequestContext::newExtraneousContext( $this->getTarget() );
 		$formatter->setContext( $context );
 
 		$logpage = SpecialPage::getTitleFor( 'Log', $this->getType() );
 
-		return RecentChange::newLogEntry(
+		return $recentChangeStore->createLogRecentChange(
 			$this->getTimestamp(),
 			$logpage,
 			$this->getPerformerIdentity(),
