@@ -2,7 +2,7 @@
 
 namespace MediaWiki\Hook;
 
-use MailAddress;
+use MediaWiki\Mail\MailAddress;
 
 /**
  * This is a hook handler interface, see docs/Hooks.md.
