@@ -1,20 +1,6 @@
 <?php
 /**
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * http://www.gnu.org/copyleft/gpl.html
- *
+ * @license GPL-2.0-or-later
  * @file
  */
 
@@ -104,7 +90,7 @@ class ChangesList extends ContextSource {
 	protected $formattedComments;
 
 	/**
-	 * @var ChangesListFilterGroup[]
+	 * @var ChangesListFilterGroupContainer
 	 */
 	protected $filterGroups;
 
@@ -124,13 +110,13 @@ class ChangesList extends ContextSource {
 
 	/**
 	 * @param IContextSource $context
-	 * @param ChangesListFilterGroup[] $filterGroups Array of ChangesListFilterGroup objects (currently optional)
+	 * @param ChangesListFilterGroupContainer|null $filterGroups
 	 */
-	public function __construct( $context, array $filterGroups = [] ) {
+	public function __construct( $context, ?ChangesListFilterGroupContainer $filterGroups = null ) {
 		$this->setContext( $context );
 		$this->preCacheMessages();
 		$this->watchMsgCache = new MapCacheLRU( 50 );
-		$this->filterGroups = $filterGroups;
+		$this->filterGroups = $filterGroups ?? new ChangesListFilterGroupContainer();
 
 		$services = MediaWikiServices::getInstance();
 		$this->linkRenderer = $services->getLinkRenderer();
@@ -146,14 +132,18 @@ class ChangesList extends ContextSource {
 	 * Some users might want to use an enhanced list format, for instance
 	 *
 	 * @param IContextSource $context
-	 * @param array $groups Array of ChangesListFilterGroup objects (currently optional)
+	 * @param ChangesListFilterGroupContainer|null $groups
 	 * @return ChangesList
 	 */
-	public static function newFromContext( IContextSource $context, array $groups = [] ) {
+	public static function newFromContext(
+		IContextSource $context,
+		?ChangesListFilterGroupContainer $groups = null
+	) {
 		$user = $context->getUser();
 		$sk = $context->getSkin();
 		$services = MediaWikiServices::getInstance();
 		$list = null;
+		$groups ??= new ChangesListFilterGroupContainer();
 		if ( ( new HookRunner( $services->getHookContainer() ) )->onFetchChangesList( $user, $sk, $list, $groups ) ) {
 			$userOptionsLookup = $services->getUserOptionsLookup();
 			$new = $context->getRequest()->getBool(
@@ -313,11 +303,7 @@ class ChangesList extends ContextSource {
 			( $nsInfo->isTalk( $rc->mAttribs['rc_namespace'] ) ? 'talk' : 'subject' )
 		);
 
-		foreach ( $this->filterGroups as $filterGroup ) {
-			foreach ( $filterGroup->getFilters() as $filter ) {
-				$filter->applyCssClassIfNeeded( $this, $rc, $classes );
-			}
-		}
+		$this->filterGroups->applyCssClassIfNeeded( $this->getContext(), $rc, $classes );
 
 		return $classes;
 	}
@@ -579,8 +565,8 @@ class ChangesList extends ContextSource {
 	public function insertDiffHist( &$s, &$rc, $unpatrolled = null ) {
 		# Diff link
 		if (
-			$rc->mAttribs['rc_type'] == RC_NEW ||
-			$rc->mAttribs['rc_type'] == RC_LOG
+			$rc->mAttribs['rc_source'] === RecentChange::SRC_NEW ||
+			$rc->mAttribs['rc_source'] === RecentChange::SRC_LOG
 		) {
 			$diffLink = $this->message['diff'];
 		} elseif ( !self::userCan( $rc, RevisionRecord::DELETED_TEXT, $this->getAuthority() ) ) {
@@ -884,7 +870,7 @@ class ChangesList extends ContextSource {
 	public static function userCan( $rc, $field, ?Authority $performer = null ) {
 		$performer ??= RequestContext::getMain()->getAuthority();
 
-		if ( $rc->mAttribs['rc_type'] == RC_LOG ) {
+		if ( $rc->mAttribs['rc_source'] === RecentChange::SRC_LOG ) {
 			return LogEventsList::userCanBitfield( $rc->mAttribs['rc_deleted'], $field, $performer );
 		}
 
@@ -920,11 +906,10 @@ class ChangesList extends ContextSource {
 	 *
 	 * @param string &$s
 	 * @param RecentChange &$rc
-	 *
 	 */
 	private function insertPageTools( &$s, &$rc ) {
 		// FIXME Some page tools (e.g. thanks) might make sense for log entries.
-		if ( !in_array( $rc->mAttribs['rc_type'], [ RC_EDIT, RC_NEW ] )
+		if ( !in_array( $rc->mAttribs['rc_source'], [ RecentChange::SRC_EDIT, RecentChange::SRC_NEW ] )
 			// FIXME When would either of these not exist when type is RC_EDIT? Document.
 			|| !$rc->mAttribs['rc_this_oldid']
 			|| !$rc->mAttribs['rc_cur_id']
@@ -948,7 +933,7 @@ class ChangesList extends ContextSource {
 			null,
 			// only show a rollback link on the top-most revision
 			$rc->getAttribute( 'page_latest' ) == $rc->mAttribs['rc_this_oldid']
-				&& $rc->mAttribs['rc_type'] != RC_NEW,
+				&& $rc->mAttribs['rc_source'] !== RecentChange::SRC_NEW,
 			$this->getHookRunner(),
 			$title,
 			$this->getContext(),
@@ -1013,10 +998,18 @@ class ChangesList extends ContextSource {
 		return $s;
 	}
 
+	/**
+	 * @param string &$s
+	 * @param RecentChange &$rc
+	 * @param string[] &$classes
+	 */
 	public function insertExtra( &$s, &$rc, &$classes ) {
 		// Empty, used for subclasses to add anything special.
 	}
 
+	/**
+	 * @return bool
+	 */
 	protected function showAsUnpatrolled( RecentChange $rc ) {
 		return self::isUnpatrolled( $rc, $this->getUser() );
 	}
@@ -1029,11 +1022,11 @@ class ChangesList extends ContextSource {
 	public static function isUnpatrolled( $rc, User $user ) {
 		if ( $rc instanceof RecentChange ) {
 			$isPatrolled = $rc->mAttribs['rc_patrolled'];
-			$rcType = $rc->mAttribs['rc_type'];
+			$rcSource = $rc->mAttribs['rc_source'];
 			$rcLogType = $rc->mAttribs['rc_log_type'];
 		} else {
 			$isPatrolled = $rc->rc_patrolled;
-			$rcType = $rc->rc_type;
+			$rcSource = $rc->rc_source;
 			$rcLogType = $rc->rc_log_type;
 		}
 
@@ -1042,7 +1035,7 @@ class ChangesList extends ContextSource {
 		}
 
 		return $user->useRCPatrol() ||
-			( $rcType == RC_NEW && $user->useNPPatrol() ) ||
+			( $rcSource === RecentChange::SRC_NEW && $user->useNPPatrol() ) ||
 			( $rcLogType === 'upload' && $user->useFilePatrol() );
 	}
 
@@ -1056,7 +1049,7 @@ class ChangesList extends ContextSource {
 	 * @return bool
 	 */
 	protected function isCategorizationWithoutRevision( $rcObj ) {
-		return intval( $rcObj->getAttribute( 'rc_type' ) ) === RC_CATEGORIZE
+		return $rcObj->getAttribute( 'rc_source' ) === RecentChange::SRC_CATEGORIZE
 			&& intval( $rcObj->getAttribute( 'rc_this_oldid' ) ) === 0;
 	}
 
@@ -1068,8 +1061,8 @@ class ChangesList extends ContextSource {
 	protected function getDataAttributes( RecentChange $rc ) {
 		$attrs = [];
 
-		$type = $rc->getAttribute( 'rc_source' );
-		switch ( $type ) {
+		$source = $rc->getAttribute( 'rc_source' );
+		switch ( $source ) {
 			case RecentChange::SRC_EDIT:
 			case RecentChange::SRC_CATEGORIZE:
 			case RecentChange::SRC_NEW:
