@@ -2,21 +2,7 @@
 /**
  * Basic search engine highlighting
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- * http://www.gnu.org/copyleft/gpl.html
- *
+ * @license GPL-2.0-or-later
  * @file
  * @ingroup Search
  */
@@ -165,7 +151,7 @@ class SearchHighlighter {
 			if ( preg_match( '/[\x80-\xff]/', $term ) ) {
 				$terms[$index] = preg_replace_callback(
 					'/./us',
-					[ $this, 'caseCallback' ],
+					$this->caseCallback( ... ),
 					$terms[$index]
 				);
 			} else {
@@ -483,7 +469,7 @@ class SearchHighlighter {
 		$text = preg_replace( "/\\[\\[([^|]+?)\\]\\]/", "\\1", $text );
 		$text = preg_replace_callback(
 			"/\\[\\[([^|]+\\|)(.*?)\\]\\]/",
-			[ $this, 'linkReplace' ],
+			$this->linkReplace( ... ),
 			$text
 		);
 		$text = preg_replace( "/<\/?[^>]+>/", "", $text );
