@@ -4,6 +4,7 @@ namespace MediaWiki\Specials;
 
 use MediaWiki\Auth\AuthManager;
 use MediaWiki\MainConfigNames;
+use MediaWiki\Session\SessionManager;
 
 /**
  * Special change to remove credentials (such as a two-factor token).
@@ -20,15 +21,17 @@ class SpecialRemoveCredentials extends SpecialChangeCredentials {
 	/** @inheritDoc */
 	protected static $loadUserData = false;
 
-	public function __construct( AuthManager $authManager ) {
-		parent::__construct( $authManager );
+	public function __construct( AuthManager $authManager, SessionManager $sessionManager ) {
+		parent::__construct( $authManager, $sessionManager );
 		$this->mName = 'RemoveCredentials';
 	}
 
+	/** @inheritDoc */
 	protected function getDefaultAction( $subPage ) {
 		return AuthManager::ACTION_REMOVE;
 	}
 
+	/** @inheritDoc */
 	protected function getRequestBlacklist() {
 		return $this->getConfig()->get( MainConfigNames::RemoveCredentialsBlacklist );
 	}
