@@ -1,10 +1,8 @@
 <?php
 /** Kinyarwanda (Ikinyarwanda)
  *
- * To improve a translation please visit https://translatewiki.net
- *
- * @ingroup Language
  * @file
+ * @ingroup Languages
  *
  * @author Nimusenge Priscille
  * @author Amir E. Aharoni
@@ -20,7 +18,7 @@ $namespaceNames = [
 	NS_FILE             => 'Dosiye',
 	NS_FILE_TALK        => 'Ibiganiro_kuri_dosiye',
 	NS_MEDIAWIKI        => 'MediyaWiki',
-	NS_MEDIAWIKI_TALK   => 'Ibiganiro kuri MediyaWiki',
+	NS_MEDIAWIKI_TALK   => 'Ibiganiro_kuri_MediyaWiki',
 	NS_TEMPLATE         => 'Inyandikorugero',
 	NS_TEMPLATE_TALK    => 'Ibiganiro_ku_nyandikorugero',
 	NS_HELP             => 'Ubufasha',
