@@ -275,7 +275,7 @@ $magicWords = [
 	'newsectionlink'            => [ '1', '__NYTTAVSNITTLÄNK__', '__NEWSECTIONLINK__' ],
 	'noeditsection'             => [ '0', '__INTEREDIGERASEKTION__', '__NOEDITSECTION__' ],
 	'nogallery'                 => [ '0', '__INGETGALLERI__', '__NOGALLERY__' ],
-	'noindex'                   => [ '1', '__INTEINDEXERA_', '__NOINDEX__' ],
+	'noindex'                   => [ '1', '__INTEINDEXERA__', '__INTEINDEXERA_', '__NOINDEX__' ],
 	'notoc'                     => [ '0', '__INGENINNEHÅLLSFÖRTECKNING__', '__NOTOC__' ],
 	'ns'                        => [ '0', 'NR:', 'NS:' ],
 	'numberingroup'             => [ '1', 'ANTALIGRUPP', 'NUMBERINGROUP', 'NUMINGROUP' ],
