@@ -20,7 +20,6 @@ use MediaWiki\Logger\LoggingContext;
 use MediaWiki\MainConfigNames;
 use MediaWiki\MediaWikiServices;
 use MediaWiki\Page\PageIdentity;
-use MediaWiki\Page\ProperPageIdentity;
 use MediaWiki\Page\WikiPage;
 use MediaWiki\Parser\ParserOptions;
 use MediaWiki\Permissions\Authority;
@@ -209,6 +208,12 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 	 */
 	private static array $dbDataOnceTables = [];
 
+	// State variables holding data associated with DB connections.
+
+	private static WeakMap $originalTablePrefixes;
+	private static WeakMap $curTestClasses;
+	private static WeakMap $activeSchemaOverrides;
+
 	/**
 	 * @stable to call
 	 * @param string|null $name
@@ -537,7 +542,6 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 		}
 
 		$wgRequest = RequestContext::getMain()->getRequest();
-		MediaWiki\Session\SessionManager::resetCache();
 
 		TestUserRegistry::clear();
 		LoggerFactory::setContext( new LoggingContext() );
@@ -555,11 +559,11 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 
 		$class = static::class;
 
-		$hasDataForTestClass = DynamicPropertyTestHelper::getDynamicProperty( $db, 'hasDataForTestClass' );
+		$hasDataForTestClass = self::$curTestClasses[$db] ?? null;
 
 		$first = $hasDataForTestClass !== $class;
 
-		DynamicPropertyTestHelper::setDynamicProperty( $db, 'hasDataForTestClass', $class );
+		self::$curTestClasses[$db] = $class;
 		return $first;
 	}
 
@@ -779,7 +783,6 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 			session_id( '' );
 		}
 		$wgRequest = RequestContext::getMain()->getRequest();
-		MediaWiki\Session\SessionManager::resetCache();
 		ProfilingContext::destroySingleton();
 
 		// If anything changed the content language, we need to
@@ -803,6 +806,17 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 
 		self::restoreMwServices();
 		$this->localServices = null;
+
+		// Reset context user, which is probably 127.0.0.1, as its loaded
+		// data is probably not valid. This used to manipulate $wgUser but
+		// since that is deprecated tests are more likely to be relying on
+		// RequestContext::getMain() instead.
+		// @todo Should we start setting the user to something nondeterministic
+		//  to encourage tests to be updated to not depend on it?
+		$user = RequestContext::getMain()->getUser();
+		// This has to happen after restoreMwServices(), as it depends on various services actually
+		// working and not being poorly mocked, e.g. SessionManager.
+		$user->clearInstanceCache( $user->mFrom );
 	}
 
 	/**
@@ -846,7 +860,6 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 	 * @param object|callable $service The service instance, or a callable that returns the service instance.
 	 *
 	 * @since 1.27
-	 *
 	 */
 	protected function setService( string $name, $service ) {
 		if ( !$this->localServices ) {
@@ -1113,7 +1126,7 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 				} else {
 					try {
 						$this->mwGlobals[$globalKey] = unserialize( serialize( $GLOBALS[$globalKey] ) );
-					} catch ( Exception $e ) {
+					} catch ( Exception ) {
 						$this->mwGlobals[$globalKey] = $GLOBALS[$globalKey];
 					}
 				}
@@ -1406,7 +1419,7 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 	 * called from inside a test case, a data provider, or a setUp or tearDown method.
 	 *
 	 * @return bool true if the original service locator was restored,
-	 *         false if there was nothing too do.
+	 *         false if there was nothing to do.
 	 */
 	public static function restoreMwServices() {
 		if ( !self::$originalServices ) {
@@ -1620,7 +1633,7 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 	 * Should be called from addDBData().
 	 *
 	 * @since 1.25 ($namespace in 1.28)
-	 * @param string|Title $title Page name or title
+	 * @param string|LinkTarget|PageIdentity $title Page name string or title object
 	 * @param string $text Page's content
 	 * @param int|null $namespace Namespace id (name cannot already contain namespace)
 	 * @param User|null $user If null, static::getTestSysop()->getUser() is used.
@@ -1636,21 +1649,30 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 			throw new RuntimeException( 'When testing with pages, the test must use @group Database.' );
 		}
 
-		if ( is_string( $title ) ) {
-			$title = Title::newFromText( $title, $namespace );
+		// If the first parameter isn't a Title, convert it to a Title.
+		// Note that Title implements PageIdentity, so passing in a Title is handled by this
+		// first condition.
+		if ( $title instanceof PageIdentity ) {
+			$titleObject = Title::newFromPageIdentity( $title );
+		} elseif ( $title instanceof LinkTarget ) {
+			$titleObject = Title::newFromLinkTarget( $title );
+		} elseif ( is_string( $title ) ) {
+			$titleObject = Title::newFromText( $title, $namespace );
+		} else {
+			throw new InvalidArgumentException(
+				'Title must be a string, LinkTarget, or PageIdentity, ' . get_debug_type( $title ) . ' given'
+			);
 		}
 
 		$user ??= static::getTestSysop()->getUser();
-		$comment = __METHOD__ . ': Sample page for unit test.';
-
-		$page = $this->getServiceContainer()->getWikiPageFactory()->newFromTitle( $title );
-		$status = $page->doUserEditContent( ContentHandler::makeContent( $text, $title ), $user, $comment );
+		$editSummary = __METHOD__ . ': Sample page for unit test.';
+		$page = $this->getServiceContainer()->getWikiPageFactory()->newFromTitle( $titleObject );
+		$status = $page->doUserEditContent( ContentHandler::makeContent( $text, $titleObject ), $user, $editSummary );
 		if ( !$status->isOK() ) {
 			$this->fail( $status->getWikiText() );
 		}
-
 		return [
-			'title' => $title,
+			'title' => $titleObject,
 			'id' => $page->getId(),
 		];
 	}
@@ -1738,7 +1760,7 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 		$prefix = null
 	) {
 		$prefix ??= self::dbPrefix();
-		$originalTablePrefix = DynamicPropertyTestHelper::getDynamicProperty( $db, 'originalTablePrefix' );
+		$originalTablePrefix = self::$originalTablePrefixes[$db] ?? null;
 
 		if ( $originalTablePrefix !== null ) {
 			return null;
@@ -1757,7 +1779,7 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 		$dbClone->cloneTableStructure();
 
 		$db->tablePrefix( $prefix );
-		DynamicPropertyTestHelper::setDynamicProperty( $db, 'originalTablePrefix', $oldPrefix );
+		self::$originalTablePrefixes[$db] = $oldPrefix;
 
 		$lb = MediaWikiServices::getInstance()->getDBLoadBalancer();
 		$lb->setTempTablesOnlyMode( self::$useTemporaryTables, $db->getDomainID() );
@@ -1769,6 +1791,12 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 
 		self::$oldTablePrefix = $wgDBprefix;
 
+		// Initialize connection state variables here, as this method may be invoked
+		// from outside the MediaWikiIntegrationTestCase class hierarchy.
+		self::$originalTablePrefixes ??= new WeakMap();
+		self::$curTestClasses ??= new WeakMap();
+		self::$activeSchemaOverrides ??= new WeakMap();
+
 		$testPrefix ??= self::dbPrefix();
 
 		// switch to a temporary clone of the database
@@ -1970,11 +1998,11 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 	 */
 	private function setUpSchema( IMaintainableDatabase $db ) {
 		// Undo any active overrides.
-		$oldOverrides = DynamicPropertyTestHelper::getDynamicProperty( $db, 'activeSchemaOverrides' ) ?? self::SCHEMA_OVERRIDE_DEFAULTS;
+		$oldOverrides = self::$activeSchemaOverrides[$db] ?? self::SCHEMA_OVERRIDE_DEFAULTS;
 
 		if ( $oldOverrides['alter'] || $oldOverrides['create'] || $oldOverrides['drop'] ) {
 			$this->undoSchemaOverrides( $db, $oldOverrides );
-			DynamicPropertyTestHelper::unsetDynamicProperty( $db, 'activeSchemaOverrides' );
+			unset( self::$activeSchemaOverrides[$db] );
 		}
 
 		// Determine new overrides.
@@ -2024,7 +2052,7 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 			$db->sourceFile( $script, null, null, __METHOD__, $inputCallback );
 		}
 
-		DynamicPropertyTestHelper::setDynamicProperty( $db, 'activeSchemaOverrides', $overrides );
+		self::$activeSchemaOverrides[$db] = $overrides;
 	}
 
 	/**
@@ -2049,7 +2077,7 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 	 * @return array
 	 */
 	private static function listOriginalTables( IMaintainableDatabase $db ) {
-		$originalTablePrefix = DynamicPropertyTestHelper::getDynamicProperty( $db, 'originalTablePrefix' );
+		$originalTablePrefix = self::$originalTablePrefixes[$db] ?? null;
 		if ( $originalTablePrefix === null ) {
 			throw new LogicException( 'No original table prefix know, cannot list tables!' );
 		}
@@ -2087,7 +2115,7 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 	private function recloneMockTables( IMaintainableDatabase $db, array $tables ) {
 		self::ensureMockDatabaseConnection( $db );
 
-		$originalTablePrefix = DynamicPropertyTestHelper::getDynamicProperty( $db, 'originalTablePrefix' );
+		$originalTablePrefix = self::$originalTablePrefixes[$db] ?? null;
 
 		if ( $originalTablePrefix === null ) {
 			throw new LogicException( 'No original table prefix know, cannot restore tables!' );
@@ -2117,15 +2145,6 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 
 		if ( in_array( 'user', $tablesUsed ) ) {
 			TestUserRegistry::clear();
-
-			// Reset context user, which is probably 127.0.0.1, as its loaded
-			// data is probably not valid. This used to manipulate $wgUser but
-			// since that is deprecated tests are more likely to be relying on
-			// RequestContext::getMain() instead.
-			// @todo Should we start setting the user to something nondeterministic
-			//  to encourage tests to be updated to not depend on it?
-			$user = RequestContext::getMain()->getUser();
-			$user->clearInstanceCache( $user->mFrom );
 		}
 
 		self::truncateTables( $tablesUsed, $db );
@@ -2424,42 +2443,39 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 	 *
 	 * @return int The ID of the wikitext Namespace
 	 */
-	protected function getDefaultWikitextNS() {
-		global $wgNamespaceContentModels;
-
+	protected function getDefaultWikitextNS(): int {
 		static $wikitextNS = null; // this is not going to change
 		if ( $wikitextNS !== null ) {
 			return $wikitextNS;
 		}
 
 		// quickly short out on the most common case:
-		if ( !isset( $wgNamespaceContentModels[NS_MAIN] ) ) {
-			return NS_MAIN;
+		if ( $this->isWikitextNS( NS_MAIN ) ) {
+			$wikitextNS = NS_MAIN;
+			return $wikitextNS;
 		}
 
 		// NOTE: prefer content namespaces
 		$nsInfo = MediaWikiServices::getInstance()->getNamespaceInfo();
-		$namespaces = array_unique( array_merge(
-			$nsInfo->getContentNamespaces(),
-			[ NS_MAIN, NS_HELP, NS_PROJECT ], // prefer these
-			$nsInfo->getValidNamespaces()
-		) );
-
-		$namespaces = array_diff( $namespaces, [
-			NS_FILE, NS_CATEGORY, NS_MEDIAWIKI, NS_USER // don't mess with magic namespaces
+		$namespaces = array_unique( [
+			...$nsInfo->getContentNamespaces(),
+			// prefer these
+			NS_HELP,
+			NS_PROJECT,
+			// prefer non-talk pages
+			...array_filter( $nsInfo->getValidNamespaces(), static fn ( $i ) => !$nsInfo->isTalk( $i ) ),
+			...$nsInfo->getValidNamespaces(),
 		] );
 
-		$talk = array_filter( $namespaces, static function ( $ns ) use ( $nsInfo ) {
-			return $nsInfo->isTalk( $ns );
-		} );
-
-		// prefer non-talk pages
-		$namespaces = array_diff( $namespaces, $talk );
-		$namespaces = array_merge( $namespaces, $talk );
-
 		// check the default content model of each namespace
 		foreach ( $namespaces as $ns ) {
-			if ( $this->isWikitextNS( $ns ) ) {
+			// don't mess with magic namespaces
+			if ( $ns !== NS_USER &&
+				$ns !== NS_FILE &&
+				$ns !== NS_MEDIAWIKI &&
+				$ns !== NS_CATEGORY &&
+				$this->isWikitextNS( $ns )
+			) {
 				$wikitextNS = $ns;
 				return $wikitextNS;
 			}
@@ -2587,21 +2603,6 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 		}
 	}
 
-	/**
-	 * Remove a temporary hook previously added with setTemporaryHook().
-	 *
-	 * @note This is implemented to remove ALL handlers for the given hook
-	 *       for the duration of the current test case.
-	 * @deprecated since 1.36, use clearHook() instead, hard deprecated
-	 *    since 1.44.
-	 *
-	 * @param string $hookName
-	 */
-	protected function removeTemporaryHook( $hookName ) {
-		wfDeprecated( __METHOD__, '1.36' );
-		$this->clearHook( $hookName );
-	}
-
 	/**
 	 * Edits or creates a page/revision. This method requires database support, which can be enabled with
 	 * "@group Database".
@@ -2625,18 +2626,8 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 		}
 
 		$services = $this->getServiceContainer();
-		if ( $page instanceof WikiPage ) {
-			$title = $page->getTitle();
-		} elseif ( $page instanceof PageIdentity ) {
-			$page = $services->getWikiPageFactory()->newFromTitle( $page );
-			$title = $page->getTitle();
-		} elseif ( $page instanceof LinkTarget ) {
-			$page = $services->getWikiPageFactory()->newFromLinkTarget( $page );
-			$title = $page->getTitle();
-		} else {
-			$title = $services->getTitleFactory()->newFromText( $page, $defaultNs );
-			$page = $services->getWikiPageFactory()->newFromTitle( $title );
-		}
+		$page = $this->makeWikiPage( $page, $defaultNs );
+		$title = $page->getTitle();
 
 		if ( is_string( $content ) ) {
 			$content = $services->getContentHandlerFactory()
@@ -2652,11 +2643,33 @@ abstract class MediaWikiIntegrationTestCase extends PHPUnit\Framework\TestCase {
 	}
 
 	/**
-	 * @param ProperPageIdentity $page
+	 * Make a WikiPage from various types of thing
+	 *
+	 * @param string|PageIdentity|LinkTarget|WikiPage $page
+	 * @param int $defaultNs
+	 * @return WikiPage
+	 */
+	private function makeWikiPage( $page, $defaultNs = NS_MAIN ) {
+		$services = $this->getServiceContainer();
+		if ( $page instanceof WikiPage ) {
+			return $page;
+		} elseif ( $page instanceof PageIdentity ) {
+			return $services->getWikiPageFactory()->newFromTitle( $page );
+		} elseif ( $page instanceof LinkTarget ) {
+			return $services->getWikiPageFactory()->newFromLinkTarget( $page );
+		} else {
+			$title = $services->getTitleFactory()->newFromText( $page, $defaultNs );
+			return $services->getWikiPageFactory()->newFromTitle( $title );
+		}
+	}
+
+	/**
+	 * @param string|PageIdentity|LinkTarget|WikiPage $page
 	 * @param string $summary
 	 * @param Authority|null $deleter
 	 */
-	protected function deletePage( ProperPageIdentity $page, string $summary = '', ?Authority $deleter = null ): void {
+	protected function deletePage( $page, string $summary = '', ?Authority $deleter = null ): void {
+		$page = $this->makeWikiPage( $page );
 		$deleter ??= new UltimateAuthority( new UserIdentityValue( 0, 'MediaWiki default' ) );
 		MediaWikiServices::getInstance()->getDeletePageFactory()
 			->newDeletePage( $page, $deleter )
