@@ -74,7 +74,7 @@ CREATE TABLE /*_*/log_search (
 CREATE INDEX ls_log_id ON /*_*/log_search (ls_log_id);
 CREATE TABLE /*_*/change_tag (
   ct_id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
-  ct_rc_id INTEGER UNSIGNED DEFAULT NULL,
+  ct_rc_id BIGINT UNSIGNED DEFAULT NULL,
   ct_log_id INTEGER UNSIGNED DEFAULT NULL,
   ct_rev_id INTEGER UNSIGNED DEFAULT NULL,
   ct_params BLOB DEFAULT NULL, ct_tag_id INTEGER UNSIGNED NOT NULL
@@ -114,27 +114,25 @@ CREATE TABLE /*_*/redirect (
 CREATE INDEX rd_ns_title ON /*_*/redirect (rd_namespace, rd_title, rd_from);
 CREATE TABLE /*_*/pagelinks (
   pl_from INTEGER UNSIGNED DEFAULT 0 NOT NULL,
-  pl_namespace INTEGER DEFAULT 0 NOT NULL,
-  pl_title BLOB DEFAULT '' NOT NULL,
+  pl_target_id BIGINT UNSIGNED NOT NULL,
   pl_from_namespace INTEGER DEFAULT 0 NOT NULL,
-  PRIMARY KEY(pl_from, pl_namespace, pl_title)
+  PRIMARY KEY(pl_from, pl_target_id)
 );
-CREATE INDEX pl_namespace ON /*_*/pagelinks (pl_namespace, pl_title, pl_from);
-CREATE INDEX pl_backlinks_namespace ON /*_*/pagelinks (
-  pl_from_namespace, pl_namespace,
-  pl_title, pl_from
+CREATE INDEX pl_target_id ON /*_*/pagelinks (pl_target_id, pl_from);
+CREATE INDEX pl_backlinks_namespace_target_id ON /*_*/pagelinks (
+  pl_from_namespace, pl_target_id,
+  pl_from
 );
 CREATE TABLE /*_*/templatelinks (
   tl_from INTEGER UNSIGNED DEFAULT 0 NOT NULL,
-  tl_namespace INTEGER DEFAULT 0 NOT NULL,
-  tl_title BLOB DEFAULT '' NOT NULL,
+  tl_target_id BIGINT UNSIGNED NOT NULL,
   tl_from_namespace INTEGER DEFAULT 0 NOT NULL,
-  PRIMARY KEY(tl_from, tl_namespace, tl_title)
+  PRIMARY KEY(tl_from, tl_target_id)
 );
-CREATE INDEX tl_namespace ON /*_*/templatelinks (tl_namespace, tl_title, tl_from);
-CREATE INDEX tl_backlinks_namespace ON /*_*/templatelinks (
-  tl_from_namespace, tl_namespace,
-  tl_title, tl_from
+CREATE INDEX tl_target_id ON /*_*/templatelinks (tl_target_id, tl_from);
+CREATE INDEX tl_backlinks_namespace_target_id ON /*_*/templatelinks (
+  tl_from_namespace, tl_target_id,
+  tl_from
 );
 CREATE TABLE /*_*/imagelinks (
   il_from INTEGER UNSIGNED DEFAULT 0 NOT NULL,
@@ -160,7 +158,6 @@ CREATE TABLE /*_*/iwlinks (
   PRIMARY KEY(iwl_from, iwl_prefix, iwl_title)
 );
 CREATE INDEX iwl_prefix_title_from ON /*_*/iwlinks (iwl_prefix, iwl_title, iwl_from);
-CREATE INDEX iwl_prefix_from_title ON /*_*/iwlinks (iwl_prefix, iwl_from, iwl_title);
 CREATE TABLE /*_*/category (
   cat_id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
   cat_title BLOB NOT NULL, cat_pages INTEGER DEFAULT 0 NOT NULL,
@@ -184,9 +181,9 @@ CREATE UNIQUE INDEX ctd_name ON /*_*/change_tag_def (ctd_name);
 CREATE INDEX ctd_count ON /*_*/change_tag_def (ctd_count);
 CREATE INDEX ctd_user_defined ON /*_*/change_tag_def (ctd_user_defined);
 CREATE TABLE /*_*/ipblocks_restrictions (
-  ir_ipb_id INTEGER NOT NULL,
+  ir_ipb_id INTEGER UNSIGNED NOT NULL,
   ir_type SMALLINT NOT NULL,
-  ir_value INTEGER NOT NULL,
+  ir_value INTEGER UNSIGNED NOT NULL,
   PRIMARY KEY(ir_ipb_id, ir_type, ir_value)
 );
 CREATE INDEX ir_type_value ON /*_*/ipblocks_restrictions (ir_type, ir_value);
@@ -212,10 +209,9 @@ CREATE INDEX qcc_titletwo ON /*_*/querycachetwo (
 );
 CREATE TABLE /*_*/page_restrictions (
   pr_id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
-  pr_page INTEGER NOT NULL, pr_type BLOB NOT NULL,
-  pr_level BLOB NOT NULL, pr_cascade SMALLINT NOT NULL,
-  pr_user INTEGER UNSIGNED DEFAULT NULL,
-  pr_expiry BLOB DEFAULT NULL
+  pr_page INTEGER UNSIGNED NOT NULL,
+  pr_type BLOB NOT NULL, pr_level BLOB NOT NULL,
+  pr_cascade SMALLINT NOT NULL, pr_expiry BLOB DEFAULT NULL
 );
 CREATE UNIQUE INDEX pr_pagetype ON /*_*/page_restrictions (pr_page, pr_type);
 CREATE INDEX pr_typelevel ON /*_*/page_restrictions (pr_type, pr_level);
@@ -259,13 +255,6 @@ CREATE TABLE /*_*/sites (
   site_config BLOB NOT NULL
 );
 CREATE UNIQUE INDEX site_global_key ON /*_*/sites (site_global_key);
-CREATE INDEX site_type ON /*_*/sites (site_type);
-CREATE INDEX site_group ON /*_*/sites (site_group);
-CREATE INDEX site_source ON /*_*/sites (site_source);
-CREATE INDEX site_language ON /*_*/sites (site_language);
-CREATE INDEX site_protocol ON /*_*/sites (site_protocol);
-CREATE INDEX site_domain ON /*_*/sites (site_domain);
-CREATE INDEX site_forward ON /*_*/sites (site_forward);
 CREATE TABLE /*_*/user_newtalk (
   user_id INTEGER UNSIGNED DEFAULT 0 NOT NULL,
   user_ip BLOB DEFAULT '' NOT NULL, user_last_timestamp BLOB DEFAULT NULL
@@ -295,14 +284,11 @@ CREATE INDEX pt_timestamp ON /*_*/protected_titles (pt_timestamp);
 CREATE TABLE /*_*/externallinks (
   el_id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
   el_from INTEGER UNSIGNED DEFAULT 0 NOT NULL,
-  el_to BLOB NOT NULL, el_index BLOB NOT NULL,
-  el_index_60 BLOB NOT NULL
+  el_to_domain_index BLOB DEFAULT '' NOT NULL,
+  el_to_path BLOB DEFAULT NULL
 );
-CREATE INDEX el_from ON /*_*/externallinks (el_from, el_to);
-CREATE INDEX el_to ON /*_*/externallinks (el_to, el_from);
-CREATE INDEX el_index ON /*_*/externallinks (el_index);
-CREATE INDEX el_index_60 ON /*_*/externallinks (el_index_60, el_id);
-CREATE INDEX el_from_index_60 ON /*_*/externallinks (el_from, el_index_60, el_id);
+CREATE INDEX el_from ON /*_*/externallinks (el_from);
+CREATE INDEX el_to_domain_index_to_path ON /*_*/externallinks (el_to_domain_index, el_to_path);
 CREATE TABLE /*_*/ip_changes (
   ipc_rev_id INTEGER UNSIGNED DEFAULT 0 NOT NULL,
   ipc_rev_timestamp BLOB NOT NULL,
@@ -311,30 +297,8 @@ CREATE TABLE /*_*/ip_changes (
 );
 CREATE INDEX ipc_rev_timestamp ON /*_*/ip_changes (ipc_rev_timestamp);
 CREATE INDEX ipc_hex_time ON /*_*/ip_changes (ipc_hex, ipc_rev_timestamp);
-CREATE TABLE /*_*/revision_comment_temp (
-  revcomment_rev INTEGER UNSIGNED NOT NULL,
-  revcomment_comment_id BIGINT UNSIGNED NOT NULL,
-  PRIMARY KEY(
-    revcomment_rev, revcomment_comment_id
-  )
-);
-CREATE UNIQUE INDEX revcomment_rev ON /*_*/revision_comment_temp (revcomment_rev);
-CREATE TABLE /*_*/revision_actor_temp (
-  revactor_rev INTEGER UNSIGNED NOT NULL,
-  revactor_actor BIGINT UNSIGNED NOT NULL,
-  revactor_timestamp BLOB NOT NULL,
-  revactor_page INTEGER UNSIGNED NOT NULL,
-  PRIMARY KEY(revactor_rev, revactor_actor)
-);
-CREATE UNIQUE INDEX revactor_rev ON /*_*/revision_actor_temp (revactor_rev);
-CREATE INDEX actor_timestamp ON /*_*/revision_actor_temp (
-  revactor_actor, revactor_timestamp
-);
-CREATE INDEX page_actor_timestamp ON /*_*/revision_actor_temp (
-  revactor_page, revactor_actor, revactor_timestamp
-);
 CREATE TABLE /*_*/page_props (
-  pp_page INTEGER NOT NULL,
+  pp_page INTEGER UNSIGNED NOT NULL,
   pp_propname BLOB NOT NULL,
   pp_value BLOB NOT NULL,
   pp_sortkey DOUBLE PRECISION DEFAULT NULL,
@@ -384,9 +348,6 @@ CREATE INDEX cl_sortkey ON /*_*/categorylinks (
   cl_to, cl_type, cl_sortkey, cl_from
 );
 CREATE INDEX cl_timestamp ON /*_*/categorylinks (cl_to, cl_timestamp);
-CREATE INDEX cl_collation_ext ON /*_*/categorylinks (
-  cl_collation, cl_to, cl_type, cl_from
-);
 CREATE TABLE /*_*/logging (
   log_id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
   log_type BLOB DEFAULT '' NOT NULL, log_action BLOB DEFAULT '' NOT NULL,
@@ -422,7 +383,7 @@ CREATE TABLE /*_*/uploadstash (
   us_status VARCHAR(50) NOT NULL,
   us_chunk_inx INTEGER UNSIGNED DEFAULT NULL,
   us_props BLOB DEFAULT NULL,
-  us_size INTEGER UNSIGNED NOT NULL,
+  us_size BIGINT UNSIGNED NOT NULL,
   us_sha1 VARCHAR(31) NOT NULL,
   us_mime VARCHAR(255) DEFAULT NULL,
   us_media_type TEXT DEFAULT NULL,
@@ -440,7 +401,7 @@ CREATE TABLE /*_*/filearchive (
   fa_storage_key BLOB DEFAULT '', fa_deleted_user INTEGER DEFAULT NULL,
   fa_deleted_timestamp BLOB DEFAULT NULL,
   fa_deleted_reason_id BIGINT UNSIGNED NOT NULL,
-  fa_size INTEGER UNSIGNED DEFAULT 0,
+  fa_size BIGINT UNSIGNED DEFAULT 0,
   fa_width INTEGER DEFAULT 0, fa_height INTEGER DEFAULT 0,
   fa_metadata BLOB DEFAULT NULL, fa_bits INTEGER DEFAULT 0,
   fa_media_type TEXT DEFAULT NULL, fa_major_mime TEXT DEFAULT 'unknown',
@@ -463,7 +424,7 @@ CREATE TABLE /*_*/text (
 );
 CREATE TABLE /*_*/oldimage (
   oi_name BLOB DEFAULT '' NOT NULL, oi_archive_name BLOB DEFAULT '' NOT NULL,
-  oi_size INTEGER UNSIGNED DEFAULT 0 NOT NULL,
+  oi_size BIGINT UNSIGNED DEFAULT 0 NOT NULL,
   oi_width INTEGER DEFAULT 0 NOT NULL,
   oi_height INTEGER DEFAULT 0 NOT NULL,
   oi_bits INTEGER DEFAULT 0 NOT NULL,
@@ -489,31 +450,39 @@ CREATE TABLE /*_*/objectcache (
   PRIMARY KEY(keyname)
 );
 CREATE INDEX exptime ON /*_*/objectcache (exptime);
-CREATE TABLE /*_*/ipblocks (
-  ipb_id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
-  ipb_address BLOB NOT NULL, ipb_user INTEGER UNSIGNED DEFAULT 0 NOT NULL,
-  ipb_by_actor BIGINT UNSIGNED NOT NULL,
-  ipb_reason_id BIGINT UNSIGNED NOT NULL,
-  ipb_timestamp BLOB NOT NULL, ipb_auto SMALLINT DEFAULT 0 NOT NULL,
-  ipb_anon_only SMALLINT DEFAULT 0 NOT NULL,
-  ipb_create_account SMALLINT DEFAULT 1 NOT NULL,
-  ipb_enable_autoblock SMALLINT DEFAULT 1 NOT NULL,
-  ipb_expiry BLOB NOT NULL, ipb_range_start BLOB NOT NULL,
-  ipb_range_end BLOB NOT NULL, ipb_deleted SMALLINT DEFAULT 0 NOT NULL,
-  ipb_block_email SMALLINT DEFAULT 0 NOT NULL,
-  ipb_allow_usertalk SMALLINT DEFAULT 0 NOT NULL,
-  ipb_parent_block_id INTEGER DEFAULT NULL,
-  ipb_sitewide SMALLINT DEFAULT 1 NOT NULL
+CREATE TABLE /*_*/block (
+  bl_id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+  bl_target INTEGER UNSIGNED NOT NULL,
+  bl_by_actor BIGINT UNSIGNED NOT NULL,
+  bl_reason_id BIGINT UNSIGNED NOT NULL,
+  bl_timestamp BLOB NOT NULL, bl_anon_only SMALLINT DEFAULT 0 NOT NULL,
+  bl_create_account SMALLINT DEFAULT 1 NOT NULL,
+  bl_enable_autoblock SMALLINT DEFAULT 1 NOT NULL,
+  bl_expiry BLOB NOT NULL, bl_deleted SMALLINT DEFAULT 0 NOT NULL,
+  bl_block_email SMALLINT DEFAULT 0 NOT NULL,
+  bl_allow_usertalk SMALLINT DEFAULT 0 NOT NULL,
+  bl_parent_block_id INTEGER UNSIGNED DEFAULT NULL,
+  bl_sitewide SMALLINT DEFAULT 1 NOT NULL
 );
-CREATE UNIQUE INDEX ipb_address_unique ON /*_*/ipblocks (ipb_address, ipb_user, ipb_auto);
-CREATE INDEX ipb_user ON /*_*/ipblocks (ipb_user);
-CREATE INDEX ipb_range ON /*_*/ipblocks (ipb_range_start, ipb_range_end);
-CREATE INDEX ipb_timestamp ON /*_*/ipblocks (ipb_timestamp);
-CREATE INDEX ipb_expiry ON /*_*/ipblocks (ipb_expiry);
-CREATE INDEX ipb_parent_block_id ON /*_*/ipblocks (ipb_parent_block_id);
+CREATE INDEX bl_timestamp ON /*_*/block (bl_timestamp);
+CREATE INDEX bl_target ON /*_*/block (bl_target);
+CREATE INDEX bl_expiry ON /*_*/block (bl_expiry);
+CREATE INDEX bl_parent_block_id ON /*_*/block (bl_parent_block_id);
+CREATE TABLE /*_*/block_target (
+  bt_id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+  bt_address BLOB DEFAULT NULL, bt_user INTEGER UNSIGNED DEFAULT NULL,
+  bt_user_text BLOB DEFAULT NULL, bt_auto SMALLINT DEFAULT 0 NOT NULL,
+  bt_range_start BLOB DEFAULT NULL,
+  bt_range_end BLOB DEFAULT NULL, bt_ip_hex BLOB DEFAULT NULL,
+  bt_count INTEGER DEFAULT 0 NOT NULL
+);
+CREATE INDEX bt_address ON /*_*/block_target (bt_address);
+CREATE INDEX bt_ip_user_text ON /*_*/block_target (bt_ip_hex, bt_user_text);
+CREATE INDEX bt_range ON /*_*/block_target (bt_range_start, bt_range_end);
+CREATE INDEX bt_user ON /*_*/block_target (bt_user);
 CREATE TABLE /*_*/image (
   img_name BLOB DEFAULT '' NOT NULL,
-  img_size INTEGER UNSIGNED DEFAULT 0 NOT NULL,
+  img_size BIGINT UNSIGNED DEFAULT 0 NOT NULL,
   img_width INTEGER DEFAULT 0 NOT NULL,
   img_height INTEGER DEFAULT 0 NOT NULL,
   img_metadata BLOB NOT NULL,
@@ -591,7 +560,6 @@ CREATE UNIQUE INDEX ar_revid_uniq ON /*_*/archive (ar_rev_id);
 CREATE TABLE /*_*/page (
   page_id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
   page_namespace INTEGER NOT NULL, page_title BLOB NOT NULL,
-  page_restrictions BLOB DEFAULT NULL,
   page_is_redirect SMALLINT UNSIGNED DEFAULT 0 NOT NULL,
   page_is_new SMALLINT UNSIGNED DEFAULT 0 NOT NULL,
   page_random DOUBLE PRECISION NOT NULL,
@@ -620,24 +588,30 @@ CREATE TABLE /*_*/user (
   user_email_token BLOB DEFAULT NULL,
   user_email_token_expires BLOB DEFAULT NULL,
   user_registration BLOB DEFAULT NULL,
-  user_editcount INTEGER DEFAULT NULL,
-  user_password_expires BLOB DEFAULT NULL
+  user_editcount INTEGER UNSIGNED DEFAULT NULL,
+  user_password_expires BLOB DEFAULT NULL,
+  user_is_temp SMALLINT DEFAULT 0 NOT NULL
 );
 CREATE UNIQUE INDEX user_name ON /*_*/user (user_name);
 CREATE INDEX user_email_token ON /*_*/user (user_email_token);
 CREATE INDEX user_email ON /*_*/user (user_email);
+CREATE TABLE /*_*/user_autocreate_serial (
+  uas_shard INTEGER UNSIGNED NOT NULL,
+  uas_year SMALLINT UNSIGNED NOT NULL,
+  uas_value INTEGER UNSIGNED NOT NULL,
+  PRIMARY KEY(uas_shard, uas_year)
+);
 CREATE TABLE /*_*/revision (
   rev_id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
   rev_page INTEGER UNSIGNED NOT NULL,
-  rev_comment_id BIGINT UNSIGNED DEFAULT 0 NOT NULL,
-  rev_actor BIGINT UNSIGNED DEFAULT 0 NOT NULL,
+  rev_comment_id BIGINT UNSIGNED NOT NULL,
+  rev_actor BIGINT UNSIGNED NOT NULL,
   rev_timestamp BLOB NOT NULL, rev_minor_edit SMALLINT UNSIGNED DEFAULT 0 NOT NULL,
   rev_deleted SMALLINT UNSIGNED DEFAULT 0 NOT NULL,
   rev_len INTEGER UNSIGNED DEFAULT NULL,
-  rev_parent_id INTEGER UNSIGNED DEFAULT NULL,
+  rev_parent_id BIGINT UNSIGNED DEFAULT NULL,
   rev_sha1 BLOB DEFAULT '' NOT NULL
 );
-CREATE INDEX rev_page_id ON /*_*/revision (rev_page, rev_id);
 CREATE INDEX rev_timestamp ON /*_*/revision (rev_timestamp);
 CREATE INDEX rev_page_timestamp ON /*_*/revision (rev_page, rev_timestamp);
 CREATE INDEX rev_actor_timestamp ON /*_*/revision (rev_actor, rev_timestamp, rev_id);
@@ -646,9 +620,14 @@ CREATE INDEX rev_page_actor_timestamp ON /*_*/revision (
 );
 CREATE TABLE /*_*/searchindex (
   si_page INTEGER UNSIGNED NOT NULL,
-  si_title VARCHAR(255) DEFAULT '' NOT NULL,
-  si_text CLOB NOT NULL
+  si_title CLOB NOT NULL,
+  si_text CLOB NOT NULL,
+  PRIMARY KEY(si_page)
 );
-CREATE UNIQUE INDEX si_page ON /*_*/searchindex (si_page);
 CREATE INDEX si_title ON /*_*/searchindex (si_title);
 CREATE INDEX si_text ON /*_*/searchindex (si_text);
+CREATE TABLE /*_*/linktarget (
+  lt_id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+  lt_namespace INTEGER NOT NULL, lt_title BLOB NOT NULL
+);
+CREATE UNIQUE INDEX lt_namespace_title ON /*_*/linktarget (lt_namespace, lt_title);
