@@ -3,10 +3,9 @@
 namespace MediaWiki\Tests\Parser;
 
 use LogicException;
-use MediaWiki\Context\RequestContext;
-use MediaWiki\Debug\MWDebug;
 use MediaWiki\MainConfigNames;
 use MediaWiki\MediaWikiServices;
+use MediaWiki\Parser\ContentHolder;
 use MediaWiki\Parser\ParserOptions;
 use MediaWiki\Parser\ParserOutput;
 use MediaWiki\Parser\ParserOutputFlags;
@@ -17,8 +16,9 @@ use MediaWiki\Title\TitleValue;
 use MediaWiki\Utils\MWTimestamp;
 use MediaWikiLangTestCase;
 use Wikimedia\Bcp47Code\Bcp47CodeValue;
+use Wikimedia\Message\MessageValue;
+use Wikimedia\Parsoid\Core\MergeStrategy;
 use Wikimedia\Parsoid\Core\SectionMetadata;
-use Wikimedia\Parsoid\Core\TOCData;
 use Wikimedia\TestingAccessWrapper;
 use Wikimedia\Tests\SerializationTestTrait;
 
@@ -39,36 +39,24 @@ class ParserOutputTest extends MediaWikiLangTestCase {
 			MainConfigNames::ParserCacheExpireTime,
 			ParserCacheSerializationTestCases::FAKE_CACHE_EXPIRY
 		);
+		$this->overrideConfigValue(
+			MainConfigNames::ParserCacheAsyncExpireTime,
+			ParserCacheSerializationTestCases::FAKE_ASYNC_CACHE_EXPIRY
+		);
 	}
 
-	/**
-	 * Overrides SerializationTestTrait::getClassToTest
-	 * @return string
-	 */
 	public static function getClassToTest(): string {
 		return ParserOutput::class;
 	}
 
-	/**
-	 * Overrides SerializationTestTrait::getSerializedDataPath
-	 * @return string
-	 */
 	public static function getSerializedDataPath(): string {
 		return __DIR__ . '/../../data/ParserCache';
 	}
 
-	/**
-	 * Overrides SerializationTestTrait::getTestInstancesAndAssertions
-	 * @return array
-	 */
 	public static function getTestInstancesAndAssertions(): array {
 		return ParserCacheSerializationTestCases::getParserOutputTestCases();
 	}
 
-	/**
-	 * Overrides SerializationTestTrait::getSupportedSerializationFormats
-	 * @return array
-	 */
 	public static function getSupportedSerializationFormats(): array {
 		return ParserCacheSerializationTestCases::getSupportedSerializationFormats(
 			self::getClassToTest() );
@@ -127,13 +115,24 @@ class ParserOutputTest extends MediaWikiLangTestCase {
 
 		$po->setJsConfigVar( 'c', '2' );
 		$po->appendJsConfigVar( 'b', 'b' );
-		$po->appendJsConfigVar( 'b', '1' );
+		$po->appendJsConfigVar( 'b', 1 );
 
 		$this->assertEqualsCanonicalizing( [
 			'a' => 1,
 			'b' => [ 'a' => true, 'b' => true, '0' => true, '1' => true ],
 			'c' => 2,
 		], $po->getJsConfigVars() );
+
+		$po->appendJsConfigVar( 'count', 1, MergeStrategy::SUM );
+		$po->appendJsConfigVar( 'count', 2, MergeStrategy::SUM );
+		$po->appendJsConfigVar( 'count', 3, MergeStrategy::SUM );
+
+		$this->assertEqualsCanonicalizing( [
+			'a' => 1,
+			'b' => [ 'a' => true, 'b' => true, '0' => true, '1' => true ],
+			'c' => 2,
+			'count' => 6,
+		], $po->getJsConfigVars() );
 	}
 
 	/**
@@ -171,6 +170,11 @@ class ParserOutputTest extends MediaWikiLangTestCase {
 			'abc' => true,
 			'xyz' => true,
 		], $po->getExtensionData( "three" ) );
+
+		$po->appendExtensionData( 'count', 1, MergeStrategy::SUM );
+		$po->appendExtensionData( 'count', 2, MergeStrategy::SUM );
+		$po->appendExtensionData( 'count', 3, MergeStrategy::SUM );
+		$this->assertSame( 6, $po->getExtensionData( 'count' ) );
 	}
 
 	/**
@@ -182,12 +186,8 @@ class ParserOutputTest extends MediaWikiLangTestCase {
 	 * @covers \MediaWiki\Parser\ParserOutput::getPageProperties
 	 * @dataProvider providePageProperties
 	 */
-	public function testPageProperties( string $setPageProperty, $value1, $value2, bool $expectDeprecation = false ) {
+	public function testPageProperties( string $setPageProperty, $value1, $value2 ) {
 		$po = new ParserOutput();
-		if ( $expectDeprecation ) {
-			MWDebug::filterDeprecationForTest( '/::setPageProperty with non-string value/' );
-		}
-
 		$po->$setPageProperty( 'foo', $value1 );
 
 		$properties = $po->getPageProperties();
@@ -211,7 +211,6 @@ class ParserOutputTest extends MediaWikiLangTestCase {
 		yield 'Unsorted' => [ 'setUnsortedPageProperty', 'val', 'second val' ];
 		yield 'Numeric' => [ 'setNumericPageProperty', 42, 3.14 ];
 		yield 'Unsorted (old style)' => [ 'setPageProperty', 'val', 'second val' ];
-		yield 'Numeric (old style)' => [ 'setPageProperty', 123, 456, true ];
 	}
 
 	/**
@@ -317,201 +316,6 @@ class ParserOutputTest extends MediaWikiLangTestCase {
 		$this->assertStringNotContainsString( 'class="', $text );
 	}
 
-	/**
-	 * This test aims at being replaced by its version in DefaultOutputPipelineFactoryTest when
-	 * ParserOutput::getText gets deprecated.
-	 * @covers \MediaWiki\Parser\ParserOutput::getText
-	 * @dataProvider provideGetText
-	 * @param array $options Options to getText()
-	 * @param string $text Parser text
-	 * @param string $expect Expected output
-	 */
-	public function testGetText( $options, $text, $expect ) {
-		// Avoid other skins affecting the section edit links
-		$this->overrideConfigValue( MainConfigNames::DefaultSkin, 'fallback' );
-		RequestContext::resetMain();
-
-		$this->overrideConfigValues( [
-			MainConfigNames::ScriptPath => '/w',
-			MainConfigNames::Script => '/w/index.php',
-		] );
-
-		$po = new ParserOutput( $text );
-		self::initSections( $po );
-		$actual = $po->getText( $options );
-		$this->assertSame( $expect, $actual );
-	}
-
-	private static function initSections( ParserOutput $po ): void {
-		$po->setTOCData( new TOCData(
-			SectionMetadata::fromLegacy( [
-				'index' => "1",
-				'level' => 1,
-				'toclevel' => 1,
-				'number' => "1",
-				'line' => "Section 1",
-				'anchor' => "Section_1"
-			] ),
-			SectionMetadata::fromLegacy( [
-				'index' => "2",
-				'level' => 1,
-				'toclevel' => 1,
-				'number' => "2",
-				'line' => "Section 2",
-				'anchor' => "Section_2"
-			] ),
-			SectionMetadata::fromLegacy( [
-				'index' => "3",
-				'level' => 2,
-				'toclevel' => 2,
-				'number' => "2.1",
-				'line' => "Section 2.1",
-				'anchor' => "Section_2.1"
-			] ),
-			SectionMetadata::fromLegacy( [
-				'index' => "4",
-				'level' => 1,
-				'toclevel' => 1,
-				'number' => "3",
-				'line' => "Section 3",
-				'anchor' => "Section_3"
-			] ),
-		) );
-	}
-
-	public static function provideGetText() {
-		$text = <<<EOF
-<p>Test document.
-</p>
-<meta property="mw:PageProp/toc" />
-<div class="mw-heading mw-heading2"><h2 id="Section_1">Section 1</h2><mw:editsection page="Test Page" section="1">Section 1</mw:editsection></div>
-<p>One
-</p>
-<div class="mw-heading mw-heading2"><h2 id="Section_2">Section 2</h2><mw:editsection page="Test Page" section="2">Section 2</mw:editsection></div>
-<p>Two
-</p>
-<div class="mw-heading mw-heading3"><h3 id="Section_2.1">Section 2.1</h3></div>
-<p>Two point one
-</p>
-<div class="mw-heading mw-heading2"><h2 id="Section_3">Section 3</h2><mw:editsection page="Test Page" section="4">Section 3</mw:editsection></div>
-<p>Three
-</p>
-EOF;
-
-		$dedupText = <<<EOF
-<p>This is a test document.</p>
-<style data-mw-deduplicate="duplicate1">.Duplicate1 {}</style>
-<style data-mw-deduplicate="duplicate1">.Duplicate1 {}</style>
-<style data-mw-deduplicate="duplicate2">.Duplicate2 {}</style>
-<style data-mw-deduplicate="duplicate1">.Duplicate1 {}</style>
-<style data-mw-deduplicate="duplicate2">.Duplicate2 {}</style>
-<style data-mw-not-deduplicate="duplicate1">.Duplicate1 {}</style>
-<style data-mw-deduplicate="duplicate1">.Same-attribute-different-content {}</style>
-<style data-mw-deduplicate="duplicate3">.Duplicate1 {}</style>
-<style>.Duplicate1 {}</style>
-EOF;
-
-		return [
-			'No options' => [
-				[], $text, <<<EOF
-<p>Test document.
-</p>
-<div id="toc" class="toc" role="navigation" aria-labelledby="mw-toc-heading"><input type="checkbox" role="button" id="toctogglecheckbox" class="toctogglecheckbox" style="display:none" /><div class="toctitle" lang="en" dir="ltr"><h2 id="mw-toc-heading">Contents</h2><span class="toctogglespan"><label class="toctogglelabel" for="toctogglecheckbox"></label></span></div>
-<ul>
-<li class="toclevel-1 tocsection-1"><a href="#Section_1"><span class="tocnumber">1</span> <span class="toctext">Section 1</span></a></li>
-<li class="toclevel-1 tocsection-2"><a href="#Section_2"><span class="tocnumber">2</span> <span class="toctext">Section 2</span></a>
-<ul>
-<li class="toclevel-2 tocsection-3"><a href="#Section_2.1"><span class="tocnumber">2.1</span> <span class="toctext">Section 2.1</span></a></li>
-</ul>
-</li>
-<li class="toclevel-1 tocsection-4"><a href="#Section_3"><span class="tocnumber">3</span> <span class="toctext">Section 3</span></a></li>
-</ul>
-</div>
-
-<div class="mw-heading mw-heading2"><h2 id="Section_1">Section 1</h2><span class="mw-editsection"><span class="mw-editsection-bracket">[</span><a href="/w/index.php?title=Test_Page&amp;action=edit&amp;section=1" title="Edit section: Section 1">edit</a><span class="mw-editsection-bracket">]</span></span></div>
-<p>One
-</p>
-<div class="mw-heading mw-heading2"><h2 id="Section_2">Section 2</h2><span class="mw-editsection"><span class="mw-editsection-bracket">[</span><a href="/w/index.php?title=Test_Page&amp;action=edit&amp;section=2" title="Edit section: Section 2">edit</a><span class="mw-editsection-bracket">]</span></span></div>
-<p>Two
-</p>
-<div class="mw-heading mw-heading3"><h3 id="Section_2.1">Section 2.1</h3></div>
-<p>Two point one
-</p>
-<div class="mw-heading mw-heading2"><h2 id="Section_3">Section 3</h2><span class="mw-editsection"><span class="mw-editsection-bracket">[</span><a href="/w/index.php?title=Test_Page&amp;action=edit&amp;section=4" title="Edit section: Section 3">edit</a><span class="mw-editsection-bracket">]</span></span></div>
-<p>Three
-</p>
-EOF
-			],
-			'Disable section edit links' => [
-				[ 'enableSectionEditLinks' => false ], $text, <<<EOF
-<p>Test document.
-</p>
-<div id="toc" class="toc" role="navigation" aria-labelledby="mw-toc-heading"><input type="checkbox" role="button" id="toctogglecheckbox" class="toctogglecheckbox" style="display:none" /><div class="toctitle" lang="en" dir="ltr"><h2 id="mw-toc-heading">Contents</h2><span class="toctogglespan"><label class="toctogglelabel" for="toctogglecheckbox"></label></span></div>
-<ul>
-<li class="toclevel-1 tocsection-1"><a href="#Section_1"><span class="tocnumber">1</span> <span class="toctext">Section 1</span></a></li>
-<li class="toclevel-1 tocsection-2"><a href="#Section_2"><span class="tocnumber">2</span> <span class="toctext">Section 2</span></a>
-<ul>
-<li class="toclevel-2 tocsection-3"><a href="#Section_2.1"><span class="tocnumber">2.1</span> <span class="toctext">Section 2.1</span></a></li>
-</ul>
-</li>
-<li class="toclevel-1 tocsection-4"><a href="#Section_3"><span class="tocnumber">3</span> <span class="toctext">Section 3</span></a></li>
-</ul>
-</div>
-
-<div class="mw-heading mw-heading2"><h2 id="Section_1">Section 1</h2></div>
-<p>One
-</p>
-<div class="mw-heading mw-heading2"><h2 id="Section_2">Section 2</h2></div>
-<p>Two
-</p>
-<div class="mw-heading mw-heading3"><h3 id="Section_2.1">Section 2.1</h3></div>
-<p>Two point one
-</p>
-<div class="mw-heading mw-heading2"><h2 id="Section_3">Section 3</h2></div>
-<p>Three
-</p>
-EOF
-			],
-			'Disable TOC, but wrap' => [
-				[ 'allowTOC' => false, 'wrapperDivClass' => 'mw-parser-output' ], $text, <<<EOF
-<div class="mw-content-ltr mw-parser-output" lang="en" dir="ltr"><p>Test document.
-</p>
-
-<div class="mw-heading mw-heading2"><h2 id="Section_1">Section 1</h2><span class="mw-editsection"><span class="mw-editsection-bracket">[</span><a href="/w/index.php?title=Test_Page&amp;action=edit&amp;section=1" title="Edit section: Section 1">edit</a><span class="mw-editsection-bracket">]</span></span></div>
-<p>One
-</p>
-<div class="mw-heading mw-heading2"><h2 id="Section_2">Section 2</h2><span class="mw-editsection"><span class="mw-editsection-bracket">[</span><a href="/w/index.php?title=Test_Page&amp;action=edit&amp;section=2" title="Edit section: Section 2">edit</a><span class="mw-editsection-bracket">]</span></span></div>
-<p>Two
-</p>
-<div class="mw-heading mw-heading3"><h3 id="Section_2.1">Section 2.1</h3></div>
-<p>Two point one
-</p>
-<div class="mw-heading mw-heading2"><h2 id="Section_3">Section 3</h2><span class="mw-editsection"><span class="mw-editsection-bracket">[</span><a href="/w/index.php?title=Test_Page&amp;action=edit&amp;section=4" title="Edit section: Section 3">edit</a><span class="mw-editsection-bracket">]</span></span></div>
-<p>Three
-</p></div>
-EOF
-			],
-			'Style deduplication' => [
-				[], $dedupText, <<<EOF
-<p>This is a test document.</p>
-<style data-mw-deduplicate="duplicate1">.Duplicate1 {}</style>
-<link rel="mw-deduplicated-inline-style" href="mw-data:duplicate1">
-<style data-mw-deduplicate="duplicate2">.Duplicate2 {}</style>
-<link rel="mw-deduplicated-inline-style" href="mw-data:duplicate1">
-<link rel="mw-deduplicated-inline-style" href="mw-data:duplicate2">
-<style data-mw-not-deduplicate="duplicate1">.Duplicate1 {}</style>
-<link rel="mw-deduplicated-inline-style" href="mw-data:duplicate1">
-<style data-mw-deduplicate="duplicate3">.Duplicate1 {}</style>
-<style>.Duplicate1 {}</style>
-EOF
-			],
-			'Style deduplication disabled' => [
-				[ 'deduplicateStyles' => false ], $dedupText, $dedupText
-			],
-		];
-		// phpcs:enable
-	}
-
 	/**
 	 * @covers \MediaWiki\Parser\ParserOutput::hasText
 	 */
@@ -537,18 +341,6 @@ EOF
 		$this->assertFalse( $po->hasText() );
 	}
 
-	/**
-	 * This test aims at being replaced by its version in DefaultOutputPipelineFactoryTest when
-	 * ParserOutput::getText gets deprecated.
-	 * @covers \MediaWiki\Parser\ParserOutput::getText
-	 */
-	public function testGetText_failsIfNoText() {
-		$po = new ParserOutput( null );
-
-		$this->expectException( LogicException::class );
-		$po->getText();
-	}
-
 	/**
 	 * @covers \MediaWiki\Parser\ParserOutput::getRawText
 	 */
@@ -621,6 +413,7 @@ EOF
 		$a->setJsConfigVar( 'test-config-var-a', 'a' );
 		$a->appendJsConfigVar( 'test-config-var-c', 'abc' );
 		$a->appendJsConfigVar( 'test-config-var-c', 'def' );
+		$a->appendJsConfigVar( 'test-config-var-d', 2, MergeStrategy::SUM );
 		$a->addExtraCSPStyleSrc( 'css.com' );
 		$a->addExtraCSPStyleSrc( 'css2.com' );
 		$a->addExtraCSPScriptSrc( 'js.com' );
@@ -636,6 +429,7 @@ EOF
 		$b->setJsConfigVar( 'test-config-var-a', 'X' );
 		$a->appendJsConfigVar( 'test-config-var-c', 'xyz' );
 		$a->appendJsConfigVar( 'test-config-var-c', 'def' );
+		$a->appendJsConfigVar( 'test-config-var-d', 40, MergeStrategy::SUM );
 		$b->addExtraCSPStyleSrc( 'https://css.ca' );
 		$b->addExtraCSPScriptSrc( 'jscript.com' );
 		$b->addExtraCSPScriptSrc( 'vbscript.com' );
@@ -665,6 +459,7 @@ EOF
 				'test-config-var-c' => [ // merged safely
 					'abc' => true, 'def' => true, 'xyz' => true,
 				],
+				'test-config-var-d' => 42, // merged counters
 			],
 			'getExtraCSPStyleSrcs' => [
 				'css.com',
@@ -711,6 +506,8 @@ EOF
 		$a->setExtensionData( 'foo', 'Foo!' );
 		$a->setExtensionData( 'bar', 'Bar!' );
 		$a->appendExtensionData( 'bat', 'abc' );
+		$a->appendExtensionData( 'count1', 7, MergeStrategy::SUM );
+		$a->appendExtensionData( 'count2', 11, MergeStrategy::SUM );
 
 		$b = new ParserOutput();
 		$b->setNoGallery( true );
@@ -724,6 +521,7 @@ EOF
 		$b->setExtensionData( 'zoo', 'Zoo!' );
 		$b->setExtensionData( 'bar', 'Barrr!' );
 		$b->appendExtensionData( 'bat', 'xyz' );
+		$b->appendExtensionData( 'count2', 13, MergeStrategy::SUM );
 
 		// Note that overwriting extension data during the merge
 		// (as this test case does for 'bar') is deprecated and will eventually
@@ -748,6 +546,8 @@ EOF
 				// internal strategy key is exposed here because we're looking
 				// at the raw property value, not using getExtensionData()
 				'bat' => [ 'abc' => true, 'xyz' => true, '_mw-strategy' => 'union' ],
+				'count1' => [ 'value' => 7, '_mw-strategy' => 'sum' ],
+				'count2' => [ 'value' => 24, '_mw-strategy' => 'sum' ],
 			],
 		] ];
 	}
@@ -765,7 +565,10 @@ EOF
 
 		$this->assertFieldValues( $a, $expected );
 
-		// test twice, to make sure the operation is idempotent (except for the TOC, see below)
+		// test twice, to make sure the operation is idempotent
+		// (except for the TOC, see below)
+		// (and except for counter values)
+		$b->setExtensionData( 'count2', null );
 		$a->mergeHtmlMetaDataFrom( $b );
 
 		// XXX: TOC joining should get smarter. Can we make it idempotent as well?
@@ -817,29 +620,36 @@ EOF
 		$a->addLink( new TitleValue( NS_TALK, 'Puppies', 'Topic' ), 17 );
 
 		$expected = [
-			NS_MAIN => [ 'Kittens' => 6, 'Goats_786827346' => 0 ],
-			NS_TALK => [ 'Kittens' => 16, 'Puppies' => 17 ]
+			[ 'link' => '0:Kittens', 'pageid' => 6 ],
+			[ 'link' => '0:Goats_786827346', 'pageid' => 0 ],
+			[ 'link' => '1:Kittens', 'pageid' => 16 ],
+			[ 'link' => '1:Puppies', 'pageid' => 17 ],
 		];
-		$this->assertSame( $expected, $a->getLinks() );
+		$this->assertEqualsCanonicalizing( $expected, array_map(
+			static fn ( $item )=>[ 'link' => strval( $item['link'] ) ] + $item,
+			$a->getLinkList( ParserOutputLinkTypes::LOCAL ) ) );
 		$expected = [
 			[
-				'link' => new TitleValue( NS_MAIN, 'Kittens' ),
+				'link' => '0:Kittens',
 				'pageid' => 6,
 			],
 			[
-				'link' => new TitleValue( NS_MAIN, 'Goats_786827346' ),
+				'link' => '0:Goats_786827346',
 				'pageid' => 0,
 			],
 			[
-				'link' => new TitleValue( NS_TALK, 'Kittens' ),
+				'link' => '1:Kittens',
 				'pageid' => 16,
 			],
 			[
-				'link' => new TitleValue( NS_TALK, 'Puppies' ),
+				'link' => '1:Puppies',
 				'pageid' => 17,
 			],
 		];
-		$this->assertEquals( $expected, $a->getLinkList( ParserOutputLinkTypes::LOCAL ) );
+		$this->assertEqualsCanonicalizing( $expected, array_map(
+			static fn ( $item )=>[ 'link' => strval( $item['link'] ) ] + $item,
+			$a->getLinkList( ParserOutputLinkTypes::LOCAL )
+		) );
 	}
 
 	public static function provideMergeTrackingMetaDataFrom() {
@@ -982,7 +792,7 @@ EOF
 			],
 			'getInterwikiLinks' => [
 				'de' => [ 'Kittens_DE' => 1 ],
-				'ru' => [ 'Kittens_RU' => 1, 'Dragons_RU' => 1, ],
+				'ru' => [ 'Kittens RU' => 1, 'Dragons RU' => 1, ],
 				'fr' => [ 'Kittens_FR' => 1 ],
 			],
 			'getLinkList!INTERWIKI' => [
@@ -991,10 +801,10 @@ EOF
 					'link' => new TitleValue( NS_MAIN, 'Kittens_DE', '', 'de' ),
 				],
 				[
-					'link' => new TitleValue( NS_MAIN, 'Kittens_RU', '', 'ru' ),
+					'link' => new TitleValue( NS_MAIN, 'Kittens RU', '', 'ru' ),
 				],
 				[
-					'link' => new TitleValue( NS_MAIN, 'Dragons_RU', '', 'ru' ),
+					'link' => new TitleValue( NS_MAIN, 'Dragons RU', '', 'ru' ),
 				],
 				[
 					'link' => new TitleValue( NS_MAIN, 'Kittens_FR', '', 'fr' ),
@@ -1052,6 +862,7 @@ EOF
 		$a->setExtensionData( 'foo', 'Foo!' );
 		$a->setExtensionData( 'bar', 'Bar!' );
 		$a->appendExtensionData( 'bat', 'abc' );
+		$a->appendExtensionData( 'counter', 3, MergeStrategy::SUM );
 
 		$b = new ParserOutput();
 
@@ -1061,6 +872,7 @@ EOF
 		$b->setExtensionData( 'zoo', 'Zoo!' );
 		$b->setExtensionData( 'bar', 'Barrr!' );
 		$b->appendExtensionData( 'bat', 'xyz' );
+		$b->appendExtensionData( 'counter', 5, MergeStrategy::SUM );
 
 		// Note that overwriting extension data during the merge
 		// (as this test case does for 'bar') is deprecated and will eventually
@@ -1079,6 +891,7 @@ EOF
 				// internal strategy key is exposed here because we're looking
 				// at the raw property value, not using getExtensionData()
 				'bat' => [ 'abc' => true, 'xyz' => true, '_mw-strategy' => 'union' ],
+				'counter' => [ 'value' => 8, '_mw-strategy' => 'sum' ],
 			],
 		] ];
 	}
@@ -1092,11 +905,20 @@ EOF
 	 * @param array $expected
 	 */
 	public function testMergeTrackingMetaDataFrom( ParserOutput $a, ParserOutput $b, $expected ) {
+		$this->filterDeprecated( '/ParserOutput::getFileSearchOptions was deprecated/' );
+		$this->filterDeprecated( '/ParserOutput::getImages was deprecated/' );
+		$this->filterDeprecated( '/ParserOutput::getInterwikiLinks was deprecated/' );
+		$this->filterDeprecated( '/ParserOutput::getTemplates was deprecated/' );
+		$this->filterDeprecated( '/ParserOutput::getTemplateIds was deprecated/' );
+		$this->filterDeprecated( '/ParserOutput::getLinks was deprecated/' );
+		$this->filterDeprecated( '/ParserOutput::getLanguageLinks was deprecated/' );
 		$a->mergeTrackingMetaDataFrom( $b );
 
 		$this->assertFieldValues( $a, $expected );
 
 		// test twice, to make sure the operation is idempotent
+		// (except for counters!)
+		$b->setExtensionData( 'counter', null );
 		$a->mergeTrackingMetaDataFrom( $b );
 
 		$this->assertFieldValues( $a, $expected );
@@ -1111,36 +933,37 @@ EOF
 	 * @param array $expected
 	 */
 	public function testCollectMetaData( ParserOutput $a, ParserOutput $b, $expected ) {
+		$this->filterDeprecated( '/ParserOutput::getFileSearchOptions was deprecated/' );
+		$this->filterDeprecated( '/ParserOutput::getImages was deprecated/' );
+		$this->filterDeprecated( '/ParserOutput::getInterwikiLinks was deprecated/' );
+		$this->filterDeprecated( '/ParserOutput::getTemplates was deprecated/' );
+		$this->filterDeprecated( '/ParserOutput::getTemplateIds was deprecated/' );
+		$this->filterDeprecated( '/ParserOutput::getLinks was deprecated/' );
+		$this->filterDeprecated( '/ParserOutput::getLanguageLinks was deprecated/' );
 		$b->collectMetadata( $a );
 
 		$this->assertFieldValues( $a, $expected );
 	}
 
-	public function provideMergeInternalMetaDataFrom() {
-		$this->filterDeprecated( '/^.*CacheTime::setCacheTime called with -1 as an argument/' );
-
+	public static function provideMergeInternalMetaDataFrom() {
 		// flags & co
-		$a = new ParserOutput();
+		$a = [
+			'warning' => [
+				[ 'duplicate-args-warning', 'A', 'B', 'C' ],
+				[ 'template-loop-warning', 'D' ],
+			],
+			'outputFlag' => [ ParserOutputFlags::USER_SIGNATURE, ParserOutputFlags::IS_PREVIEW ],
+			'recordOption' => [ 'Foo', 'Bar' ],
+		];
 
-		$a->addWarningMsg( 'duplicate-args-warning', 'A', 'B', 'C' );
-		$a->addWarningMsg( 'template-loop-warning', 'D' );
-
-		$a->setOutputFlag( 'foo' );
-		$a->setOutputFlag( 'bar' );
-
-		$a->recordOption( 'Foo' );
-		$a->recordOption( 'Bar' );
-
-		$b = new ParserOutput();
-
-		$b->addWarningMsg( 'template-equals-warning' );
-		$b->addWarningMsg( 'template-loop-warning', 'D' );
-
-		$b->setOutputFlag( 'zoo' );
-		$b->setOutputFlag( 'bar' );
-
-		$b->recordOption( 'Zoo' );
-		$b->recordOption( 'Bar' );
+		$b = [
+			'warning' => [
+				[ 'template-equals-warning' ],
+				[ 'template-loop-warning', 'D' ],
+			],
+			'outputFlag' => [ ParserOutputFlags::SHOW_TOC, ParserOutputFlags::IS_PREVIEW ],
+			'recordOption' => [ 'Zoo', 'Bar' ],
+		];
 
 		yield 'flags' => [ $a, $b, [
 			'getWarnings' => [
@@ -1148,119 +971,148 @@ EOF
 				wfMessage( 'template-loop-warning', 'D' )->text(),
 				wfMessage( 'template-equals-warning' )->text(),
 			],
-			'$mFlags' => [ 'foo' => true, 'bar' => true, 'zoo' => true ],
+			'getWarningMsgs' => [
+				MessageValue::new( 'duplicate-args-warning', [ 'A', 'B', 'C' ] ),
+				MessageValue::new( 'template-loop-warning', [ 'D' ] ),
+				MessageValue::new( 'template-equals-warning' ),
+			],
+			'$mFlags' => [ 'user-signature' => true, 'is-preview' => true, 'show-toc' => true ],
 			'getUsedOptions' => [ 'Foo', 'Bar', 'Zoo' ],
 		] ];
 
 		// cache time
 		$someTime = "20240207202040";
 		$someLaterTime = "20240207202112";
-		$a = new ParserOutput();
-		$a->setCacheTime( $someTime );
-		$b = new ParserOutput();
+		$a = [
+			'cacheTime' => $someTime,
+		];
+		$b = [];
 		yield 'only left cache time' => [ $a, $b, [ 'getCacheTime' => $someTime ] ];
 
-		$a = new ParserOutput();
-		$b = new ParserOutput();
-		$b->setCacheTime( $someTime );
+		$a = [];
+		$b = [
+			'cacheTime' => $someTime,
+		];
 		yield 'only right cache time' => [ $a, $b, [ 'getCacheTime' => $someTime ] ];
 
-		$a = new ParserOutput();
-		$b = new ParserOutput();
-		$a->setCacheTime( $someLaterTime );
-		$b->setCacheTime( $someTime );
+		$a = [
+			'cacheTime' => $someLaterTime,
+		];
+		$b = [
+			'cacheTime' => $someTime,
+		];
 		yield 'left has later cache time' => [ $a, $b, [ 'getCacheTime' => $someLaterTime ] ];
 
-		$a = new ParserOutput();
-		$b = new ParserOutput();
-		$a->setCacheTime( $someTime );
-		$b->setCacheTime( $someLaterTime );
+		$a = [
+			'cacheTime' => $someTime,
+		];
+		$b = [
+			'cacheTime' => $someLaterTime,
+		];
 		yield 'right has later cache time' => [ $a, $b, [ 'getCacheTime' => $someLaterTime ] ];
 
-		$a = new ParserOutput();
-		$b = new ParserOutput();
-		$a->setCacheTime( -1 );
-		$b->setCacheTime( $someTime );
+		$a = [
+			'cacheTime' => -1,
+		];
+		$b = [
+			'cacheTime' => $someTime,
+		];
 		yield 'left is uncacheable' => [ $a, $b, [ 'getCacheTime' => "-1" ] ];
 
-		$a = new ParserOutput();
-		$b = new ParserOutput();
-		$a->setCacheTime( $someTime );
-		$b->setCacheTime( -1 );
+		$a = [
+			'cacheTime' => $someTime,
+		];
+		$b = [
+			'cacheTime' => -1,
+		];
 		yield 'right is uncacheable' => [ $a, $b, [ 'getCacheTime' => "-1" ] ];
 
 		// timestamp ------------
-		$a = new ParserOutput();
-		$a->setRevisionTimestamp( '20180101000011' );
-		$b = new ParserOutput();
-		yield 'only left timestamp' => [ $a, $b, [ 'getTimestamp' => '20180101000011' ] ];
+		$a = [
+			'revisionTimestamp' => '20180101000011',
+		];
+		$b = [];
+		yield 'only left timestamp' => [ $a, $b, [ 'getRevisionTimestamp' => '20180101000011' ] ];
 
-		$a = new ParserOutput();
-		$b = new ParserOutput();
-		$b->setRevisionTimestamp( '20180101000011' );
-		yield 'only right timestamp' => [ $a, $b, [ 'getTimestamp' => '20180101000011' ] ];
+		$a = [];
+		$b = [
+			'revisionTimestamp' => '20180101000011',
+		];
+		yield 'only right timestamp' => [ $a, $b, [ 'getRevisionTimestamp' => '20180101000011' ] ];
 
-		$a = new ParserOutput();
-		$a->setRevisionTimestamp( '20180101000011' );
-		$b = new ParserOutput();
-		$b->setRevisionTimestamp( '20180101000001' );
-		yield 'left timestamp wins' => [ $a, $b, [ 'getTimestamp' => '20180101000011' ] ];
+		$a = [
+			'revisionTimestamp' => '20180101000011',
+		];
+		$b = [
+			'revisionTimestamp' => '20180101000001',
+		];
+		yield 'left timestamp wins' => [ $a, $b, [ 'getRevisionTimestamp' => '20180101000011' ] ];
 
-		$a = new ParserOutput();
-		$a->setRevisionTimestamp( '20180101000001' );
-		$b = new ParserOutput();
-		$b->setRevisionTimestamp( '20180101000011' );
-		yield 'right timestamp wins' => [ $a, $b, [ 'getTimestamp' => '20180101000011' ] ];
+		$a = [
+			'revisionTimestamp' => '20180101000001',
+		];
+		$b = [
+			'revisionTimestamp' => '20180101000011',
+		];
+		yield 'right timestamp wins' => [ $a, $b, [ 'getRevisionTimestamp' => '20180101000011' ] ];
 
 		// speculative rev id ------------
-		$a = new ParserOutput();
-		$a->setSpeculativeRevIdUsed( 9 );
-		$b = new ParserOutput();
+		$a = [
+			'speculativeRevIdUsed' => 9,
+		];
+		$b = [];
 		yield 'only left speculative rev id' => [ $a, $b, [ 'getSpeculativeRevIdUsed' => 9 ] ];
 
-		$a = new ParserOutput();
-		$b = new ParserOutput();
-		$b->setSpeculativeRevIdUsed( 9 );
+		$a = [];
+		$b = [
+			'speculativeRevIdUsed' => 9,
+		];
 		yield 'only right speculative rev id' => [ $a, $b, [ 'getSpeculativeRevIdUsed' => 9 ] ];
 
-		$a = new ParserOutput();
-		$a->setSpeculativeRevIdUsed( 9 );
-		$b = new ParserOutput();
-		$b->setSpeculativeRevIdUsed( 9 );
+		$a = [
+			'speculativeRevIdUsed' => 9,
+		];
+		$b = [
+			'speculativeRevIdUsed' => 9,
+		];
 		yield 'same speculative rev id' => [ $a, $b, [ 'getSpeculativeRevIdUsed' => 9 ] ];
 
 		// limit report (recursive max) ------------
-		$a = new ParserOutput();
+		$a = [
+			'limitReportData' => [
+				[ 'naive1', 7 ],
+				[ 'naive2', 27 ],
 
-		$a->setLimitReportData( 'naive1', 7 );
-		$a->setLimitReportData( 'naive2', 27 );
+				[ 'limitreport-simple1', 7 ],
+				[ 'limitreport-simple2', 27 ],
 
-		$a->setLimitReportData( 'limitreport-simple1', 7 );
-		$a->setLimitReportData( 'limitreport-simple2', 27 );
+				[ 'limitreport-pair1', [ 7, 9 ] ],
+				[ 'limitreport-pair2', [ 27, 29 ] ],
 
-		$a->setLimitReportData( 'limitreport-pair1', [ 7, 9 ] );
-		$a->setLimitReportData( 'limitreport-pair2', [ 27, 29 ] );
+				[ 'limitreport-more1', [ 7, 9, 1 ] ],
+				[ 'limitreport-more2', [ 27, 29, 21 ] ],
 
-		$a->setLimitReportData( 'limitreport-more1', [ 7, 9, 1 ] );
-		$a->setLimitReportData( 'limitreport-more2', [ 27, 29, 21 ] );
+				[ 'limitreport-only-a', 13 ],
+			],
+		];
 
-		$a->setLimitReportData( 'limitreport-only-a', 13 );
+		$b = [
+			'limitReportData' => [
+				[ 'naive1', 17 ],
+				[ 'naive2', 17 ],
 
-		$b = new ParserOutput();
+				[ 'limitreport-simple1', 17 ],
+				[ 'limitreport-simple2', 17 ],
 
-		$b->setLimitReportData( 'naive1', 17 );
-		$b->setLimitReportData( 'naive2', 17 );
+				[ 'limitreport-pair1', [ 17, 19 ] ],
+				[ 'limitreport-pair2', [ 17, 19 ] ],
 
-		$b->setLimitReportData( 'limitreport-simple1', 17 );
-		$b->setLimitReportData( 'limitreport-simple2', 17 );
+				[ 'limitreport-more1', [ 17, 19, 11 ] ],
+				[ 'limitreport-more2', [ 17, 19, 11 ] ],
 
-		$b->setLimitReportData( 'limitreport-pair1', [ 17, 19 ] );
-		$b->setLimitReportData( 'limitreport-pair2', [ 17, 19 ] );
-
-		$b->setLimitReportData( 'limitreport-more1', [ 17, 19, 11 ] );
-		$b->setLimitReportData( 'limitreport-more2', [ 17, 19, 11 ] );
-
-		$b->setLimitReportData( 'limitreport-only-b', 23 );
+				[ 'limitreport-only-b', 23 ],
+			],
+		];
 
 		// first write wins
 		yield 'limit report' => [ $a, $b, [
@@ -1289,20 +1141,17 @@ EOF
 				],
 			],
 		] ];
-
-		MWDebug::clearDeprecationFilters();
 	}
 
 	/**
 	 * @dataProvider provideMergeInternalMetaDataFrom
 	 * @covers \MediaWiki\Parser\ParserOutput::mergeInternalMetaDataFrom
-	 *
-	 * @param ParserOutput $a
-	 * @param ParserOutput $b
-	 * @param array $expected
 	 */
-	public function testMergeInternalMetaDataFrom( ParserOutput $a, ParserOutput $b, $expected ) {
+	public function testMergeInternalMetaDataFrom( array $aSpec, array $bSpec, $expected ) {
 		$this->filterDeprecated( '/^.*CacheTime::setCacheTime called with -1 as an argument/' );
+		$this->filterDeprecated( '/ParserOutput::getWarnings was deprecated/' );
+		$a = $this->createParserOutput( $aSpec );
+		$b = $this->createParserOutput( $bSpec );
 		$a->mergeInternalMetaDataFrom( $b );
 
 		$this->assertFieldValues( $a, $expected );
@@ -1313,6 +1162,32 @@ EOF
 		$this->assertFieldValues( $a, $expected );
 	}
 
+	private function createParserOutput( array $spec ): ParserOutput {
+		$po = new ParserOutput();
+		foreach ( $spec['warning'] ?? [] as $warning ) {
+			$po->addWarningMsg( ...$warning );
+		}
+		foreach ( $spec['outputFlag'] ?? [] as $outputFlag ) {
+			$po->setOutputFlag( $outputFlag );
+		}
+		foreach ( $spec['recordOption'] ?? [] as $recordOption ) {
+			$po->recordOption( $recordOption );
+		}
+		foreach ( $spec['limitReportData'] ?? [] as $limitReportData ) {
+			$po->setLimitReportData( ...$limitReportData );
+		}
+		if ( isset( $spec['cacheTime'] ) ) {
+			$po->setCacheTime( $spec['cacheTime'] );
+		}
+		if ( isset( $spec['revisionTimestamp'] ) ) {
+			$po->setRevisionTimestamp( $spec['revisionTimestamp'] );
+		}
+		if ( isset( $spec['speculativeRevIdUsed'] ) ) {
+			$po->setSpeculativeRevIdUsed( $spec['speculativeRevIdUsed'] );
+		}
+		return $po;
+	}
+
 	/**
 	 * @covers \MediaWiki\Parser\ParserOutput::mergeInternalMetaDataFrom
 	 * @covers \MediaWiki\Parser\ParserOutput::getTimes
@@ -1533,11 +1408,47 @@ EOF
 		// But calling ::get assigns a cache time
 		$po->getCacheTime();
 		$this->assertTrue( $po->hasCacheTime() );
+		$this->assertTrue( $po->isCacheable() );
 		// Reset cache time
 		$po->setCacheTime( "20240207202040" );
 		$this->assertSame( "20240207202040", $po->getCacheTime() );
 	}
 
+	/**
+	 * @covers \MediaWiki\Parser\ParserOutput::isCacheable()
+	 * @covers \MediaWiki\Parser\ParserOutput::getCacheExpiry()
+	 * @covers \MediaWiki\Parser\ParserOutput::hasReducedExpiry()
+	 */
+	public function testAsyncNotReady() {
+		$defaultExpiry = ParserCacheSerializationTestCases::FAKE_CACHE_EXPIRY;
+		$asyncExpiry = ParserCacheSerializationTestCases::FAKE_ASYNC_CACHE_EXPIRY;
+		// $asyncExpiry has to be smaller than the default for these tests to
+		// work properly.
+		$this->assertTrue( $asyncExpiry < $defaultExpiry );
+
+		$po = new ParserOutput();
+		$po->getCacheTime(); // assign a cache time
+		$this->assertTrue( $po->isCacheable() );
+		$this->assertFalse( $po->hasReducedExpiry() );
+
+		// hasReducedExpiry is set if there is/was any async content
+		$po->setOutputFlag( ParserOutputFlags::HAS_ASYNC_CONTENT );
+		$this->assertTrue( $po->isCacheable() );
+		$this->assertTrue( $po->hasReducedExpiry() );
+		$this->assertTrue( $po->getCacheExpiry() === $defaultExpiry );
+
+		// Setting ASYNC_NOT_READY also shortens the cache expiry
+		$po->setOutputFlag( ParserOutputFlags::ASYNC_NOT_READY );
+		$this->assertTrue( $po->isCacheable() );
+		$this->assertTrue( $po->hasReducedExpiry() );
+		$this->assertTrue( $po->getCacheExpiry() === $asyncExpiry );
+
+		$po->updateCacheExpiry( $defaultExpiry - 1 );
+		$this->assertTrue( $po->isCacheable() );
+		$this->assertTrue( $po->hasReducedExpiry() );
+		$this->assertTrue( $po->getCacheExpiry() === $asyncExpiry );
+	}
+
 	/**
 	 * @covers \MediaWiki\Parser\ParserOutput::getRenderId()
 	 * @covers \MediaWiki\Parser\ParserOutput::setRenderId()
@@ -1588,4 +1499,10 @@ EOF
 		$this->assertTrue( $pOutput->getOutputFlag( ParserOutputFlags::NO_SECTION_EDIT_LINKS ) );
 		$this->assertTrue( $pOutput->getOutputFlag( ParserOutputFlags::COLLAPSIBLE_SECTIONS ) );
 	}
+
+	public function ignoreForObjectEquality(): array {
+		return [
+			[ ContentHolder::class, 'ownerDocument' ],
+		];
+	}
 }
