@@ -1,20 +1,17 @@
 <?php
 
+use MediaWiki\Title\Title;
+
 class SpecialSearchTestMockResultSet extends SearchResultSet {
-	/** @var array */
-	protected $results;
-	/** @var string|null */
-	protected $suggestion;
-	/** @var string|null */
-	protected $rewrittenQuery;
-	/** @var bool */
-	protected $containedSyntax;
+
+	private ?string $suggestion;
+	private ?string $rewrittenQuery;
 
 	public function __construct(
-		$suggestion = null,
-		$rewrittenQuery = null,
+		?string $suggestion = null,
+		?string $rewrittenQuery = null,
 		array $results = [],
-		$containedSyntax = false
+		bool $containedSyntax = false
 	) {
 		$this->suggestion = $suggestion;
 		$this->rewrittenQuery = $rewrittenQuery;
@@ -22,42 +19,43 @@ class SpecialSearchTestMockResultSet extends SearchResultSet {
 		$this->containedSyntax = $containedSyntax;
 	}
 
-	public function expandResults() {
-		return $this->results;
-	}
-
+	/** @inheritDoc */
 	public function getTotalHits() {
 		return $this->numRows();
 	}
 
+	/** @inheritDoc */
 	public function hasSuggestion() {
 		return $this->suggestion !== null;
 	}
 
+	/** @inheritDoc */
 	public function getSuggestionQuery() {
 		return $this->suggestion;
 	}
 
+	/** @inheritDoc */
 	public function getSuggestionSnippet() {
 		return $this->suggestion;
 	}
 
+	/** @inheritDoc */
 	public function hasRewrittenQuery() {
 		return $this->rewrittenQuery !== null;
 	}
 
+	/** @inheritDoc */
 	public function getQueryAfterRewrite() {
 		return $this->rewrittenQuery;
 	}
 
+	/** @inheritDoc */
 	public function getQueryAfterRewriteSnippet() {
 		return htmlspecialchars( $this->rewrittenQuery );
 	}
 
-	public function getFirstResult() {
-		if ( count( $this->results ) === 0 ) {
-			return null;
-		}
-		return $this->results[0]->getTitle();
+	public function getFirstResult(): ?Title {
+		$first = reset( $this->results );
+		return $first ? $first->getTitle() : null;
 	}
 }
