@@ -8,7 +8,6 @@ use MediaWiki\Linker\LinkTarget;
 use MediaWiki\MainConfigNames;
 use MediaWiki\Message\Message;
 use MediaWiki\Page\Article;
-use MediaWiki\Page\PageIdentity;
 use MediaWiki\Page\PageIdentityValue;
 use MediaWiki\Tests\Unit\DummyServicesTrait;
 use MediaWiki\Title\MalformedTitleException;
@@ -1837,24 +1836,6 @@ class TitleTest extends MediaWikiIntegrationTestCase {
 		);
 	}
 
-	/**
-	 * @covers \MediaWiki\Title\Title::getTitleProtection
-	 */
-	public function testGetTitleProtection() {
-		$title = $this->getNonexistingTestPage( 'UTest1' )->getTitle();
-		$this->hideDeprecated( Title::class . '::getTitleProtection' );
-		$this->assertFalse( $title->getTitleProtection() );
-	}
-
-	/**
-	 * @covers \MediaWiki\Title\Title::deleteTitleProtection
-	 */
-	public function testDeleteTitleProtection() {
-		$title = $this->getExistingTestPage( 'UTest1' )->getTitle();
-		$this->hideDeprecated( Title::class . '::getTitleProtection' );
-		$this->assertFalse( $title->getTitleProtection() );
-	}
-
 	/**
 	 * @covers \MediaWiki\Page\PageStore::getSubpages
 	 */
@@ -2148,7 +2129,7 @@ class TitleTest extends MediaWikiIntegrationTestCase {
 		$title->resetArticleID( 1 );
 		yield '(PageIdentityValue) same text, title has ID 0' => [
 			$title,
-			new PageIdentityValue( 1, 0, 'Foo', PageIdentity::LOCAL ),
+			PageIdentityValue::localIdentity( 1, 0, 'Foo' ),
 			true
 		];
 
@@ -2156,7 +2137,7 @@ class TitleTest extends MediaWikiIntegrationTestCase {
 		$title->resetArticleID( 0 );
 		yield '(PageIdentityValue) same text, PageIdentityValue has ID 0' => [
 			$title,
-			new PageIdentityValue( 0, 1, 'Bar_Baz', PageIdentity::LOCAL ),
+			PageIdentityValue::localIdentity( 0, 1, 'Bar_Baz' ),
 			true
 		];
 
@@ -2164,7 +2145,7 @@ class TitleTest extends MediaWikiIntegrationTestCase {
 		$title->resetArticleID( 0 );
 		yield '(PageIdentityValue) different text, both IDs are 0' => [
 			$title,
-			new PageIdentityValue( 0, 0, 'Foozz', PageIdentity::LOCAL ),
+			PageIdentityValue::localIdentity( 0, 0, 'Foozz' ),
 			false
 		];
 
@@ -2172,7 +2153,7 @@ class TitleTest extends MediaWikiIntegrationTestCase {
 		$title->resetArticleID( 0 );
 		yield '(PageIdentityValue) different namespace' => [
 			$title,
-			new PageIdentityValue( 0, 1, 'Foo', PageIdentity::LOCAL ),
+			PageIdentityValue::localIdentity( 0, 1, 'Foo' ),
 			false
 		];
 
