@@ -8,29 +8,32 @@ class MockSearchResult extends RevisionSearchResult {
 	/** @var string */
 	private $interwikiPrefix = '';
 
+	/** @inheritDoc */
 	public function isMissingRevision() {
 		return $this->isMissingRevision;
 	}
 
-	public function setMissingRevision( $isMissingRevision ) {
+	public function setMissingRevision( bool $isMissingRevision ): self {
 		$this->isMissingRevision = $isMissingRevision;
 		return $this;
 	}
 
+	/** @inheritDoc */
 	public function isBrokenTitle() {
 		return $this->isBrokenTitle;
 	}
 
-	public function setBrokenTitle( $isBrokenTitle ) {
+	public function setBrokenTitle( bool $isBrokenTitle ): self {
 		$this->isBrokenTitle = $isBrokenTitle;
 		return $this;
 	}
 
+	/** @inheritDoc */
 	public function getInterwikiPrefix() {
 		return $this->interwikiPrefix;
 	}
 
-	public function setInterwikiPrefix( $interwikiPrefix ) {
+	public function setInterwikiPrefix( string $interwikiPrefix ): self {
 		$this->interwikiPrefix = $interwikiPrefix;
 		return $this;
 	}
