@@ -206,6 +206,17 @@ class HandlerTest extends MediaWikiUnitTestCase {
 		}
 	}
 
+	public function testApplyDeprecationHeader() {
+		$handler = $this->newHandler( [ 'getDeprecatedDate' ] );
+		$handler->method( 'getDeprecatedDate' )->willReturn( 1735689600 );
+
+		$this->initHandler( $handler, new RequestData() );
+		$response = $handler->getResponseFactory()->create();
+		$handler->applyDeprecationHeader( $response );
+
+		$this->assertSame( '@1735689600', $response->getHeaderLine( 'Deprecation' ) );
+	}
+
 	public function testApplyConditionalResponseHeaders() {
 		$util = $this->createNoOpMock( ConditionalHeaderUtil::class, [ 'applyResponseHeaders' ] );
 		$util->method( 'applyResponseHeaders' )->willReturnCallback(
@@ -1006,7 +1017,14 @@ class HandlerTest extends MediaWikiUnitTestCase {
 					'headers' => [ 'Content-Type' => 'multipart/form-data' ]
 				] ),
 				[ 'param' => "L\u{00E4}rm" ]
-			]
+			],
+			'form data with array' => [
+				new RequestData( [
+					'postParams' => [ 'foo' => [ 'a' => "L\u{0061}\u{0308}rm", 'b' => 'X' ] ],
+					'headers' => [ 'Content-Type' => 'application/x-www-form-urlencoded' ]
+				] ),
+				[ 'foo' => [ 'a' => "L\u{00E4}rm", 'b' => 'X' ] ]
+			],
 		];
 	}
 
@@ -1318,6 +1336,7 @@ class HandlerTest extends MediaWikiUnitTestCase {
 			'$requestTypes' => [ 'application/json' ],
 			'$responseBodySchema' => null,
 			'$routeConfig' => [ 'path' => '/test' ],
+			'$openApiSpec' => [],
 			'$method' => 'GET',
 			'$assertions' =>
 				static function ( array $spec ) {
@@ -1325,8 +1344,7 @@ class HandlerTest extends MediaWikiUnitTestCase {
 					$resp = $spec['responses'];
 
 					Assert::assertArrayHasKey( 200, $resp );
-					Assert::assertArrayHasKey( 400, $resp );
-					Assert::assertArrayHasKey( 500, $resp );
+					Assert::assertArrayHasKey( 'default', $resp );
 				},
 		];
 
@@ -1359,6 +1377,7 @@ class HandlerTest extends MediaWikiUnitTestCase {
 			'$requestTypes' => [ 'application/json' ],
 			'$responseBodySchema' => null,
 			'$routeConfig' => [ 'path' => '/test/{a}/{b}/{d}' ],
+			'$openApiSpec' => [],
 			'$method' => 'GET',
 			'$assertions' =>
 				static function ( array $spec ) {
@@ -1414,6 +1433,7 @@ class HandlerTest extends MediaWikiUnitTestCase {
 			'$requestTypes' => [ 'application/json' ],
 			'$responseBodySchema' => null,
 			'$routeConfig' => [ 'path' => '/test' ],
+			'$openApiSpec' => [],
 			'$method' => 'GET',
 			'$assertions' =>
 				static function ( array $spec ) {
@@ -1470,6 +1490,7 @@ class HandlerTest extends MediaWikiUnitTestCase {
 			'$requestTypes' => [ 'application/foo+json', 'application/bar+json' ],
 			'$responseBodySchema' => null,
 			'$routeConfig' => [ 'path' => '/test' ],
+			'$openApiSpec' => [],
 			'$method' => 'PUT',
 			'$assertions' =>
 				static function ( array $spec ) {
@@ -1526,6 +1547,7 @@ class HandlerTest extends MediaWikiUnitTestCase {
 			'$requestTypes' => [ 'application/x-www-form-urlencoded' ],
 			'$responseBodySchema' => null,
 			'$routeConfig' => [ 'path' => '/test' ],
+			'$openApiSpec' => [],
 			'$method' => 'POST',
 			'$assertions' =>
 				static function ( array $spec ) {
@@ -1560,6 +1582,7 @@ class HandlerTest extends MediaWikiUnitTestCase {
 			'$requestTypes' => [ 'application/json' ],
 			'$responseBodySchema' => null,
 			'$routeConfig' => [ 'path' => '/test' ],
+			'$openApiSpec' => [],
 			'$method' => 'GET',
 			'$assertions' =>
 				static function ( array $spec ) {
@@ -1582,6 +1605,7 @@ class HandlerTest extends MediaWikiUnitTestCase {
 			'$requestTypes' => [ 'application/json' ],
 			'$responseBodySchema' => null,
 			'$routeConfig' => [ 'path' => '/test' ],
+			'$openApiSpec' => [],
 			'$method' => 'DELETE',
 			'$assertions' =>
 				static function ( array $spec ) {
@@ -1608,6 +1632,7 @@ class HandlerTest extends MediaWikiUnitTestCase {
 			'$requestTypes' => [ 'application/json' ],
 			'$responseBodySchema' => null,
 			'$routeConfig' => [ 'path' => '/test/{p}' ],
+			'$openApiSpec' => [],
 			'$method' => 'GET',
 			'$assertions' =>
 				static function ( array $spec ) {
@@ -1664,6 +1689,7 @@ class HandlerTest extends MediaWikiUnitTestCase {
 				]
 			],
 			'$routeConfig' => [ 'path' => 'test' ],
+			'$openApiSpec' => [],
 			'$method' => 'GET',
 			'$assertions' =>
 				static function ( array $spec ) {
@@ -1705,17 +1731,17 @@ class HandlerTest extends MediaWikiUnitTestCase {
 			'$responseBodySchema' => null,
 			'$routeConfig' => [
 				'path' => 'test/{p}',
-				'OAS' => [
-					'title' => 'just a test',
-					'parameters' => 'will be ignored',
-				]
+			],
+			'$openApiSpec' => [
+				'summary' => 'just a test',
+				'parameters' => 'will be ignored',
 			],
 			'$method' => 'GET',
 			'$assertions' =>
 				static function ( array $spec ) {
-					self::assertWellFormedOAS( $spec, [ 'title', 'parameters' ] );
-					Assert::assertArrayHasKey( 'title', $spec );
-					Assert::assertSame( 'just a test', $spec['title'] );
+					self::assertWellFormedOAS( $spec, [ 'summary', 'parameters' ] );
+					Assert::assertArrayHasKey( 'summary', $spec );
+					Assert::assertSame( 'just a test', $spec['summary'] );
 
 					$params = self::makeMap( $spec['parameters'], 'name' );
 					Assert::assertArrayHasKey( 'p', $params );
@@ -1732,6 +1758,7 @@ class HandlerTest extends MediaWikiUnitTestCase {
 		$requestTypes,
 		$responseBodySchema,
 		$routeConfig,
+		$openApiSpec,
 		$method,
 		$assertions
 	) {
@@ -1747,11 +1774,13 @@ class HandlerTest extends MediaWikiUnitTestCase {
 		$handler->method( 'getResponseBodySchema' )->willReturn( $responseBodySchema );
 
 		// The "body" parameter should be processed as "body", not as "parameter".
-		$module = $this->createNoOpMock( Module::class );
+		$module = $this->createNoOpMock( Module::class, [ 'getModuleDescription' ] );
+		$module->method( 'getModuleDescription' )->willReturn( [] );
 		$handler->initContext(
 			$module,
 			$routeConfig['path'],
-			$routeConfig
+			$routeConfig,
+			$openApiSpec
 		);
 
 		// Because the dummy text formatter uses MessageValue::dump(), translated message keys
