@@ -1,6 +1,14 @@
 <?php
 
+namespace MediaWiki\Tests\Parser;
+
+use DummyContentForTesting;
 use MediaWiki\Parser\ParserObserver;
+use MediaWiki\Parser\ParserOptions;
+use MediaWiki\Parser\ParserOutput;
+use MediaWiki\Title\Title;
+use MediaWikiUnitTestCase;
+use TestLogger;
 
 /**
  * @covers \MediaWiki\Parser\ParserObserver
@@ -25,15 +33,17 @@ class ParserObserverTest extends MediaWikiUnitTestCase {
 		$options = $this->createNoOpMock( ParserOptions::class, [ 'optionsHash' ] );
 		$options->method( 'optionsHash' )->willReturnOnConsecutiveCalls( $hashOne, $hashTwo );
 
+		$content = new DummyContentForTesting( "hello world" );
+
 		$output = new ParserOutput();
 		$observer = new ParserObserver( $logger );
-		$observer->notifyParse( $title, null, $options, $output );
-		$observer->notifyParse( $title, null, $options, $output );
+		$observer->notifyParse( $title, null, $options, $content, $output );
+		$observer->notifyParse( $title, null, $options, $content, $output );
 
 		$this->assertArrayEquals( $expects, $logger->getBuffer() );
 	}
 
-	public function provideDuplicateParse() {
+	public static function provideDuplicateParse() {
 		yield [
 			'foo',
 			'bar',
